unit gmLayers;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF}
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} BrowserTools,
  {$IFDEF USE_ANONYMOUS_METHODS}AnonymousMethodsTools,{$ENDIF} JScriptObjects,
  DispObject, gmBase, gmMVC, gmMap, gmEvents, gmAdditionalConstants,
  gmAdditionalMessages;

type
  TBicyclingLayer = class;
  TFusionTablesCell = class;
  TFusionTablesHeatMap = class;
  TFusionTablesLayer = class;
  TFusionTablesLayerOptions = class;
  TFusionTablesMarkerOptions = class;
  TFusionTablesMouseEvent = class;
  TFusionTablesPolygonOptions = class;
  TFusionTablesPolylineOptions = class;
  TFusionTablesQuery = class;
  TFusionTablesStyle = class;
  TFusionTablesStyleArray = class;
  TKmlAuthor = class;
  TKmlFeatureData = class;
  TKmlLayer = class;
  TKmlLayerOptions = class;
  TKmlLayerMetadata = class;
  TKmlMouseEvent = class;
  TTrafficLayer = class;

  TLayer = class(TMVCObject);

  //A layer showing bike lanes and paths. This class extends MVCObject.
  TBicyclingLayer = class(TLayer)
  private
    FMap: TMap;
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Gibt die Ebene auf der angegebenen Karte wieder. Wenn fr die Karte der
    //Wert "null" festgelegt wurde, wird die Ebene entfernt:
    //Renders the layer on the specified map. If map is set to nil, the layer
    //will be removed:
    procedure SetMap(Map: TMap);
    //Gibt die Karte zurck, auf der diese Ebene angezeigt wird:
    //Returns the map on which this layer is displayed:
    function GetMap: TMap;
    //Additional Delphi property:
    property Map: TMap read GetMap write SetMap;
  end;

  //Describes a single cell from a Fusion Tables table.
  TFusionTablesCell = class(TCustomJScriptObject)
  private
    procedure SetColumnName(Value: String);
    procedure SetValue(Value: String);
    function GetColumnName: String;
    function GetValue: String;
  public
    //Der Name der Spalte, in der sich die Zelle befand:
    //The name of the column in which the cell was located:
    property ColumnName: String read GetColumnName write SetColumnName;
    //Der Inhalt der Zelle:
    //The contents of the cell:
    property Value: String read GetValue write SetValue;
  end;

  {$IFDEF USE_ANONYMOUS_METHODS}
  TFusionTablesLayerClickEvent = TProc<TObject,TFusionTablesMouseEvent>;
  {$ELSE}
  TFusionTablesLayerClickEvent = procedure(Sender: TObject; MouseEvent: TFusionTablesMouseEvent) of object;
  {$ENDIF}

  //Specifies the appearance for a FusionTablesLayer when rendered as a heatmap.
  TFusionTablesHeatMap = class(TCustomJScriptObject)
  private
    procedure SetEnabled(Value: Boolean);
    function GetEnabled: Boolean;
  public
    //If true, render the layer as a heatmap:
    property Enabled: Boolean read GetEnabled write SetEnabled;
  end;

  //A FusionTablesLayer allows you to display data from a Google Fusion Table on
  //a map, as a rendered layer. (See http://code.google.com/apis/fusiontables for
  //more information about Fusion Tables). Table data can be queried using the
  //same query language as is used in the Fusion Tables API.
  //This class extends MVCObject.
  TFusionTablesLayer = class(TLayer)
  private
    FMap: TMap;
    FOnClick: TFusionTablesLayerClickEvent;
    FOnClickFunction: TJScriptFunction;
    FOnClickListener: TMapsEventListener;
    procedure SetOnClick(Value: TFusionTablesLayerClickEvent);
  protected
    class function New(Script: TCustomScript; Opts: TFusionTablesLayerOptions): IDispatch; reintroduce;
    //von JScript aufgerufene Event-Handler:
    //JavaScript event handler:
    procedure Click;
  public
    constructor Create(Opts: TFusionTablesLayerOptions); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TFusionTablesLayerOptions); reintroduce; overload;
    destructor Destroy; override;
    //Gibt die Ebene auf der angegebenen Karte wieder. Wenn fr die Karte der
    //Wert nil festgelegt wurde, wird die Ebene entfernt:
    //Renders the layer on the specified map. If map is set to nil, the layer
    //will be removed:
    procedure SetMap(Map: TMap);
    procedure SetOptions(Options: TFusionTablesLayerOptions); reintroduce;
    //Legt fest, dass die Abfrage bei der Tabelle ausgefhrt werden soll, die
    //durch die tableId-Eigenschaft angegeben wurde. Die Ebene wird mit den
    //Ergebnissen der Abfrage aktualisiert:
    procedure SetQuery(Query: String); {$IFDEF USE_DEPRECATED}deprecated;{$ENDIF}
    //Legt die ID der Tabelle fest, aus der Daten abgerufen werden sollen. Durch
    //das Festlegen dieses Wertes wird die Ebene neu gezeichnet:
    procedure SetTableID(TableID: Integer); {$IFDEF USE_DEPRECATED}deprecated;{$ENDIF}
    //Gibt die Karte zurck, auf der diese Ebene angezeigt wird:
    //Returns the map on which this layer is displayed:
    function GetMap: TMap;
    function GetQuery: String; {$IFDEF USE_DEPRECATED}deprecated;{$ENDIF}
    //Gibt die ID der Tabelle zurck, aus der Daten abgerufen werden sollen.
    function GetTableID: Integer; {$IFDEF USE_DEPRECATED}deprecated;{$ENDIF}
    //zustzliche Eigenschaften:
    //additional Delphi properties:
    property Map: TMap read GetMap write SetMap;
    //property Query: String read GetQuery write SetQuery; deprecated;
    //property TableID: Integer read GetTableID write SetTableID; deprecated;
    //Dieses Ereignis wird ausgelst, wenn eine Funktion auf der Ebene angeklickt wird:
    //This event is fired when a feature in the layer is clicked:
    property OnClick: TFusionTablesLayerClickEvent read FOnClick write SetOnClick;
  end;

  //This object defines the properties that can be set on a FusionTablesLayer
  //object.
  TFusionTablesLayerOptions  = class(TCustomJScriptObject)
  private
    FHeatmap: TFusionTablesHeatmap;
    FMap: TMap;
    FQuery: TFusionTablesQuery;
    FStyles: TFusionTablesStyleArray;
    procedure SetClickable(Value: Boolean);
    procedure SetHeatmap(Value: {Boolean deprecated} TFusionTablesHeatmap);
    procedure SetMap(Value: TMap);
    procedure SetQuery(Value: {String deprecated} TFusionTablesQuery);
    procedure SetStyles(Value: TFusionTablesStyleArray);
    procedure SetSuppressInfoWindows(Value: Boolean);
    function GetClickable: Boolean;
    function GetHeatmap: {Boolean deprecated} TFusionTablesHeatmap;
    function GetMap: TMap;
    function GetQuery: {String deprecated} TFusionTablesQuery;
    function GetStyles: TFusionTablesStyleArray;
    function GetSuppressInfoWindows: Boolean;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //If true, the layer receives mouse events. Default value is true:
    property Clickable: Boolean read GetClickable write SetClickable;
    //Standardmig werden die Tabellendaten als geografische Merkmale dargestellt.
    //Falls wahr, wird auf der Ebene eine Wrmekarte angezeigt, auf der die Dichte
    //der geografischen Merkmale dargestellt wird, die bei der Abfrage von der
    //ausgewhlten Tabelle zurckgegeben wurden. Der Typ dieser Eigenschaft wurde
    //von Boolean in TFusionTablesHeatmap gendert (siehe engl. Dokumentation):
    //Options which define the appearance of the layer as a heatmap:
    property Heatmap: {Boolean deprecated} TFusionTablesHeatmap read GetHeatmap write SetHeatmap;
    //Die Karte, auf der die Ebene angezeigt wird:
    //The map on which to display the layer:
    property Map: TMap read GetMap write SetMap;
    //Fusion Tables-Abfrage, die zur Auswahl der anzuzeigenden Daten angewendet
    //werden soll. Die Abfragen sollten nicht mit URL-Encoding codiert sein.
    //Der Typ dieser Eigenschaft wurde von String in TFusionTablesQuery gendert
    //(siehe engl. Dokumentation):
    //Options defining the data to display:
    property Query: {String deprecated} TFusionTablesQuery read GetQuery write SetQuery;
    //An array of up to 5 style specifications, which control the appearance of
    //features within the layer (Array.<FusionTablesStyle>):
    property Styles: TFusionTablesStyleArray read GetStyles write SetStyles;
    //Unterdrckt die Wiedergabe von Infofenstern, wenn Ebenenmerkmale angeklickt
    //werden:
    //Suppress the rendering of info windows when layer features are clicked:
    property SuppressInfoWindows: Boolean read GetSuppressInfoWindows write SetSuppressInfoWindows;
  end;

  //Options which control the appearance of point features in a FusionTablesLayer.
  TFusionTablesMarkerOptions = class(TCustomJScriptObject)
  private
    procedure SetIconName(Value: String);
    function GetIconName: String;
  public
    //The name of a Fusion Tables supported icon:
    property IconName: String read GetIconName write SetIconName;
  end;

  //The properties of a mouse event on a FusionTablesLayer.
  TFusionTablesMouseEvent = class(TCustomJScriptObject)
  private
    FLatLng: TLatLng;
    FPixelOffset: TSize;
    FRow: TCustomJScriptObject;
    procedure SetInfoWindowHtml(Value: String);
    procedure SetLatLng(Value: TLatLng);
    procedure SetPixelOffset(Value: TSize);
    procedure SetRow(Value: TCustomJScriptObject);
    function GetInfoWindowHtml: String;
    function GetLatLng: TLatLng;
    function GetPixelOffset: TSize;
    function GetRow: TCustomJScriptObject;
  public
    destructor Destroy; override;
    //Vorgefertigter HTML-Inhalt, der von der Standard-UI im Infofenster platziert wird:
    //Pre-rendered HTML content, as placed in the infowindow by the default UI:
    property InfoWindowHtml: String read GetInfoWindowHtml write SetInfoWindowHtml;
    //Die Position, an der ein Infofenster auf dem angeklickten Merkmal verankert werden soll:
    //The position at which to anchor an infowindow on the clicked feature:
    property LatLng: TLatLng read GetLatLng write SetLatLng;
    property Position: TLatLng read GetLatLng write SetLatLng; //deprecated
    //Der Abstand, der zwischen einem Infofenster und seiner Verankerung auf dem
    //angeklickten Merkmal bestehen soll:
    //The offset to apply to an infowindow anchored on the clicked feature:
    property PixelOffset: TSize read GetPixelOffset write SetPixelOffset;
    //Eine Sammlung von FusionTablesCell-Objekten, indiziert nach Spaltenname, die die
    //Inhalte der Tabellenzeile darstellen, in der das angeklickte Merkmal enthalten war:
    //A collection of FusionTablesCell objects, indexed by column name,
    //representing the contents of the table row which included the clicked feature:
    property Row: TCustomJScriptObject read GetRow write SetRow;
  end;

  //Options which control the appearance of polygons in a FusionTablesLayer.
  TFusionTablesPolygonOptions = class(TCustomJScriptObject)
  private
    procedure SetFillColor(Value: String);
    procedure SetFillOpacity(Value: Double);
    procedure SetStrokeColor(Value: String);
    procedure SetStrokeOpacity(Value: Double);
    procedure SetStrokeWeight(Value: Integer);
    function GetFillColor: String;
    function GetFillOpacity: Double;
    function GetStrokeColor: String;
    function GetStrokeOpacity: Double;
    function GetStrokeWeight: Integer;
  public
    //The fill color, defined by a six-digit hexadecimal number in RRGGBB format
    //(e.g. #00AAFF):
    property FillColor: String read GetFillColor write SetFillColor;
    //The fill opacity between 0.0 and 1.0:
    property FillOpacity: Double read GetFillOpacity write SetFillOpacity;
    //The fill color, defined by a six-digit hexadecimal number in RRGGBB format
    //(e.g. #00AAFF):
    property StrokeColor: String read GetStrokeColor write SetStrokeColor;
    //The stroke opacity between 0.0 and 1.0:
    property StrokeOpacity: Double read GetStrokeOpacity write SetStrokeOpacity;
    //The stroke width in pixels, between 0 and 10:
    property StrokeWeight: Integer read GetStrokeWeight write SetStrokeWeight;
  end;

  //Options which control the appearance of polylines in a FusionTablesLayer.
  TFusionTablesPolylineOptions = class(TCustomJScriptObject)
  private
    procedure SetStrokeColor(Value: String);
    procedure SetStrokeOpacity(Value: Double);
    procedure SetStrokeWeight(Value: Integer);
    function GetStrokeColor: String;
    function GetStrokeOpacity: Double;
    function GetStrokeWeight: Integer;
  public
    //The fill color, defined by a six-digit hexadecimal number in RRGGBB format
    //(e.g. #00AAFF):
    property StrokeColor: String read GetStrokeColor write SetStrokeColor;
    //The stroke opacity between 0.0 and 1.0:
    property StrokeOpacity: Double read GetStrokeOpacity write SetStrokeOpacity;
    //The stroke width in pixels:
    property StrokeWeight: Integer read GetStrokeWeight write SetStrokeWeight;
  end;

  //Specifies the data to retrieve from a Fusion Tables.
  TFusionTablesQuery = class(TCustomJScriptObject)
  private
    procedure SetFrom(Value: String);
    procedure SetSelect(Value: String);
    procedure SetWhere(Value: String);
    function GetFrom: String;
    function GetSelect: String;
    function GetWhere: String;
  public
    //The ID of the Fusion Tables table to display. This ID can be found in the
    //tables URL, as the value of the dsrcid parameter:
    property From: String read GetFrom write SetFrom;
    //A column, containing geographic features to be displayed on the map. See
    //Fusion Table Setup in the Maps API documentation for information about
    //valid columns:
    property Select: String read GetSelect write SetSelect;
    //The SQL predicate to be applied to the layer:
    property Where: String read GetWhere write SetWhere;
  end;

  //Controls the appearance of a set of features within a FusionTablesLayer.
  //Features which match the provided SQL predicate will be styled with the
  //supplied options.
  TFusionTablesStyle = class(TCustomJScriptObject)
  private
    FMarkerOptions: TFusionTablesMarkerOptions;
    FPolygonOptions: TFusionTablesPolygonOptions;
    FPolylineOptions: TFusionTablesPolylineOptions;
    procedure SetMarkerOptions(Value: TFusionTablesMarkerOptions);
    procedure SetPolygonOptions(Value: TFusionTablesPolygonOptions);
    procedure SetPolylineOptions(Value: TFusionTablesPolylineOptions);
    procedure SetWhere(Value: String);
    function GetMarkerOptions: TFusionTablesMarkerOptions;
    function GetPolygonOptions: TFusionTablesPolygonOptions;
    function GetPolylineOptions: TFusionTablesPolylineOptions;
    function GetWhere: String;
  public
    destructor Destroy; override;
    //Options which control the appearance of point features:
    property MarkerOptions: TFusionTablesMarkerOptions read GetMarkerOptions write SetMarkerOptions;
    //Options which control the appearance of polygons:
    property PolygonOptions: TFusionTablesPolygonOptions read GetPolygonOptions write SetPolygonOptions;
    //Options which control the appearance of polylines:
    property PolylineOptions: TFusionTablesPolylineOptions read GetPolylineOptions write SetPolylineOptions;
    //The SQL predicate to be applied to the layer:
    property Where: String read GetWhere write SetWhere;
  end;

  //JavaScript: Array.<FusionTablesStyle>
  TFusionTablesStyleArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TFusionTablesStyle);
    function GetItem(Index: Integer): TFusionTablesStyle;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    procedure Insert(Index: Integer; Value: TFusionTablesStyle); reintroduce;
    function AddStyle: TFusionTablesStyle;
    function InsertStyle(Index: Integer): TFusionTablesStyle;
    property Items[Index: Integer]: TFusionTablesStyle read GetItem write SetItem; default;
  end;

  //Contains details of the author of a KML document or feature.
  TKmlAuthor = class(TCustomJScriptObject)
  private
    function GetEmail: String;
    function GetName: String;
    function GetURI: String;
  public
    //E-Mail-Adresse des Autors; falls nicht angegeben, eine leere Zeichenfolge:
    //The author's e-mail address, or an empty string if not specified:
    property Email: String read GetEmail;
    //Name des Autors; falls nicht angegeben, eine leere Zeichenfolge:
    //The author's name, or an empty string if not specified:
    property Name: String read GetName;
    //Homepage des Autors; falls nicht angegeben, eine leere Zeichenfolge:
    //The author's home page, or an empty string if not specified:
    property URI: String read GetURI;
  end;

  //Data for a single KML feature in JSON format, returned when a KML feature is
  //clicked. The data contained in this object mirrors that associated with the
  //feature in the KML or GeoRSS markup in which it is declared.
  TKmlFeatureData = class(TCustomJScriptObject)
  private
    FAuthor: TKmlAuthor;
    function GetAuthor: TKmlAuthor;
    function GetDescription: String;
    function GetID: String;
    function GetInfoWindowHTML: String;
    function GetName: String;
    function GetSnippet: String;
  public
    destructor Destroy; override;
    //Autor des Merkmals, extrahiert aus der Ebenenauszeichnung, falls angegeben:
    //The feature's <atom:author>, extracted from the layer markup (if specified):
    property Author: TKmlAuthor read GetAuthor;
    //Beschreibung des Merkmals, extrahiert aus der Ebenen-Auszeichnung:
    //The feature's <description>, extracted from the layer markup:
    property Description: String read GetDescription;
    //ID des Merkmals, extrahiert aus der Ebenen-Auszeichnung. Falls keine ID
    //angegeben wurde, wird eine eindeutige ID fr dieses Merkmal erzeugt:
    //The feature's <id>, extracted from the layer markup. If no <id> has been
    //specified, a unique ID will be generated for this feature:
    property ID: String read GetID;
    //Der Pop-up-Text des Merkmals, falls festgelegt:
    //The feature's balloon styled text, if set:
    property InfoWindowHTML: String read GetInfoWindowHTML;
    //Name des Merkmals, extrahiert aus der Ebenen-Auszeichnung:
    //The feature's <name>, extracted from the layer markup:
    property Name: String read GetName;
    //Snippet des Merkmals, extrahiert aus der Ebenen-Auszeichnung:
    //The feature's <Snippet>, extracted from the layer markup:
    property Snippet: String read GetSnippet;
  end;

  {$IFDEF USE_ANONYMOUS_METHODS}
  TKmlLayerClickEvent = TProc<TObject,TKmlMouseEvent>;
  {$ELSE}
  TKmlLayerClickEvent = procedure(Sender: TObject; KmlMouseEvent: TKmlMouseEvent) of object;
  {$ENDIF}

  //A KmlLayer adds geographic markup to the map from a KML, KMZ or GeoRSS file
  //that is hosted on a publicly accessible web server. A KmlFeatureData object
  //is provided for each feature when clicked. This class extends MVCObject.
  TKmlLayer = class(TLayer)
  private
    FDefaultViewport: TLatLngBounds;
    FMap: TMap;
    FMetadata: TKmlLayerMetadata;
    FOnClick: TKmlLayerClickEvent;
    FOnClickFunction: TJScriptFunction;
    FOnClickListener: TMapsEventListener;
    FOnDefaultViewportChanged: TNotifyEvent;
    FOnDefaultViewportFunction: TJScriptFunction;
    FOnDefaultViewportListener: TMapsEventListener;
    procedure SetOnClick(Value: TKmlLayerClickEvent);
    procedure SetOnDefaultViewportChanged(Value: TNotifyEvent);
  protected
    class function New(Script: TCustomScript; const URL: String): IDispatch; reintroduce; overload;
    class function New(Script: TCustomScript; const URL: String; Opts: TKmlLayerOptions): IDispatch; reintroduce; overload;
    //von JScript aufgerufene Event-Handler:
    //JavaScript event handler:
    procedure Click;
    procedure DefaultViewportChanged;
  public
    constructor Create(const URL: String; Opts: TKmlLayerOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; const URL: String; Opts: TKmlLayerOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    //Gibt die KML-Ebene auf der angegebenen Karte wieder. Falls fr die Karte der
    //Wert nil festgelegt wird, wird die Karte entfernt:
    //Renders the KML Layer on the specified map. If map is set to nil, the layer
    //is removed:
    procedure SetMap(Map: TMap);
    //Get the default viewport for the layer being displayed.
    function GetDefaultViewport: TLatLngBounds;
    //Ruft die Karte ab, auf der die KML-Ebene wiedergegeben wird:
    //Get the map on which the KML Layer is being rendered:
    function GetMap: TMap;
    //Ruft die mit dieser Ebene verknpften Metadaten ab, wie in der
    //Ebenenauszeichnung angegeben:
    //Get the metadata associated with this layer, as specified in the layer markup:
    function GetMetadata: TKmlLayerMetadata;
    //Ruft die URL der angezeigten geografischen Auszeichnung ab:
    //Get the URL of the geographic markup which is being displayed:
    function GetUrl: String;
    //Dieses Ereignis wird ausgelst, wenn eine Funktion auf der Ebene angeklickt wird:
    //This event is fired when a feature in the layer is clicked:
    property OnClick: TKmlLayerClickEvent read FOnClick write SetOnClick;
    //Dieses Ereignis wird ausgelst, wenn sich der standardmige
    //Darstellungsbereich der KML-Ebene gendert hat:
    //This event is fired when the KML layers default viewport has changed:
    property OnDefaultViewportChanged: TNotifyEvent read FOnDefaultViewportChanged write SetOnDefaultViewportChanged;
  end;

  //This object defines the properties that can be set on a KmlLayer object.
  TKmlLayerOptions = class(TCustomJScriptObject)
  private
    FMap: TMap;
    procedure SetClickable(Value: Boolean);
    procedure SetMap(Value: TMap);
    procedure SetPreserveViewport(Value: Boolean);
    procedure SetSuppressInfoWindows(Value: Boolean);
    function GetClickable: Boolean;
    function GetMap: TMap;
    function GetPreserveViewport: Boolean;
    function GetSuppressInfoWindows: Boolean;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //If true, the layer receives mouse events. Default value is true:
    property Clickable: Boolean read GetClickable write SetClickable;
    //Die Karte, auf der die Ebene angezeigt wird:
    //The map on which to display the layer:
    property Map: TMap read GetMap write SetMap;
    //Standardmig wird die Eingabekarte bezglich des Markierungsrahmens der
    //Ebeneninhalte zentriert und gezoomt. Wenn fr diese Option der Wert true
    //festgelegt wird, bleibt der Darstellungsbereich unverndert, es sei denn,
    //der Zoomwert und der Mittelpunkt der Karte wurden nie festgelegt:
    //By default, the input map is centered and zoomed to the bounding box of the
    //contents of the layer. If this option is set to true, the viewport is left
    //unchanged, unless the map's center and zoom were never set:
    property PreserveViewport: Boolean read GetPreserveViewport write SetPreserveViewport;
    //Unterdrckt die Wiedergabe von Infofenstern, wenn Ebenenmerkmale angeklickt
    //werden:
    //Suppress the rendering of info windows when layer features are clicked:
    property SuppressInfoWindows: Boolean read GetSuppressInfoWindows write SetSuppressInfoWindows;
  end;

  //Metadata for a single KML layer, in JSON format.
  TKmlLayerMetadata = class(TCustomJScriptObject)
  private
    FAuthor: TKmlAuthor;
    function GetAuthor: TKmlAuthor;
    function GetDescription: String;
    function GetName: String;
    function GetSnippet: String;
  public
    destructor Destroy; override;
    //Author der Ebene, extrahiert aus der Ebenen-Auszeichnung:
    //The layer's <atom:author>, extracted from the layer markup:
    property Author: TKmlAuthor read GetAuthor;
    //Description der Ebene, extrahiert aus der Ebenen-Auszeichnung:
    //The layer's <atom:author>, extracted from the layer markup:
    property Description: String read GetDescription;
    //Name der Ebene, extrahiert aus der Ebenen-Auszeichnung:
    //The layer's <name>, extracted from the layer markup:
    property Name: String read GetName;
    //Snippet der Ebene, extrahiert aus der Ebenen-Auszeichnung:
    //The layer's <Snippet>, extracted from the layer markup:
    property Snippet: String read GetSnippet;
  end;

  //The properties of a click event on a KML/KMZ or GeoRSS document.
  TKmlMouseEvent = class(TCustomJScriptObject)
  private
    FFeatureData: TKmlFeatureData;
    FLatLng: TLatLng;
    FPixelOffset: TSize;
    function GetFeatureData: TKmlFeatureData;
    function GetLatLng: TLatLng;
    function GetPixelOffset: TSize;
  public
    destructor Destroy; override;
    //Ein KmlFeatureData-Objekt, das Informationen ber das angeklickte Merkmal enthlt:
    //A KmlFeatureData object, containing information about the clicked feature:
    property FeatureData: TKmlFeatureData read GetFeatureData;
    //Der Abstand, der zwischen einem Infofenster und seiner Verankerung auf dem
    //angeklickten Merkmal bestehen soll:
    //The offset to apply to an infowindow anchored on the clicked feature:
    property PixelOffset: TSize read GetPixelOffset;
    //Die Position, an der ein Infofenster auf dem angeklickten Merkmal verankert werden soll:
    //The position at which to anchor an infowindow on the clicked feature:
    property LatLng: TLatLng read GetLatLng;
    property Position: TLatLng read GetLatLng; //deprecated
  end;

  //Eine Ebene, auf der der aktuelle Straenverkehr angezeigt wird.
  //A traffic layer. This class extends MVCObject.
  TTrafficLayer = class(TLayer)
  private
    FMap: TMap;
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //Gibt die Ebene auf der angegebenen Karte wieder. Wenn fr die Karte der
    //Wert nil festgelegt wurde, wird die Ebene entfernt:
    //Renders the layer on the specified map. If map is set to nil, the layer
    //will be removed:
    procedure SetMap(Map: TMap);
    //Gibt die Karte zurck, auf der diese Ebene angezeigt wird:
    //Returns the map on which this layer is displayed:
    function GetMap: TMap;
    //zustzliche Eigenschaft:
    //additional Delphi property:
    property Map: TMap read GetMap write SetMap;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TBicyclingLayer

constructor TBicyclingLayer.Create;
begin
  inherited;
end;

constructor TBicyclingLayer.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TBicyclingLayer.Destroy;
begin
  FMap.Free;
  inherited;
end;

class function TBicyclingLayer.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[BicyclingLayerMethod_GetMap,
                                 BicyclingLayerMethod_SetMap]);
end;

class function TBicyclingLayer.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_BicyclingLayer,true);
end;

function TBicyclingLayer.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(BicyclingLayerMethod_GetMap);
  if VarIsValidDispatch(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

procedure TBicyclingLayer.SetMap(Map: TMap);
begin
  Exec(BicyclingLayerMethod_SetMap,Map);
end;

//------------------------------------------------------------------------------
//TFusionTablesCell

function TFusionTablesCell.GetColumnName: String;
begin
  Result:=GetPropAsString(FusionTablesCellProp_ColumnName);
end;

procedure TFusionTablesCell.SetColumnName(Value: String);
begin
  SetProp(FusionTablesCellProp_ColumnName,Value,true);
end;

function TFusionTablesCell.GetValue: String;
begin
  Result:=GetPropAsString(FusionTablesCellProp_Value);
end;

procedure TFusionTablesCell.SetValue(Value: String);
begin
  SetProp(FusionTablesCellProp_Value,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesHeatMap

function TFusionTablesHeatMap.GetEnabled: Boolean;
begin
  Result:=GetPropAsBoolean(FusionTablesHeatMapProp_Enabled);
end;

procedure TFusionTablesHeatMap.SetEnabled(Value: Boolean);
begin
  SetProp(FusionTablesHeatMapProp_Enabled,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesLayer

constructor TFusionTablesLayer.Create(Opts: TFusionTablesLayerOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TFusionTablesLayer.Create(Script: TCustomScript; Opts: TFusionTablesLayerOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TFusionTablesLayer.Destroy;
begin
  SetOnClick(nil);
  FOnClickFunction.Free;
  FMap.Free;
  inherited;
end;

class function TFusionTablesLayer.New(Script: TCustomScript; Opts: TFusionTablesLayerOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then raise Exception.Create(ExceptionNoOpts);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_FusionTablesLayer,Opts.Disp,true);
end;

procedure TFusionTablesLayer.Click;
var
  EventObject: TFusionTablesMouseEvent;
begin
  if assigned(FOnClick) then
  begin
    try
      EventObject:=TFusionTablesMouseEvent(CreateObjectWrapper(Script,
                                                               TFusionTablesMouseEvent,
                                                               FOnClickFunction.Arguments[0]));
      try
        FOnClick(self,EventObject);
      finally
        //EventObject.Free;
      end;
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
end;

procedure TFusionTablesLayer.SetOnClick(Value: TFusionTablesLayerClickEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(Event_Click,FOnClickListener,FOnClickFunction,Click);
end;

function TFusionTablesLayer.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(FusionTablesLayerMethod_GetMap,TMap,FMap));
end;

procedure TFusionTablesLayer.SetMap(Map: TMap);
begin
  Exec(FusionTablesLayerMethod_SetMap,Map);
end;

procedure TFusionTablesLayer.SetOptions(Options: TFusionTablesLayerOptions);
begin
  Exec(FusionTablesLayerMethod_SetOptions,Options);
end;

function TFusionTablesLayer.GetQuery: String;
begin
  Result:=VarAsString(Exec(FusionTablesLayerMethod_GetQuery));
end;

procedure TFusionTablesLayer.SetQuery(Query: String);
begin
  Exec(FusionTablesLayerMethod_SetQuery,[Query]);
end;

function TFusionTablesLayer.GetTableID: Integer;
begin
  Result:=VarAsInteger(Exec(FusionTablesLayerMethod_GetTableID));
end;

procedure TFusionTablesLayer.SetTableID(TableID: Integer);
begin
  Exec(FusionTablesLayerMethod_SetTableID,[TableID]);
end;

//------------------------------------------------------------------------------
//TFusionTablesStyleArray.

function TFusionTablesStyleArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TFusionTablesStyle,Dispatch);
end;

function TFusionTablesStyleArray.GetItem(Index: Integer): TFusionTablesStyle;
begin
  Result:=TFusionTablesStyle(inherited Items[Index]);
end;

procedure TFusionTablesStyleArray.SetItem(Index: Integer; Value: TFusionTablesStyle);
begin
  inherited Items[Index]:=Value;
end;

procedure TFusionTablesStyleArray.Insert(Index: Integer; Value: TFusionTablesStyle);
begin
  inherited Insert(Index,Value);
end;

function TFusionTablesStyleArray.AddStyle: TFusionTablesStyle;
begin
  Result:=InsertStyle(Count);                              //add new TFusionTablesStyle object
end;

function TFusionTablesStyleArray.InsertStyle(Index: Integer): TFusionTablesStyle;
begin
  Result:=TFusionTablesStyle.Create(Script);
  Insert(Index,Result);
end;

//------------------------------------------------------------------------------
//TFusionTablesLayerOptions

constructor TFusionTablesLayerOptions.Create;
begin
  inherited;
end;

constructor TFusionTablesLayerOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TFusionTablesLayerOptions.Destroy;
begin
  FHeatmap.Free;
  FMap.Free;
  FQuery.Free;
  FStyles.Free;
  inherited;
end;

function TFusionTablesLayerOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(FusionTablesLayerOptionsProp_Clickable);
end;

procedure TFusionTablesLayerOptions.SetClickable(Value: Boolean);
begin
  SetProp(FusionTablesLayerOptionsProp_Clickable,Value,true);
end;

function TFusionTablesLayerOptions.GetHeatmap: TFusionTablesHeatmap; {Boolean deprecated}
begin
  Result:=TFusionTablesHeatmap(GetPropAsObject(FusionTablesLayerOptionsProp_Heatmap,
                                               TFusionTablesHeatmap,FHeatmap,true));
  //Result:=GetPropAsBoolean(FusionTablesLayerOptionsProp_Heatmap); deprecated
end;

procedure TFusionTablesLayerOptions.SetHeatmap(Value: TFusionTablesHeatmap); {Boolean deprecated}
begin
  SetProp(FusionTablesLayerOptionsProp_Heatmap,Value,true);
end;

function TFusionTablesLayerOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(FusionTablesLayerOptionsProp_Map,TMap,FMap));
end;

procedure TFusionTablesLayerOptions.SetMap(Value: TMap);
begin
  SetProp(FusionTablesLayerOptionsProp_Map,Value,true);
end;

function TFusionTablesLayerOptions.GetQuery: {String deprecated} TFusionTablesQuery;
begin
  Result:=TFusionTablesQuery(GetPropAsObject(FusionTablesLayerOptionsProp_Query,
                                             TFusionTablesQuery,FQuery,true));
  //Result:=GetPropAsString(FusionTablesLayerOptionsProp_Query); deprecated
end;

procedure TFusionTablesLayerOptions.SetQuery(Value: {String deprecated} TFusionTablesQuery);
begin
  SetProp(FusionTablesLayerOptionsProp_Query,Value,true);
end;

function TFusionTablesLayerOptions.GetStyles: TFusionTablesStyleArray;
begin
  Result:=TFusionTablesStyleArray(GetPropAsObject(FusionTablesLayerOptionsProp_Styles,
                                                  TFusionTablesStyleArray,
                                                  FStyles,true));
end;

procedure TFusionTablesLayerOptions.SetStyles(Value: TFusionTablesStyleArray);
begin
  SetProp(FusionTablesLayerOptionsProp_Styles,Value,true);
end;

function TFusionTablesLayerOptions.GetSuppressInfoWindows: Boolean;
begin
  Result:=GetPropAsBoolean(FusionTablesLayerOptionsProp_SuppressInfoWindows);
end;

procedure TFusionTablesLayerOptions.SetSuppressInfoWindows(Value: Boolean);
begin
  SetProp(FusionTablesLayerOptionsProp_SuppressInfoWindows,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesMarkerOptions

function TFusionTablesMarkerOptions.GetIconName: String;
begin
  Result:=GetPropAsString(FusionTablesMarkerOptionsProp_IconName);
end;

procedure TFusionTablesMarkerOptions.SetIconName(Value: String);
begin
  SetProp(FusionTablesMarkerOptionsProp_IconName,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesMouseEvent

destructor TFusionTablesMouseEvent.Destroy;
begin
  FLatLng.Free;
  FPixelOffset.Free;
  FRow.Free;
  inherited;
end;

function TFusionTablesMouseEvent.GetInfoWindowHTML: String;
begin
  Result:=GetPropAsString(FusionTablesMouseEventProp_InfoWindowHTML);
end;

procedure TFusionTablesMouseEvent.SetInfoWindowHtml(Value: String);
begin
   SetProp(FusionTablesMouseEventProp_InfoWindowHTML,Value,true);
end;

function TFusionTablesMouseEvent.GetPixelOffset: TSize;
begin
  Result:=TSize(GetPropAsObject(FusionTablesMouseEventProp_PixelOffset,
                                TSize,FPixelOffset));
end;

procedure TFusionTablesMouseEvent.SetPixelOffset(Value: TSize);
begin
   SetProp(FusionTablesMouseEventProp_PixelOffset,Value,true);
end;

function TFusionTablesMouseEvent.GetLatLng: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(FusionTablesMouseEventProp_LatLng,
                                  TLatLng,FLatLng));
end;

procedure TFusionTablesMouseEvent.SetLatLng(Value: TLatLng);
begin
   SetProp(FusionTablesMouseEventProp_LatLng,Value,true);
end;

function TFusionTablesMouseEvent.GetRow: TCustomJScriptObject;
begin
  Result:=TCustomJScriptObject(GetPropAsObject(FusionTablesMouseEventProp_Row,
                                         TCustomJScriptObject,FRow));
end;

procedure TFusionTablesMouseEvent.SetRow(Value: TCustomJScriptObject);
begin
   SetProp(FusionTablesMouseEventProp_Row,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesPolygonOptions

function TFusionTablesPolygonOptions.GetFillColor: String;
begin
  Result:=GetPropAsString(FusionTablesPolygonOptionsProp_FillColor);
end;

procedure TFusionTablesPolygonOptions.SetFillColor(Value: String);
begin
   SetProp(FusionTablesPolygonOptionsProp_FillColor,Value,true);
end;

function TFusionTablesPolygonOptions.GetFillOpacity: Double;
begin
  Result:=GetPropAsDouble(FusionTablesPolygonOptionsProp_FillOpacity);
end;

procedure TFusionTablesPolygonOptions.SetFillOpacity(Value: Double);
begin
   SetProp(FusionTablesPolygonOptionsProp_FillOpacity,Value,true);
end;

function TFusionTablesPolygonOptions.GetStrokeColor: String;
begin
  Result:=GetPropAsString(FusionTablesPolygonOptionsProp_StrokeColor);
end;

procedure TFusionTablesPolygonOptions.SetStrokeColor(Value: String);
begin
   SetProp(FusionTablesPolygonOptionsProp_StrokeColor,Value,true);
end;

function TFusionTablesPolygonOptions.GetStrokeOpacity: Double;
begin
  Result:=GetPropAsDouble(FusionTablesPolygonOptionsProp_StrokeOpacity);
end;

procedure TFusionTablesPolygonOptions.SetStrokeOpacity(Value: Double);
begin
   SetProp(FusionTablesPolygonOptionsProp_StrokeOpacity,Value,true);
end;

function TFusionTablesPolygonOptions.GetStrokeWeight: Integer;
begin
  Result:=GetPropAsInteger(FusionTablesPolygonOptionsProp_StrokeWeight);
end;

procedure TFusionTablesPolygonOptions.SetStrokeWeight(Value: Integer);
begin
   SetProp(FusionTablesPolygonOptionsProp_StrokeWeight,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesPolylineOptions

function TFusionTablesPolylineOptions.GetStrokeColor: String;
begin
  Result:=GetPropAsString(FusionTablesPolylineOptionsProp_StrokeColor);
end;

procedure TFusionTablesPolylineOptions.SetStrokeColor(Value: String);
begin
   SetProp(FusionTablesPolylineOptionsProp_StrokeColor,Value,true);
end;

function TFusionTablesPolylineOptions.GetStrokeOpacity: Double;
begin
  Result:=GetPropAsDouble(FusionTablesPolylineOptionsProp_StrokeOpacity);
end;

procedure TFusionTablesPolylineOptions.SetStrokeOpacity(Value: Double);
begin
   SetProp(FusionTablesPolylineOptionsProp_StrokeOpacity,Value,true);
end;

function TFusionTablesPolylineOptions.GetStrokeWeight: Integer;
begin
  Result:=GetPropAsInteger(FusionTablesPolylineOptionsProp_StrokeWeight);
end;

procedure TFusionTablesPolylineOptions.SetStrokeWeight(Value: Integer);
begin
   SetProp(FusionTablesPolylineOptionsProp_StrokeWeight,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesQuery

function TFusionTablesQuery.GetFrom: String;
begin
  Result:=GetPropAsString(FusionTablesQueryProp_From);
end;

procedure TFusionTablesQuery.SetFrom(Value: String);
begin
   SetProp(FusionTablesQueryProp_From,Value,true);
end;

function TFusionTablesQuery.GetSelect: String;
begin
  Result:=GetPropAsString(FusionTablesQueryProp_Select);
end;

procedure TFusionTablesQuery.SetSelect(Value: String);
begin
   SetProp(FusionTablesQueryProp_Select,Value,true);
end;

function TFusionTablesQuery.GetWhere: String;
begin
  Result:=GetPropAsString(FusionTablesQueryProp_Where);
end;

procedure TFusionTablesQuery.SetWhere(Value: String);
begin
   SetProp(FusionTablesQueryProp_Where,Value,true);
end;

//------------------------------------------------------------------------------
//TFusionTablesStyle

destructor TFusionTablesStyle.Destroy;
begin
  FMarkerOptions.Free;
  FPolygonOptions.Free;
  FPolylineOptions.Free;
  inherited;
end;

function TFusionTablesStyle.GetMarkerOptions: TFusionTablesMarkerOptions;
begin
  Result:=TFusionTablesMarkerOptions(GetPropAsObject(FusionTablesStyleProp_MarkerOptions,
                                                     TFusionTablesMarkerOptions,
                                                     FMarkerOptions,true));
end;

procedure TFusionTablesStyle.SetMarkerOptions(Value: TFusionTablesMarkerOptions);
begin
   SetProp(FusionTablesStyleProp_MarkerOptions,Value,true);
end;

function TFusionTablesStyle.GetPolygonOptions: TFusionTablesPolygonOptions;
begin
  Result:=TFusionTablesPolygonOptions(GetPropAsObject(FusionTablesStyleProp_PolygonOptions,
                                                     TFusionTablesPolygonOptions,
                                                     FPolygonOptions,true));
end;

procedure TFusionTablesStyle.SetPolygonOptions(Value: TFusionTablesPolygonOptions);
begin
   SetProp(FusionTablesStyleProp_PolygonOptions,Value,true);
end;

function TFusionTablesStyle.GetPolylineOptions: TFusionTablesPolylineOptions;
begin
  Result:=TFusionTablesPolylineOptions(GetPropAsObject(FusionTablesStyleProp_PolylineOptions,
                                                       TFusionTablesPolylineOptions,
                                                       FPolylineOptions,true));
end;

procedure TFusionTablesStyle.SetPolylineOptions(Value: TFusionTablesPolylineOptions);
begin
   SetProp(FusionTablesStyleProp_PolylineOptions,Value,true);
end;

function TFusionTablesStyle.GetWhere: String;
begin
  Result:=GetPropAsString(FusionTablesStyleProp_Where);
end;

procedure TFusionTablesStyle.SetWhere(Value: String);
begin
   SetProp(FusionTablesStyleProp_Where,Value,true);
end;

//------------------------------------------------------------------------------
//TKmlAuthor

function TKmlAuthor.GetEmail: String;
begin
  Result:=GetPropAsString(KmlAuthorProp_Email);
end;

function TKmlAuthor.GetName: String;
begin
  Result:=GetPropAsString(KmlAuthorProp_Name);
end;

function TKmlAuthor.GetURI: String;
begin
  Result:=GetPropAsString(KmlAuthorProp_URI);
end;

//------------------------------------------------------------------------------
//TKmlLayer

constructor TKmlLayer.Create(const URL: String; Opts: TKmlLayerOptions);
begin
  Create(JScriptObjects.Script,URL,Opts);
end;

constructor TKmlLayer.Create(Script: TCustomScript; const URL: String; Opts: TKmlLayerOptions);
begin
  inherited Create(Script,New(Script,URL,Opts));
end;

destructor TKmlLayer.Destroy;
begin
  SetOnClick(nil);
  SetOnDefaultViewportChanged(nil);
  FOnClickFunction.Free;
  FOnDefaultViewportFunction.Free;
  FDefaultViewport.Free;
  FMap.Free;
  FMetadata.Free;
  inherited;
end;

class function TKmlLayer.New(Script: TCustomScript; const URL: String): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_KmlLayer,URL,true);

end;

class function TKmlLayer.New(Script: TCustomScript; const URL: String; Opts: TKmlLayerOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script,URL)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_KmlLayer,
                                     {$IFDEF CAN_OVERLOAD}
                                     [URL,Opts.Disp]
                                     {$ELSE}
                                     URL,Opts.Disp
                                     {$ENDIF}
                                     ,true);
end;

function TKmlLayer.GetDefaultViewport: TLatLngBounds;
var
  DefaultViewport: OleVariant;
begin
  Result:=nil;
  DefaultViewport:=Exec(KmlLayerMethod_GetDefaultViewport);
  if VarIsValidDispatch(DefaultViewport) then
  begin
    if assigned(FDefaultViewport)
      then FDefaultViewport.Disp:=DefaultViewport
      else FDefaultViewport:=TLatLngBounds(CreateObjectWrapper(Script,TLatLngBounds,
                                                               DefaultViewport));
    Result:=FDefaultViewport;
  end;
end;

function TKmlLayer.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(KmlLayerMethod_GetMap);
  if VarIsValidDispatch(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

procedure TKmlLayer.SetMap(Map: TMap);
begin
  Exec(KmlLayerMethod_SetMap,Map);
end;

function TKmlLayer.GetMetadata: TKmlLayerMetadata;
var
  Metadata: OleVariant;
begin
  Result:=nil;
  Metadata:=Exec(KmlLayerMethod_GetMetadata);
  if VarIsValidDispatch(Metadata) then
  begin
    if assigned(FMetadata)
      then FMetadata.Disp:=Metadata
      else FMetadata:=TKmlLayerMetadata(CreateObjectWrapper(Script,TKmlLayerMetadata,Metadata));
    Result:=FMetadata;
  end;
end;

function TKmlLayer.GetUrl: String;
begin
  Result:=VarAsString(Exec(KmlLayerMethod_GetUrl));
end;

procedure TKmlLayer.Click;
var
  EventObject: TKmlMouseEvent;
begin
  if assigned(FOnClick) then
  begin
    try
      EventObject:=TKmlMouseEvent(CreateObjectWrapper(Script,TKmlMouseEvent,
                                                      FOnClickFunction.Arguments[0]));
      try
        FOnClick(self,EventObject);
      finally
        //EventObject.Free;
      end;
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
end;

procedure TKmlLayer.DefaultViewportChanged;
begin
  CallEvent(FOnDefaultViewportChanged);
end;

procedure TKmlLayer.SetOnClick(Value: TKmlLayerClickEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(Event_Click,FOnClickListener,FOnClickFunction,Click);
end;

procedure TKmlLayer.SetOnDefaultViewportChanged(Value: TNotifyEvent);
begin
  FOnDefaultViewportChanged:=Value;
  RemoveListener(FOnDefaultViewportListener);
  if assigned(FOnDefaultViewportChanged)
    then AddListener(KmlLayerEvent_DefaultViewportChanged,FOnDefaultViewportListener,
                     FOnDefaultViewportFunction,DefaultViewportChanged);
end;

//------------------------------------------------------------------------------
//TKmlLayerOptions.

constructor TKmlLayerOptions.Create;
begin
  inherited;
end;

constructor TKmlLayerOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TKmlLayerOptions.Destroy;
begin
  FMap.Free;
  inherited;
end;

function TKmlLayerOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(KmlLayerOptionsProp_Clickable);
end;

procedure TKmlLayerOptions.SetClickable(Value: Boolean);
begin
  SetProp(KmlLayerOptionsProp_Clickable,Value,true);
end;

function TKmlLayerOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(KmlLayerOptionsProp_Map,TMap,FMap));
end;

procedure TKmlLayerOptions.SetMap(Value: TMap);
begin
  SetProp(KmlLayerOptionsProp_Map,Value,true);
end;

function TKmlLayerOptions.GetPreserveViewport: Boolean;
begin
  Result:=GetPropAsBoolean(KmlLayerOptionsProp_PreserveViewport);
end;

procedure TKmlLayerOptions.SetPreserveViewport(Value: Boolean);
begin
  SetProp(KmlLayerOptionsProp_PreserveViewport,Value,true);
end;

function TKmlLayerOptions.GetSuppressInfoWindows: Boolean;
begin
  Result:=GetPropAsBoolean(KmlLayerOptionsProp_SuppressInfoWindows);
end;

procedure TKmlLayerOptions.SetSuppressInfoWindows(Value: Boolean);
begin
  SetProp(KmlLayerOptionsProp_SuppressInfoWindows,Value,true);
end;

//------------------------------------------------------------------------------
//TKmlFeatureData

destructor TKmlFeatureData.Destroy;
begin
  FAuthor.Free;
  inherited;
end;

function TKmlFeatureData.GetAuthor: TKmlAuthor;
begin
  Result:=TKmlAuthor(GetPropAsObject(KmlFeatureData_PropAuthor,TKmlAuthor,FAuthor));
end;

function TKmlFeatureData.GetDescription: String;
begin
  Result:=GetPropAsString(KmlFeatureData_Description);
end;

function TKmlFeatureData.GetID: String;
begin
  Result:=GetPropAsString(KmlFeatureData_ID);
end;

function TKmlFeatureData.GetInfoWindowHTML: String;
begin
  Result:=GetPropAsString(KmlFeatureData_InfoWindowHTML);
end;

function TKmlFeatureData.GetName: String;
begin
  Result:=GetPropAsString(KmlFeatureData_Name);
end;

function TKmlFeatureData.GetSnippet: String;
begin
  Result:=GetPropAsString(KmlFeatureData_Snippet);
end;

//------------------------------------------------------------------------------
//TKmlLayerMetadata

destructor TKmlLayerMetadata.Destroy;
begin
  FAuthor.Free;
  inherited;
end;

function TKmlLayerMetadata.GetAuthor: TKmlAuthor;
begin
  Result:=TKmlAuthor(GetPropAsObject(KmlLayerMetadataProp_Author,TKmlAuthor,FAuthor));
end;

function TKmlLayerMetadata.GetDescription: String;
begin
  Result:=GetPropAsString(KmlLayerMetadataProp_Description);
end;

function TKmlLayerMetadata.GetName: String;
begin
  Result:=GetPropAsString(KmlLayerMetadataProp_Name);
end;

function TKmlLayerMetadata.GetSnippet: String;
begin
  Result:=GetPropAsString(KmlLayerMetadataProp_Snippet);
end;

//------------------------------------------------------------------------------
//TKmlMouseEvent

destructor TKmlMouseEvent.Destroy;
begin
  FFeatureData.Free;
  FLatLng.Free;
  FPixelOffset.Free;
  inherited;
end;

function TKmlMouseEvent.GetFeatureData: TKmlFeatureData;
begin
  Result:=TKmlFeatureData(GetPropAsObject(KmlMouseEventProp_FeatureData,TKmlFeatureData,FFeatureData));
end;

function TKmlMouseEvent.GetPixelOffset: TSize;
begin
  Result:=TSize(GetPropAsObject(KmlMouseEventProp_PixelOffset,TSize,FPixelOffset));
end;

function TKmlMouseEvent.GetLatLng: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(KmlMouseEventProp_LatLng,TLatLng,FLatLng));
end;

//------------------------------------------------------------------------------
//TTrafficLayer

constructor TTrafficLayer.Create;
begin
  inherited;
end;

constructor TTrafficLayer.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TTrafficLayer.Destroy;
begin
  FMap.Free;
  inherited;
end;

class function TTrafficLayer.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_TrafficLayer,true);
end;

procedure TTrafficLayer.SetMap(Map: TMap);
begin
  Exec(TrafficLayerMethod_SetMap,Map);
end;

function TTrafficLayer.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(TrafficLayerMethod_GetMap);
  if VarIsValidDispatch(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

end.
