///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Places|TAutocomplete"/></item>
///    <item><see cref="gmLibraries.Places|TAutocompleteOptions"/></item>
///    <item><see cref="gmLibraries.Places|TPlaces"/></item>
///    <item><see cref="gmLibraries.Places|TPlaceDetailsRequest"/></item>
///    <item><see cref="gmLibraries.Places|TPlaceGeometry"/></item>
///    <item><see cref="gmLibraries.Places|TPlaceResult"/></item>
///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest"/></item>
///    <item><see cref="gmLibraries.Places|TPlacesService"/></item>
///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus"/></item>
///  </list>
///  <para><b>Additional Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Places|TPlaceResultArray"/></item>
///  </list>
///  <para><b>Types</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Places|TPlacesServiceGetDetailsCallback"/></item>
///    <item><see cref="gmLibraries.Places|TPlacesServiceSearchCallback"/></item>
///  </list>
///  <para><b>Functions</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Places|New(TAutocomplete)">New: TAutocomplete</see></item>
///    <item><see cref="gmLibraries.Places|New(TPlacesService)">New: TPlacesService</see></item>
///  </list>
///</summary>
unit gmLibrariesPlaces;

{$INCLUDE gmConfig.inc}

interface

uses
  Classes, SysUtils,{$IFDEF DEBUG} Dialogs,{$ENDIF} BrowserTools,
  JScriptObjects, DispObject, HTMLObjects, gmBase, gmMap, gmServicesGeocoder,
  gmMVC, gmLayers, gmEvents, gmControls, gmAdditionalConstants,
  gmAdditionalMessages;

type
  TAutocomplete = class;
  TAutocompleteOptions = class;
  THTMLEdit = class;
  TPlaces = class;
  TPlaceDetailsRequest = class;
  TPlaceGeometry = class;
  TPlaceResult = class;
  TPlaceResultArray = class;
  TPlaceSearchRequest = class;
  TPlacesService = class;
  TPlacesServiceStatus = class;

  THTMLDivElement = class(TCustomJScriptObject);                 //ToDo -> HTMLObjects

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TAutocomplete'}{$ENDIF}
  ///<summary>
  ///  A service to provide Place predictions based on a user's text input.
  ///  It attaches to an input element of type text, and listens for text entry
  ///  in that field. The list of predictions is presented as a drop-down list,
  ///  and is updated as text is entered. This class extends <c>TMVCObject</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaces.Autocomplete(THTMLInputElement,TAutocompleteOptions)">Google.Maps.Places.Autocomplete</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.SetBounds(TLatLngBounds)">SetBounds</see></item>
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.SetTypes(TStringArray)">SetTypes</see></item>
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.GetBounds">GetBounds</see></item>
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.GetPlace">GetPlace</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.Bounds">Bounds</see></item>
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.Place">Place</see></item>
  ///  </list>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TAutocomplete.OnPlaceChanged">OnPlaceChanged</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Autocomplete">
  ///  Autocomplete
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TAutocomplete = class(TMVCObject)
  private
    FBounds: TLatLngBounds;
    FPlace: TPlaceResult;
    FOnPlaceChanged: TNotifyEvent;
    FOnPlaceChangedListener: TMapsEventListener;
    FOnPlaceChangedFunction: TJScriptFunction;
    procedure SetOnPlaceChanged(Value: TNotifyEvent);
  protected
    //JavaScript event handler:
    procedure PlaceChanged;
  public
    constructor Create(InputField: THTMLEdit; Opts: TAutocompleteOptions); reintroduce; overload;
    constructor Create(InputField: THTMLInputElement; Opts: TAutocompleteOptions); reintroduce; overload;
    constructor Create(Script: TCustomScript; InputField: THTMLEdit; Opts: TAutocompleteOptions); reintroduce; overload;
    constructor Create(Script: TCustomScript; InputField: THTMLInputElement; Opts: TAutocompleteOptions); reintroduce; overload;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetBounds'}{$ENDIF}
    ///<summary>
    ///  Sets the preferred area within which to return Place results. Results
    ///  are biased towards, but not restricted to, this area.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetBounds(Bounds: TLatLngBounds);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetTypes'}{$ENDIF}
    ///<summary>
    ///  Sets the types of predictions to be returned. Supported types are
    ///  'establishment' for businesses and 'geocode' for addresses. If no type
    ///  is specified, both types will be returned.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray"/>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetTypes(Types: TStringArray);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetBounds'}{$ENDIF}
    ///<summary>
    ///  Returns the bounds to which predictions are biased.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetBounds: TLatLngBounds;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPlace'}{$ENDIF}
    ///<summary>
    ///  Returns the details of the Place selected by the user, or nil if no
    ///  Place has been selected yet.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPlace: TPlaceResult;
    //Additional Delphi properties:
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Bounds'}{$ENDIF}
    ///<summary>
    ///  Additional Delphi property.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Bounds: TLatLngBounds read GetBounds write SetBounds;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Place'}{$ENDIF}
    ///<summary>
    ///  Additional Delphi property.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Place: TPlaceResult read GetPlace;
    //Events:
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnPlaceChanged'}{$ENDIF}
    ///<summary>
    ///  This event is fired when a PlaceResult is made available for a Place
    ///  the user has selected. If the user enters the name of a Place that was
    ///  not suggested by the control and presses the Enter key, a
    ///  <c>OnPlaceChanged</c> event will be fired that contains the user input
    ///  in the name property, with no other properties defined.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnPlaceChanged: TNotifyEvent read FOnPlaceChanged write SetOnPlaceChanged;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TAutocompleteOptions'}{$ENDIF}
  ///<summary>
  ///  The options that can be set on an <c>TAutocomplete</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TAutocompleteOptions.Bounds">Bounds</see></item>
  ///    <item><see cref="gmLibraries.Places|TAutocompleteOptions.Types">Types</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmLibraries.Places|TAutocomplete"/>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#AutocompleteOptions">
  ///  AutocompleteOptions
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TAutocompleteOptions = class(TCustomJScriptObject)
  private
    FBounds: TLatLngBounds;
    FTypes: TStringArray;
    procedure SetBounds(Value: TLatLngBounds);
    procedure SetTypes(Value: TStringArray);
    function GetBounds: TLatLngBounds;
    function GetTypes: TStringArray;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Bounds'}{$ENDIF}
    ///<summary>
    ///  The area in which to search for Places. Results are biased towards, but
    ///  not restricted to, Places contained within these bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmLibraries.Places|TAutocompleteOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Bounds: TLatLngBounds read GetBounds write SetBounds;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Types'}{$ENDIF}
    ///<summary>
    ///  The types of predictions to be returned. Supported types are
    ///  'establishment' for businesses and 'geocode' for addresses. If neither
    ///  is specified, both types are returned.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray"/>
    ///<seealso cref="gmLibraries.Places|TAutocompleteOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Types: TStringArray read GetTypes write SetTypes;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation THTMLEdit'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;PlaceResult&gt;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|THTMLEdit.Left">Left</see></item>
  ///    <item><see cref="gmLibraries.Places|THTMLEdit.Name">Name</see></item>
  ///    <item><see cref="gmLibraries.Places|THTMLEdit.Text">Text</see></item>
  ///    <item><see cref="gmLibraries.Places|THTMLEdit.Top">Top</see></item>
  ///    <item><see cref="gmLibraries.Places|THTMLEdit.Visible">Visible</see></item>
  ///    <item><see cref="gmLibraries.Places|THTMLEdit.Width">Width</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  THTMLEdit = class(TObject)
  private
    FHTMLInputElement: THTMLInputElement;
    FScript: TCustomScript;
    function GetLeft: Integer;
    function GetName: String;
    function GetText: String;
    function GetTop: Integer;
    function GetWidth: Integer;
    function GetVisible: Boolean;
    procedure SetLeft(Value: Integer);
    procedure SetName(Value: String);
    procedure SetText(Value: String);
    procedure SetTop(Value: Integer);
    procedure SetWidth(Value: Integer);
    procedure SetVisible(Value: Boolean);
  public
    constructor Create; overload;
    constructor Create(Script: TCustomScript); overload;
    constructor Create(Map: TMap; Position: Integer = -1); overload;
    constructor Create(Script: TCustomScript; Map: TMap; Position: Integer = -1); overload;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Left'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|THTMLEdit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Left: Integer read GetLeft write SetLeft;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Name'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|THTMLEdit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Name: String read GetName write SetName;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Text'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|THTMLEdit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Text: String read GetText write SetText;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Top'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|THTMLEdit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Top: Integer read GetTop write SetTop;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Visible'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|THTMLEdit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Visible: Boolean read GetVisible write SetVisible;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Width'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|THTMLEdit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Width: Integer read GetWidth write SetWidth;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlaces'}{$ENDIF}
  ///<summary>
  ///  Library.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaces.Autocomplete(THTMLEdit,TAutocompleteOptions)">Autocomplete(THTMLEdit,TAutocompleteOptions)</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaces.Autocomplete(THTMLInputElement,TAutocompleteOptions)">Autocomplete(THTMLInputElement,TAutocompleteOptions))</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaces.PlacesService(THTMLDivElement)">PlacesService(THTMLDivElement)</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaces.PlacesService(TMap)">PlacesService(TMap)</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaces.PlacesServiceStatus">PlacesServiceStatus</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/places.html">Places</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlaces = class(TCustomJScriptObject)
  private
    FPlacesServiceStatus: TPlacesServiceStatus;
    function GetPlacesServiceStatus: TPlacesServiceStatus;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Autocomplete'}{$ENDIF}
    ///<summary>
    ///  Creates a new instance of <c>TAutocomplete</c> that attaches to the
    ///  specified input text field with the given options.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaces"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Autocomplete(InputField: THTMLEdit; Opts: TAutocompleteOptions = nil): TAutocomplete; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Autocomplete'}{$ENDIF}
    ///<summary>
    ///  Creates a new instance of <c>TAutocomplete</c> that attaches to the
    ///  specified input text field with the given options.
    ///</summary>
    ///<seealso cref="HTMLObjects|THTMLInputElement"/>
    ///<seealso cref="gmLibraries.Places|TPlaces"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Autocomplete(InputField: THTMLInputElement; Opts: TAutocompleteOptions = nil): TAutocomplete; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PlacesService'}{$ENDIF}
    ///<summary>
    ///  Creates a new instance of the PlacesService that renders attributions
    ///  in the specified container.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaces"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function PlacesService(AttrContainer: THTMLDivElement): TPlacesService; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PlacesService'}{$ENDIF}
    ///<summary>
    ///  Creates a new instance of the PlacesService that renders attributions
    ///  in the specified container.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Places|TPlaces"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function PlacesService(AttrContainer: TMap): TPlacesService; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PlacesServiceStatus'}{$ENDIF}
    ///<summary>
    ///  The status returned by the PlacesService on the completion of its searches.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaces"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PlacesServiceStatus: TPlacesServiceStatus read GetPlacesServiceStatus;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlaceDetailsRequest'}{$ENDIF}
  ///<summary>
  ///  A Place details query to be sent to the PlacesService.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaceDetailsRequest.Reference">Reference</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlaceDetailsRequest">
  ///  PlaceDetailsRequest
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlaceDetailsRequest = class(TCustomJScriptObject)
  private
    procedure SetReference(Value: String);
    function GetReference: String;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Reference'}{$ENDIF}
    ///<summary>
    ///  The reference of the Place for which details are being requested
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceDetailsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Reference: String read GetReference write SetReference;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlaceGeometry'}{$ENDIF}
  ///<summary>
  ///  Defines information about the geometry of a Place.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaceGeometry.Location">Location</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceGeometry.Viewport">Viewport</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlaceGeometry">
  ///  PlaceGeometry
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlaceGeometry = class(TCustomJScriptObject)
  private
    FLocation: TLatLng;
    FViewport: TLatLngBounds;
    procedure SetLocation(Value: TLatLng);
    procedure SetViewport(Value: TLatLngBounds);
    function GetLocation: TLatLng;
    function GetViewport: TLatLngBounds;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Location'}{$ENDIF}
    ///<summary>
    ///  The Place's position.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmLibraries.Places|TPlaceGeometry"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Location: TLatLng read GetLocation write SetLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Viewport'}{$ENDIF}
    ///<summary>
    ///  The preferred viewport when displaying this Place on a map. This
    ///  property will be nil if the preferred viewport for the Place is not
    ///  known.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmLibraries.Places|TPlaceGeometry"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Viewport: TLatLngBounds read GetViewport write SetViewport;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlaceResult'}{$ENDIF}
  ///<summary>
  ///  Defines information about a Place.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.AddressComponents">AddressComponents</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.FormattedAddress">FormattedAddress</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.FormattedPhoneNumber">FormattedPhoneNumber</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Geometry">Geometry</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.HtmlAttributions">HtmlAttributions</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Icon">Icon</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.ID">ID</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Name">Name</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Rating">Rating</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Reference">Reference</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Types">Types</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.URL">URL</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceResult.Vicinity">Vicinity</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlaceResult">
  ///  PlaceResult
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlaceResult = class(TCustomJScriptObject)
  private
    FAddressComponents: TGeocoderAddressComponentArray;
    FGeometry: TPlaceGeometry;
    FTypes: TStringArray;
    procedure SetAddressComponents(Value: TGeocoderAddressComponentArray);
    procedure SetFormattedAddress(Value: String);
    procedure SetFormattedPhoneNumber(Value: String);
    procedure SetGeometry(Value: TPlaceGeometry);
    procedure SetHtmlAttributions(Value: String);
    procedure SetIcon(Value: String);
    procedure SetID(Value: String);
    procedure SetName(Value: String);
    procedure SetRating(Value: Double);
    procedure SetReference(Value: String);
    procedure SetTypes(Value: TStringArray);
    procedure SetURL(Value: String);
    procedure SetVicinity(Value: String);
    function GetAddressComponents: TGeocoderAddressComponentArray;
    function GetFormattedAddress: String;
    function GetFormattedPhoneNumber: String;
    function GetGeometry: TPlaceGeometry;
    function GetHtmlAttributions: String;
    function GetIcon: String;
    function GetID: String;
    function GetName: String;
    function GetRating: Double;
    function GetReference: String;
    function GetTypes: TStringArray;
    function GetURL: String;
    function GetVicinity: String;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AddressComponents'}{$ENDIF}
    ///<summary>
    ///  The collection of address components for this Place's location.
    ///</summary>
    ///<seealso cref="gmGeocoder|TGeocoderAddressComponentArray"/>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AddressComponents: TGeocoderAddressComponentArray read GetAddressComponents write SetAddressComponents;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation FormattedAddress'}{$ENDIF}
    ///<summary>
    ///  The Place's full address.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property FormattedAddress: String read GetFormattedAddress write SetFormattedAddress;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation FormattedPhoneNumber'}{$ENDIF}
    ///<summary>
    ///  The Place's phone number, formatted according to the number's regional
    ///  convention.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property FormattedPhoneNumber: String read GetFormattedPhoneNumber write SetFormattedPhoneNumber;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Geometry'}{$ENDIF}
    ///<summary>
    ///  The Place's geometry-related information.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Geometry: TPlaceGeometry read GetGeometry write SetGeometry;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation HtmlAttributions'}{$ENDIF}
    ///<summary>
    ///  Attribution text to be displayed for this Place result.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property HtmlAttributions: String read GetHtmlAttributions write SetHtmlAttributions;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Icon'}{$ENDIF}
    ///<summary>
    ///  URL to an image resource that can be used to represent this Place's
    ///  category.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Icon: String read GetIcon write SetIcon;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ID'}{$ENDIF}
    ///<summary>
    ///  A unique identifier denoting this Place. This identifier may not be
    ///  used to retrieve information about this Place, and to verify the
    ///  identity of a Place across separate searches. As ids can occasionally
    ///  change, it is recommended that the stored id for a Place be compared
    ///  with the id returned in later Details requests for the same Place, and
    ///  updated if necessary.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ID: String read GetID write SetID;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Name'}{$ENDIF}
    ///<summary>
    ///  The Place's name. Note: In the case of user entered Places, this is the
    ///  raw text, as typed by the user. Please exercise caution when using this
    ///  data, as malicious users may try to use it as a vector for
    ///  <see href="http://en.wikipedia.org/wiki/Code_injection">
    ///  code injection</see> attacks.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Name: String read GetName write SetName;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Rating'}{$ENDIF}
    ///<summary>
    ///  The Place's rating, from 0.0 to 5.0, based on user reviews
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Rating: Double read GetRating write SetRating;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Reference'}{$ENDIF}
    ///<summary>
    ///  An opaque string that may be used to retrieve up-to-date information
    ///  about this Place (via <c>TPlacesService.GetDetails</c>) reference
    ///  contains a unique token that you can use to retrieve additional
    ///  information about this Place in a Place Details request. You can store
    ///  this token and use it at any time in future to refresh cached data
    ///  about this Place, but the same token is not guaranteed to be returned
    ///  for any given Place across different searches.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Reference: String read GetReference write SetReference;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Types'}{$ENDIF}
    ///<summary>
    ///  An array of types for this Place (e.g., ['political','locality'] or
    ///  ['restaurant','establishment']).
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Types: TStringArray read GetTypes write SetTypes;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation URL'}{$ENDIF}
    ///<summary>
    ///  URL of the associated Google Place Page.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property URL: String read GetURL write SetURL;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Vicinity'}{$ENDIF}
    ///<summary>
    ///  A fragment of the Place's address for disambiguation (usually street
    ///  name and locality).
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Vicinity: String read GetVicinity write SetVicinity;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlaceResultArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;PlaceResult&gt;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Bounds">Bounds</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Location">Location</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Name">Name</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Radius">Radius</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Types">Types</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  ///<seealso cref="gmLibraries.Places|TPlacesServiceSearchCallback"/>
  ///<seealso cref="gmLibraries.Places|TPlacesService.Search(TPlaceSearchRequest,TPlacesServiceSearchCallback)">TPlacesService.Search</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlaceSearchRequest">
  ///  PlaceSearchRequest
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlaceResultArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TPlaceResult;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<seealso cref="gmLibraries.Places|TPlaceResultArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TPlaceResult read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlaceSearchRequest'}{$ENDIF}
  ///<summary>
  ///  A Place search query to be sent to the PlacesService.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Bounds">Bounds</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Location">Location</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Name">Name</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Radius">Radius</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaceSearchRequest.Types">Types</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlaceSearchRequest">
  ///  PlaceSearchRequest
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlaceSearchRequest = class(TCustomJScriptObject)
  private
    FBounds: TLatLngBounds;
    FLocation: TLatLng;
    FTypes: TStringArray;
    procedure SetBounds(Value: TLatLngBounds);
    procedure SetLocation(Value: TLatLng);
    procedure SetName(Value: String);
    procedure SetRadius(Value: Integer);
    procedure SetTypes(Value: TStringArray);
    function GetBounds: TLatLngBounds;
    function GetLocation: TLatLng;
    function GetName: String;
    function GetRadius: Integer;
    function GetTypes: TStringArray;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Bounds'}{$ENDIF}
    ///<summary>
    /// The bounds within which to search for Places. Both location and radius
    /// will be ignored if bounds is set.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmLibraries.Places|TPlaceSearchRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Bounds: TLatLngBounds read GetBounds write SetBounds;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Location'}{$ENDIF}
    ///<summary>
    ///  The location around which to search for Places.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmLibraries.Places|TPlaceSearchRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Location: TLatLng read GetLocation write SetLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Name'}{$ENDIF}
    ///<summary>
    ///  Restricts the Place search results to Places that include this text in
    ///  the name.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceSearchRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Name: String read GetName write SetName;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Radius'}{$ENDIF}
    ///<summary>
    ///  The distance from the given location within which to search for Places.
    ///  Note that this is not a strict filter. Places outside this distance may
    ///  be returned if an insufficient number of matching Places are found
    ///  within the specified distance.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceSearchRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Radius: Integer read GetRadius write SetRadius;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Types'}{$ENDIF}
    ///<summary>
    ///  Restricts the Place search results to Places with a type matching at
    ///  least one of the specified types in this array. Valid types are given
    ///  here.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray"/>
    ///<seealso cref="gmLibraries.Places|TPlaceSearchRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Types: TStringArray read GetTypes write SetTypes;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlacesServiceGetDetailsCallback'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; SearchResult: </c>
  ///  <see cref="gmLibraries.Places|TPlaceResult"/><c>; Status: </c>
  ///  <see cref="String"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
  ///<seealso cref="gmLibraries.Places|TPlacesService.GetDetails(TPlaceDetailsRequest,TPlacesServiceGetDetailsCallback)">
  ///  TPlacesService.GetDetails
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlacesServiceGetDetailsCallback = procedure(Sender: TObject;
                                               SearchResult: TPlaceResult;
                                               Status: String) of object;
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlacesServiceGetDetailsCallbackA'}{$ENDIF}
  ///<summary>
  ///  <c>reference to procedure(Sender: </c><see cref="TObject"/><c>; SearchResult: </c>
  ///  <see cref="gmLibraries.Places|TPlaceResult"/><c>; Status: </c>
  ///  <see cref="String"/><c>);</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
  ///<seealso cref="gmLibraries.Places|TPlacesService.GetDetails(TPlaceDetailsRequest,TPlacesServiceGetDetailsCallback)">
  ///  TPlacesService.GetDetails
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TPlacesServiceGetDetailsCallbackA = reference to procedure(Sender: TObject;
                                                             SearchResult: TPlaceResult;
                                                             Status: String);
  {$ENDIF}
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlacesServiceSearchCallback'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; SearchResult: </c>
  ///  <see cref="gmLibraries.Places|TPlaceResultArray"/><c>; Status: </c>
  ///  <see cref="String"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
  ///<seealso cref="gmLibraries.Places|TPlacesService.Search(TPlaceSearchRequest,TPlacesServiceSearchCallback)">
  ///  TPlacesService.Search
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlacesServiceSearchCallback = procedure(Sender: TObject;
                                           SearchResult: TPlaceResultArray;
                                           Status: String) of object;
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlacesServiceSearchCallbackA'}{$ENDIF}
  ///<summary>
  ///  <c>reference to procedure(Sender: </c><see cref="TObject"/><c>; SearchResult: </c>
  ///  <see cref="gmLibraries.Places|TPlaceResultArray"/><c>; Status: </c>
  ///  <see cref="String"/><c>);</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
  ///<seealso cref="gmLibraries.Places|TPlacesService.Search(TPlaceSearchRequest,TPlacesServiceSearchCallback)">
  ///  TPlacesService.Search
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TPlacesServiceSearchCallbackA = reference to procedure(Sender: TObject;
                                                         SearchResult: TPlaceResultArray;
                                                         Status: String);
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlacesService'}{$ENDIF}
  ///<summary>
  ///  Contains methods related to searching for Places and retrieving details
  ///  about a Place.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlaces.PlacesService(THTMLDivElement)">Google.Maps.Places.PlacesService(THTMLDivElement)</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlaces.PlacesService(TMap)">Google.Maps.Places.PlacesService(TMap)</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlacesService.GetDetails(TPlaceDetailsRequest,TPlacesServiceGetDetailsCallback)">GetDetails(TPlaceDetailsRequest,TPlacesServiceGetDetailsCallback)</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesService.GetDetails(TPlaceDetailsRequest,TPlacesServiceGetDetailsCallbackA)">GetDetails(TPlaceDetailsRequest,TPlacesServiceGetDetailsCallbackA)</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesService.Search(TPlaceSearchRequest,TPlacesServiceSearchCallback)">Search(TPlaceSearchRequest,TPlacesServiceSearchCallback)</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesService.Search(TPlaceSearchRequest,TPlacesServiceSearchCallbackA)">Search(TPlaceSearchRequest,TPlacesServiceSearchCallbackA)</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmLibraries.Places|TPlaces"/>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlacesService">
  ///  PlacesService
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlacesService = class(TCustomJScriptObject)
  protected
    constructor Create(Script: TCustomScript; AttrContainer: TCustomJScriptObject); reintroduce; overload;
  public
    constructor Create(AttrContainer: TMap); reintroduce; overload;
    constructor Create(AttrContainer: THTMLDivElement); reintroduce; overload;
    constructor Create(Script: TCustomScript; AttrContainer: TMap); reintroduce; overload;
    constructor Create(Script: TCustomScript; AttrContainer: THTMLDivElement); reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetDetails'}{$ENDIF}
    ///<summary>
    ///  Retrieves details about the Place identified by the given reference.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    ///<seealso cref="gmLibraries.Places|TPlacesService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure GetDetails(Request: TPlaceDetailsRequest; Callback: TPlacesServiceGetDetailsCallback); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetDetails'}{$ENDIF}
    ///<summary>
    ///  Retrieves details about the Place identified by the given reference.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    ///<seealso cref="gmLibraries.Places|TPlacesService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetDetails(Request: TPlaceDetailsRequest; Callback: TPlacesServiceGetDetailsCallbackA); overload;
    {$ENDIF}
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Search'}{$ENDIF}
    ///<summary>
    ///  Retrieves a list of Places in a given area. The <c>TPlaceResultArray</c>
    ///  passed to the callback are stripped-down versions of a full
    ///  <c>TPlaceResult</c>. A more detailed <c>TPlaceResult</c> for each Place
    ///  can be obtained by sending a Place Details request with the desired
    ///  Place's reference value.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    ///<seealso cref="gmLibraries.Places|TPlaceResultArray"/>
    ///<seealso cref="gmLibraries.Places|TPlacesService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Search(Request: TPlaceSearchRequest; Callback: TPlacesServiceSearchCallback); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Search'}{$ENDIF}
    ///<summary>
    ///  Retrieves a list of Places in a given area. The <c>TPlaceResultArray</c>
    ///  passed to the callback are stripped-down versions of a full
    ///  <c>TPlaceResult</c>. A more detailed <c>TPlaceResult</c> for each Place
    ///  can be obtained by sending a Place Details request with the desired
    ///  Place's reference value.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaceResult"/>
    ///<seealso cref="gmLibraries.Places|TPlaceResultArray"/>
    ///<seealso cref="gmLibraries.Places|TPlacesService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure Search(Request: TPlaceSearchRequest; Callback: TPlacesServiceSearchCallbackA); overload;
    {$ENDIF}
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPlacesServiceStatus'}{$ENDIF}
  ///<summary>
  ///  The status returned by the PlacesService on the completion of its searches.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Places"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus.InvalidRequest">InvalidRequest</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus.OK">OK</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus.OverQueryLimit">OverQueryLimit</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus.RequestDenied">RequestDenied</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus.UnknownError">UnknownError</see></item>
  ///    <item><see cref="gmLibraries.Places|TPlacesServiceStatus.ZeroResults">ZeroResults</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PlacesServiceStatus">
  ///  PlacesServiceStatus
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPlacesServiceStatus = class(TCustomJScriptObject)
  private
    procedure SetInvalidRequest(Value: String);
    procedure SetOK(Value: String);
    procedure SetOverQueryLimit(Value: String);
    procedure SetRequestDenied(Value: String);
    procedure SetUnknownError(Value: String);
    procedure SetZeroResults(Value: String);
    function GetInvalidRequest: String;
    function GetOK: String;
    function GetOverQueryLimit: String;
    function GetRequestDenied: String;
    function GetUnknownError: String;
    function GetZeroResults: String;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InvalidRequest'}{$ENDIF}
    ///<summary>
    ///  This request was invalid.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property InvalidRequest: String read GetInvalidRequest write SetInvalidRequest;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OK'}{$ENDIF}
    ///<summary>
    ///  The response contains a valid result.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OK: String read GetOK write SetOK;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OverQueryLimit'}{$ENDIF}
    ///<summary>
    ///  The application has gone over its request quota.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OverQueryLimit: String read GetOverQueryLimit write SetOverQueryLimit;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RequestDenied'}{$ENDIF}
    ///<summary>
    ///  The application is not allowed to use the PlacesService.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RequestDenied: String read GetRequestDenied write SetRequestDenied;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnknownError'}{$ENDIF}
    ///<summary>
    ///  The PlacesService request could not be processed due to a server error.
    ///  The request may succeed if you try again.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UnknownError: String read GetUnknownError write SetUnknownError;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZeroResults'}{$ENDIF}
    ///<summary>
    ///  No result was found for this request.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlacesServiceStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZeroResults: String read GetZeroResults write SetZeroResults;
  end;

{$IFDEF USE_REGIONS}{$REGION 'Documentation New'}{$ENDIF}
///<summary>
///  <para><b>Unit</b></para>
///    <see cref="gmLibraries.Places"/>
///  <para/>
///</summary>
///<seealso cref="gmLibraries.Places|TPlaces"/>
///<seealso cref="gmLibraries.Places|TPlaces.Autocomplete(THTMLInputElement,TAutocompleteOptions)">
///  TPlaces.Autocomplete
///</seealso>
{$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
function New(Value: TAutocomplete): TAutocomplete; overload;
{$IFDEF USE_REGIONS}{$REGION 'Documentation New'}{$ENDIF}
///<summary>
///  <para><b>Unit</b></para>
///    <see cref="gmLibraries.Places"/>
///  <para/>
///</summary>
///<seealso cref="gmLibraries.Places|TPlaces"/>
///<seealso cref="gmLibraries.Places|TPlaces.PlacesService(THTMLDivElement)"/>
///<seealso cref="gmLibraries.Places|TPlaces.PlacesService(TMap)"/>
{$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
function New(Value: TPlacesService): TPlacesService; overload;

var
  PlacesRequests: Integer = 0;

implementation

uses
  gmAPI;

function New(Value: TAutocomplete): TAutocomplete;
begin
  Result:=Value;
end;

function New(Value: TPlacesService): TPlacesService;
begin
  Result:=Value;
end;

//------------------------------------------------------------------------------
//TAutocomplete

constructor TAutocomplete.Create(InputField: THTMLInputElement; Opts: TAutocompleteOptions);
begin
  Create(JScriptObjects.Script,InputField,Opts);
end;

constructor TAutocomplete.Create(Script: TCustomScript; InputField: THTMLInputElement; Opts: TAutocompleteOptions);
var
  Dispatch: IDispatch;
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
  if not assigned(InputField)
    then  raise Exception.Create(ExceptionNoInputField);
  if not assigned(Opts)
    then Dispatch:=BrowserTools.Create(TScript(Script).Google.Maps.Places.Disp,
                                       PlacesMethod_Autocomlete,InputField.Disp,
                                       true)
    else Dispatch:=BrowserTools.Create(TScript(Script).Google.Maps.Places.Disp,
                                       PlacesMethod_Autocomlete,
                                       {$IFDEF CAN_OVERLOAD}
                                       [InputField.Disp,Opts.Disp]
                                       {$ELSE}
                                       InputField.Disp,Opts.Disp
                                       {$ENDIF},
                                       true);
  inherited Create(Script,Dispatch);
end;

constructor TAutocomplete.Create(InputField: THTMLEdit; Opts: TAutocompleteOptions);
begin
  Create(JScriptObjects.Script,InputField,Opts);
end;

constructor TAutocomplete.Create(Script: TCustomScript; InputField: THTMLEdit; Opts: TAutocompleteOptions);
begin
  if not assigned(InputField)
    then raise Exception.Create(ExceptionNoInputField);
  Create(Script,InputField.FHTMLInputElement,Opts);
end;

destructor TAutocomplete.Destroy;
begin
  SetOnPlaceChanged(nil);
  FBounds.Free;
  FPlace.Free;
  inherited;
end;

function TAutocomplete.GetBounds: TLatLngBounds;
var
  Bounds: OleVariant;
begin
  Result:=nil;
  Bounds:=Exec(AutocompleteMethod_GetBounds);
  if VarIsValidDispatch(Bounds) then
  begin
    if assigned(FBounds)
      then FBounds.Disp:=Bounds
      else FBounds:=TLatLngBounds(CreateObjectWrapper(Script,TLatLngBounds,Bounds));
    Result:=FBounds;
  end;
end;

function TAutocomplete.GetPlace: TPlaceResult;
var
  Place: OleVariant;
begin
  Result:=nil;
  Place:=Exec(AutocompleteMethod_GetPlace);
  if VarIsValidDispatch(Place) then
  begin
    if assigned(FPlace)
      then FPlace.Disp:=Place
      else FPlace:=TPlaceResult(CreateObjectWrapper(Script,TPlaceResult,Place));
    Result:=FPlace;
  end;
end;

procedure TAutocomplete.PlaceChanged;
begin
  CallEvent(FOnPlaceChanged);
end;

procedure TAutocomplete.SetBounds(Bounds: TLatLngBounds);
begin
  Exec(AutocompleteMethod_SetBounds,Bounds);
end;

procedure TAutocomplete.SetOnPlaceChanged(Value: TNotifyEvent);
begin
  FOnPlaceChanged:=Value;
  RemoveListener(FOnPlaceChangedListener);
  if assigned(FOnPlaceChanged)
    then AddListener(AutocompleteEvent_PlaceChanged,FOnPlaceChangedListener,
                     FOnPlaceChangedFunction,PlaceChanged);
end;

procedure TAutocomplete.SetTypes(Types: TStringArray);
begin
  Exec(AutocompleteMethod_SetTypes,Types);
end;

//------------------------------------------------------------------------------
//TAutocompleteOptions

constructor TAutocompleteOptions.Create;
begin
  inherited;
end;

constructor TAutocompleteOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TAutocompleteOptions.Destroy;
begin
  FBounds.Free;
  FTypes.Free;
  inherited;
end;

function TAutocompleteOptions.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(AutocompleteOptionsProp_Bounds,TLatLngBounds,FBounds));
end;

function TAutocompleteOptions.GetTypes: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(AutocompleteOptionsProp_Types,TStringArray,FTypes));
end;

procedure TAutocompleteOptions.SetBounds(Value: TLatLngBounds);
begin
  SetProp(AutocompleteOptionsProp_Bounds,Value);
end;

procedure TAutocompleteOptions.SetTypes(Value: TStringArray);
begin
  SetProp(AutocompleteOptionsProp_Types,Value);
end;

//------------------------------------------------------------------------------
//THTMLEdit

constructor THTMLEdit.Create;
begin
  Create(JScriptObjects.Script,nil);
end;

constructor THTMLEdit.Create(Script: TCustomScript);
begin
  Create(Script,nil);
end;

constructor THTMLEdit.Create(Map: TMap; Position: Integer);
begin
  Create(JScriptObjects.Script,Map,Position);
end;

type
  TDocumentAccess = class(TDocument);

constructor THTMLEdit.Create(Script: TCustomScript; Map: TMap; Position: Integer);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  FScript:=Script;
  FHTMLInputElement:=THTMLInputElement(CreateObjectWrapper(FScript,THTMLInputElement,
                                       TDocumentAccess(FScript.Document).NewElement('input')));
  FHTMLInputElement.{$IFDEF AND_OP}&Type{$ELSE}Type_{$ENDIF}:='text';
  if assigned(Map) and (Script is TScript) then
  begin
    if Position<0
      then Position:=TScript(Script).Google.Maps.ControlPosition.TopCenter;
    Map.Controls.Items[Position].Push(FHTMLInputElement);
  end else FScript.Document.Body.AppendChild(FHTMLInputElement);
end;

destructor THTMLEdit.Destroy;
begin
  FHTMLInputElement.Free;
  inherited;
end;

function THTMLEdit.GetLeft: Integer;
var
  Value: String;
begin
  Value:=FHTMLInputElement.Style.MarginLeft;
  Result:=StrToInt(Copy(Value,1,Length(Value)-2));         //remove 'px'
end;

function THTMLEdit.GetName: String;
begin
  Result:=FHTMLInputElement.ID;
end;

function THTMLEdit.GetText: String;
begin
  Result:=FHTMLInputElement.Value;
end;

function THTMLEdit.GetTop: Integer;
var
  Value: String;
begin
  Value:=FHTMLInputElement.Style.MarginTop;
  Result:=StrToInt(Copy(Value,1,Length(Value)-2));         //remove 'px'
end;

function THTMLEdit.GetVisible: Boolean;
begin
  Result:=FHTMLInputElement.Style.Visibility<>'hidden';
end;

function THTMLEdit.GetWidth: Integer;
begin
  Result:=StrToInt(FHTMLInputElement.Size);
end;

procedure THTMLEdit.SetLeft(Value: Integer);
begin
  FHTMLInputElement.Style.MarginLeft:=IntToStr(Value)+'px';
end;

procedure THTMLEdit.SetName(Value: String);
begin
  FHTMLInputElement.ID:=Value;
end;

procedure THTMLEdit.SetText(Value: String);
begin
  FHTMLInputElement.Value:=Value;
end;

procedure THTMLEdit.SetTop(Value: Integer);
begin
  FHTMLInputElement.Style.MarginTop:=IntToStr(Value)+'px';
end;

procedure THTMLEdit.SetVisible(Value: Boolean);
begin
  if Value
    then FHTMLInputElement.Style.Visibility:='visible'
    else FHTMLInputElement.Style.Visibility:='hidden';
end;

procedure THTMLEdit.SetWidth(Value: Integer);
begin
  FHTMLInputElement.Size:=IntToStr(Value);
end;

//------------------------------------------------------------------------------
//TPlaces

destructor TPlaces.Destroy;
begin
  FPlacesServiceStatus.Free;
  inherited;
end;

function TPlaces.Autocomplete(InputField: THTMLInputElement; Opts: TAutocompleteOptions): TAutocomplete;
begin
  Result:=TAutocomplete.Create(Script,InputField,Opts);
end;

function TPlaces.Autocomplete(InputField: THTMLEdit; Opts: TAutocompleteOptions): TAutocomplete;
begin
  Result:=TAutocomplete.Create(Script,InputField,Opts);
end;

function TPlaces.PlacesService(AttrContainer: THTMLDivElement): TPlacesService;
begin
  Result:=TPlacesService.Create(Script,AttrContainer);
end;

function TPlaces.GetPlacesServiceStatus: TPlacesServiceStatus;
begin
  Result:=TPlacesServiceStatus(GetPropAsObject(PlacesProp_PlacesServiceStatus,
                                               TPlacesServiceStatus,
                                               FPlacesServiceStatus));
end;

function TPlaces.PlacesService(AttrContainer: TMap): TPlacesService;
begin
  Result:=TPlacesService.Create(Script,AttrContainer);
end;

//------------------------------------------------------------------------------
//TPlaceDetailsRequest

constructor TPlaceDetailsRequest.Create;
begin
  inherited;
end;

constructor TPlaceDetailsRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

function TPlaceDetailsRequest.GetReference: String;
begin
  Result:=GetPropAsString(PlaceDetailsRequestProp_Reference);
end;

procedure TPlaceDetailsRequest.SetReference(Value: String);
begin
  SetProp(PlaceDetailsRequestProp_Reference,Value,true);
end;

//------------------------------------------------------------------------------
//TPlaceGeometry

destructor TPlaceGeometry.Destroy;
begin
  FLocation.Free;
  FViewport.Free;
  inherited;
end;

function TPlaceGeometry.GetLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(PlaceGeometryProp_Location,TLatLng,FLocation));
end;

function TPlaceGeometry.GetViewport: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(PlaceGeometryProp_Viewport,TLatLngBounds,
                                        FViewport));
end;

procedure TPlaceGeometry.SetLocation(Value: TLatLng);
begin
  SetProp(PlaceGeometryProp_Location,Value,true);
end;

procedure TPlaceGeometry.SetViewport(Value: TLatLngBounds);
begin
  SetProp(PlaceGeometryProp_Viewport,Value,true);
end;

//------------------------------------------------------------------------------
//TPlaceResult

destructor TPlaceResult.Destroy;
begin
  FAddressComponents.Free;
  FGeometry.Free;
  FTypes.Free;
  inherited;
end;

function TPlaceResult.GetAddressComponents: TGeocoderAddressComponentArray;
begin
  Result:=TGeocoderAddressComponentArray(GetPropAsObject(PlaceResultProp_AddressComponents,
                                                         TGeocoderAddressComponentArray,
                                                         FAddressComponents));
end;

function TPlaceResult.GetFormattedAddress: String;
begin
  Result:=GetPropAsString(PlaceResultProp_FormattedAddress);
end;

function TPlaceResult.GetFormattedPhoneNumber: String;
begin
  Result:=GetPropAsString(PlaceResultProp_FormattedPhoneNumber);
end;

function TPlaceResult.GetGeometry: TPlaceGeometry;
begin
  Result:=TPlaceGeometry(GetPropAsObject(PlaceResultProp_Geometry,
                                         TPlaceGeometry,
                                         FGeometry));
end;

function TPlaceResult.GetHtmlAttributions: String;
begin
  Result:=GetPropAsString(PlaceResultProp_HtmlAttributions);
end;

function TPlaceResult.GetIcon: String;
begin
  Result:=GetPropAsString(PlaceResultProp_Icon);
end;

function TPlaceResult.GetID: String;
begin
  Result:=GetPropAsString(PlaceResultProp_Id);
end;

function TPlaceResult.GetName: String;
begin
  Result:=GetPropAsString(PlaceResultProp_Name);
end;

function TPlaceResult.GetRating: Double;
begin
  Result:=GetPropAsDouble(PlaceResultProp_Name);
end;

function TPlaceResult.GetReference: String;
begin
  Result:=GetPropAsString(PlaceResultProp_Reference);
end;

function TPlaceResult.GetTypes: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(PlaceResultProp_Types,
                                       TStringArray,
                                       FTypes));
end;

function TPlaceResult.GetURL: String;
begin
  Result:=GetPropAsString(PlaceResultProp_URL);
end;

function TPlaceResult.GetVicinity: String;
begin
  Result:=GetPropAsString(PlaceResultProp_Vicinity);
end;

procedure TPlaceResult.SetAddressComponents(Value: TGeocoderAddressComponentArray);
begin
  SetProp(PlaceResultProp_AddressComponents,Value,true);
end;

procedure TPlaceResult.SetFormattedAddress(Value: String);
begin
  SetProp(PlaceResultProp_FormattedAddress,Value,true);
end;

procedure TPlaceResult.SetFormattedPhoneNumber(Value: String);
begin
  SetProp(PlaceResultProp_FormattedPhoneNumber,Value,true);
end;

procedure TPlaceResult.SetGeometry(Value: TPlaceGeometry);
begin
  SetProp(PlaceResultProp_Geometry,Value,true);
end;

procedure TPlaceResult.SetHtmlAttributions(Value: String);
begin
  SetProp(PlaceResultProp_HtmlAttributions,Value,true);
end;

procedure TPlaceResult.SetIcon(Value: String);
begin
  SetProp(PlaceResultProp_Icon,Value,true);
end;

procedure TPlaceResult.SetID(Value: String);
begin
  SetProp(PlaceResultProp_ID,Value,true);
end;

procedure TPlaceResult.SetName(Value: String);
begin
  SetProp(PlaceResultProp_Name,Value,true);
end;

procedure TPlaceResult.SetRating(Value: Double);
begin
  SetProp(PlaceResultProp_Rating,Value,true);
end;

procedure TPlaceResult.SetReference(Value: String);
begin
  SetProp(PlaceResultProp_Reference,Value,true);
end;

procedure TPlaceResult.SetTypes(Value: TStringArray);
begin
  SetProp(PlaceResultProp_Types,Value,true);
end;

procedure TPlaceResult.SetURL(Value: String);
begin
  SetProp(PlaceResultProp_URL,Value,true);
end;

procedure TPlaceResult.SetVicinity(Value: String);
begin
  SetProp(PlaceResultProp_Vicinity,Value,true);
end;

//------------------------------------------------------------------------------
//TPlaceResultArray

function TPlaceResultArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TPlaceResult.Create(Script,Dispatch);
end;

function TPlaceResultArray.GetItem(Index: Integer): TPlaceResult;
begin
  Result:=TPlaceResult(inherited Items[Index]);
end;

//------------------------------------------------------------------------------
//TPlaceSearchRequest

constructor TPlaceSearchRequest.Create;
begin
  inherited;
end;

constructor TPlaceSearchRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TPlaceSearchRequest.Destroy;
begin
  FBounds.Free;
  FLocation.Free;
  FTypes.Free;
  inherited;
end;

function TPlaceSearchRequest.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(PlaceSearchRequestProp_Bounds,
                                        TLatLngBounds,FBounds));
end;

function TPlaceSearchRequest.GetLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(PlaceSearchRequestProp_Location,
                                  TLatLng,FLocation));
end;

function TPlaceSearchRequest.GetName: String;
begin
  Result:=GetPropAsString(PlaceSearchRequestProp_Name);
end;

function TPlaceSearchRequest.GetRadius: Integer;
begin
  Result:=GetPropAsInteger(PlaceSearchRequestProp_Radius);
end;

function TPlaceSearchRequest.GetTypes: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(PlaceSearchRequestProp_Types,
                                       TStringArray,FTypes,true));
end;

procedure TPlaceSearchRequest.SetBounds(Value: TLatLngBounds);
begin
  SetProp(PlaceSearchRequestProp_Bounds,Value,true);
end;

procedure TPlaceSearchRequest.SetLocation(Value: TLatLng);
begin
  SetProp(PlaceSearchRequestProp_Location,Value,true);
end;

procedure TPlaceSearchRequest.SetName(Value: String);
begin
  SetProp(PlaceSearchRequestProp_Name,Value,true);
end;

procedure TPlaceSearchRequest.SetRadius(Value: Integer);
begin
  SetProp(PlaceSearchRequestProp_Radius,Value,true);
end;

procedure TPlaceSearchRequest.SetTypes(Value: TStringArray);
begin
  SetProp(PlaceSearchRequestProp_Types,Value,true);
end;

//------------------------------------------------------------------------------
//TPlacesService

constructor TPlacesService.Create(AttrContainer: TMap);
begin
  Create(JScriptObjects.Script,AttrContainer);
end;

constructor TPlacesService.Create(AttrContainer: THTMLDivElement);
begin
  Create(JScriptObjects.Script,AttrContainer);
end;

constructor TPlacesService.Create(Script: TCustomScript; AttrContainer: TMap);
begin
  Create(Script,TCustomJScriptObject(AttrContainer));
end;

constructor TPlacesService.Create(Script: TCustomScript; AttrContainer: THTMLDivElement);
begin
  Create(Script,TCustomJScriptObject(AttrContainer));
end;

constructor TPlacesService.Create(Script: TCustomScript; AttrContainer: TCustomJScriptObject);
var
  Dispatch: IDispatch;
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
  if not assigned(AttrContainer)
    then Dispatch:=BrowserTools.Create(TScript(Script).Google.Maps.Places.Disp,
                                       PlacesMethod_PlacesService,true)
    else Dispatch:=BrowserTools.Create(TScript(Script).Google.Maps.Places.Disp,
                                       PlacesMethod_PlacesService,
                                       AttrContainer.Disp,true);
  inherited Create(Script,Dispatch);
end;

type
  TSearchType = (stSearch, stGetDetails);

  TPlacesServiceCallbackObject = class(TCallbackObject)
  private
    FSearchType: TSearchType;
    FSearchCallback: TPlacesServiceSearchCallback;
    FGetDetailsCallback: TPlacesServiceGetDetailsCallback;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FSearchCallbackA: TPlacesServiceSearchCallbackA;
    FGetDetailsCallbackA: TPlacesServiceGetDetailsCallbackA;
    {$ENDIF}
    FPlacesService: TPlacesService;
  protected
    procedure Callback; override;
  public
    constructor Create(PlacesService: TPlacesService; Request: TPlaceSearchRequest); reintroduce; overload;
    constructor Create(PlacesService: TPlacesService; Request: TPlaceDetailsRequest); reintroduce; overload;
    constructor Create(PlacesService: TPlacesService; Request: TPlaceSearchRequest;
                       Callback: TPlacesServiceSearchCallback); reintroduce; overload;
    constructor Create(PlacesService: TPlacesService; Request: TPlaceDetailsRequest;
                       Callback: TPlacesServiceGetDetailsCallback); reintroduce; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    constructor Create(PlacesService: TPlacesService; Request: TPlaceSearchRequest;
                       Callback: TPlacesServiceSearchCallbackA); reintroduce; overload;
    constructor Create(PlacesService: TPlacesService; Request: TPlaceDetailsRequest;
                       Callback: TPlacesServiceGetDetailsCallbackA); reintroduce; overload;
    {$ENDIF}
  end;

constructor TPlacesServiceCallbackObject.Create(PlacesService: TPlacesService;
                                                Request: TPlaceSearchRequest);
begin
  inherited Create(PlacesService.Script);
  FSearchType:=stSearch;
  FPlacesService:=PlacesService;
  FPlacesService.Exec(PlacesServiceMethod_Search,[Request.Disp,JScriptFunction.Disp]);
end;

constructor TPlacesServiceCallbackObject.Create(PlacesService: TPlacesService;
                                                Request: TPlaceDetailsRequest);
begin
  inherited Create(PlacesService.Script);
  FSearchType:=stGetDetails;
  FPlacesService:=PlacesService;
  FPlacesService.Exec(PlacesServiceMethod_GetDetails,[Request.Disp,JScriptFunction.Disp]);
end;

constructor TPlacesServiceCallbackObject.Create(PlacesService: TPlacesService;
                                                Request: TPlaceSearchRequest;
                                                Callback: TPlacesServiceSearchCallback);
begin
  FSearchCallback:=Callback;
  Create(PlacesService,Request);
end;

constructor TPlacesServiceCallbackObject.Create(PlacesService: TPlacesService;
                                                Request: TPlaceDetailsRequest;
                                                Callback: TPlacesServiceGetDetailsCallback);
begin
  FGetDetailsCallback:=Callback;
  Create(PlacesService,Request);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
constructor TPlacesServiceCallbackObject.Create(PlacesService: TPlacesService;
                                                Request: TPlaceSearchRequest;
                                                Callback: TPlacesServiceSearchCallbackA);
begin
  FSearchCallbackA:=Callback;
  Create(PlacesService,Request);
end;

constructor TPlacesServiceCallbackObject.Create(PlacesService: TPlacesService;
                                                Request: TPlaceDetailsRequest;
                                                Callback: TPlacesServiceGetDetailsCallbackA);
begin
  FGetDetailsCallbackA:=Callback;
  Create(PlacesService,Request);
end;
{$ENDIF}

procedure TPlacesServiceCallbackObject.Callback;
var
  GetDetailsResult: TPlaceResult;
  SearchResult: TPlaceResultArray;
begin
  try
    if ArgumentCount>=2 then
    begin
      case FSearchType of
        stSearch:
        begin
          SearchResult:=nil;
          if VarIsValidDispatch(Arguments[0])
            then SearchResult:=TPlaceResultArray(CreateObjectWrapper(Script,
                                                                     TPlaceResultArray,
                                                                     Arguments[0]));
          try
            {$IFDEF USE_ANONYMOUS_METHODS}
            if assigned(FSearchCallbackA)
              then FSearchCallbackA(FPlacesService,SearchResult,Arguments[1])
              else
            {$ENDIF}
            if assigned(FSearchCallback)
              then FSearchCallback(FPlacesService,SearchResult,Arguments[1]);
          finally
            //SearchResult.Free;       -> to save it for future use in anonymous methods
          end;
        end;
        stGetDetails:
        begin
          GetDetailsResult:=nil;
          if VarIsValidDispatch(Arguments[0])
            then GetDetailsResult:=TPlaceResult(CreateObjectWrapper(Script,
                                                                    TPlaceResult,
                                                                    Arguments[0]));
          try
            {$IFDEF USE_ANONYMOUS_METHODS}
            if assigned(FGetDetailsCallbackA)
              then FGetDetailsCallbackA(FPlacesService,GetDetailsResult,Arguments[1])
              else
            {$ENDIF}
            if assigned(FGetDetailsCallback)
              then FGetDetailsCallback(FPlacesService,GetDetailsResult,Arguments[1]);
          finally
            //GetDetailsResult.Free;   -> to save it for future use in anonymous methods
          end;
        end;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TPlacesService.GetDetails(Request: TPlaceDetailsRequest;
                                    Callback: TPlacesServiceGetDetailsCallback);
begin
  if assigned(Request) and assigned(Callback)
    then TPlacesServiceCallbackObject.Create(Self,Request,Callback);
end;

procedure TPlacesService.Search(Request: TPlaceSearchRequest;
                                Callback: TPlacesServiceSearchCallback);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Inc(PlacesRequests);
    TPlacesServiceCallbackObject.Create(Self,Request,Callback);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TPlacesService.GetDetails(Request: TPlaceDetailsRequest;
                                    Callback: TPlacesServiceGetDetailsCallbackA);
begin
  if assigned(Request) and assigned(Callback)
    then TPlacesServiceCallbackObject.Create(Self,Request,Callback);
end;

procedure TPlacesService.Search(Request: TPlaceSearchRequest;
                                Callback: TPlacesServiceSearchCallbackA);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Inc(PlacesRequests);
    TPlacesServiceCallbackObject.Create(Self,Request,Callback);
  end;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TPlacesServiceStatus

function TPlacesServiceStatus.GetInvalidRequest: String;
begin
  Result:=GetPropAsString(PlacesServiceStatusProp_InvalidRequest);
end;

function TPlacesServiceStatus.GetOK: String;
begin
  Result:=GetPropAsString(PlacesServiceStatusProp_OK);
end;

function TPlacesServiceStatus.GetOverQueryLimit: String;
begin
  Result:=GetPropAsString(PlacesServiceStatusProp_OverQueryLimit);
end;

function TPlacesServiceStatus.GetRequestDenied: String;
begin
  Result:=GetPropAsString(PlacesServiceStatusProp_RequestDenied);
end;

function TPlacesServiceStatus.GetUnknownError: String;
begin
  Result:=GetPropAsString(PlacesServiceStatusProp_UnknownError);
end;

function TPlacesServiceStatus.GetZeroResults: String;
begin
  Result:=GetPropAsString(PlacesServiceStatusProp_ZeroResults);
end;

procedure TPlacesServiceStatus.SetInvalidRequest(Value: String);
begin
  SetProp(PlacesServiceStatusProp_InvalidRequest,Value,true);
end;

procedure TPlacesServiceStatus.SetOK(Value: String);
begin
  SetProp(PlacesServiceStatusProp_OK,Value,true);
end;

procedure TPlacesServiceStatus.SetOverQueryLimit(Value: String);
begin
  SetProp(PlacesServiceStatusProp_OverQueryLimit,Value,true);
end;

procedure TPlacesServiceStatus.SetRequestDenied(Value: String);
begin
  SetProp(PlacesServiceStatusProp_RequestDenied,Value,true);
end;

procedure TPlacesServiceStatus.SetUnknownError(Value: String);
begin
  SetProp(PlacesServiceStatusProp_UnknownError,Value,true);
end;

procedure TPlacesServiceStatus.SetZeroResults(Value: String);
begin
  SetProp(PlacesServiceStatusProp_ZeroResults,Value,true);
end;

end.
