unit gmMapTypes;

interface

{$INCLUDE gmConfig.inc}

uses
  SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} {$IFDEF DEBUG}Dialogs,{$ENDIF}
  BrowserTools, {$IFDEF USE_ANONYMOUS_METHODS}AnonymousMethodsTools,{$ENDIF}
  JScriptObjects, HTMLObjects, DispObject, gmBase, gmMVC, gmEvents,
  gmAdditionalConstants, gmAdditionalMessages;

type
  TImageMapType = class;
  TImageMapTypeOptions = class;
  TMapType = class;
  TMapTypeRegistry = class;
  TMapTypeStyle = class;
  TMapTypeStyleArray = class;
  TMapTypeStylerArray = class;
  //Sind - im Gegensatz zur API-Dokumentation - keine Objekte!!!
  //-> Umsetzung als Record:
  //TMapTypeStyleFeatureType = class;
  //TMapTypeStyleElementType = class;
  TMapTypeStyler = class;
  TProjection = class;
  TStyledMapType = class;
  TStyledMapTypeOptions = class;

  TMapTypeStyleElementType = record
    All: String;
    Geometry: String;
    Labels: String;
  end;

  TAdministrative = record
    All: String;
    Country: String;
    Land_Parcel: String;
    Locality: String;
    Neighborhood: String;
    Province: String;
  end;

  TLandscape = record
    All: String;
    Man_Made: String;
    Natural: String;
  end;

  TPoi = record
    All: String;
    Attraction: String;
    Business: String;
    Government: String;
    Medical: String;
    Park: String;
    Place_Of_Worship: String;
    School: String;
    Sports_Complex: String;
  end;

  TRoad = record
    All: String;
    Arterial: String;
    Highway: String;
    Local: String;
  end;

  TStation = record
    All: String;
    Airport: String;
    Bus: String;
    Rail: String;
  end;

  TTransit = record
    All: String;
    Line: String;
    Station: TStation;
  end;

  TMapTypeStyleFeatureType = record
    All: String;
    Administrative: TAdministrative;
    Landscape: TLandscape;
    Poi: TPoi;
    Road: TRoad;
    Transit: TTransit;
    Water: String;
  end;

  //Mit dieser Schnittstelle wird der Kartentyp definiert. Diese Schnittstelle
  //wird normalerweise fr grundlegende Karten wie Straen-, Satelliten- oder
  //Hybridkarten verwendet. Nicht vernderbar.
  //This interface defines map type. This interface is typically used for base
  //maps such as road, satellite or hybrid maps. Immutable.
  TMapType = class(TMVCObject)
  private
    FGetTileFunction: TJScriptFunction;
    FProjection: TProjection;
    FReleaseTileFunction: TJScriptFunction;
    FTileSize: TSize;
    procedure SetAlt(Value: String);
    procedure SetMaxZoom(Value: Integer);
    procedure SetMinZoom(Value: Integer);
    procedure SetName(Value: String);
    procedure SetProjection(Value: TProjection);
    procedure SetRadius(Value: Integer);
    procedure SetTileSize(Value: TSize);
    function GetAlt: String;
    function GetMaxZoom: Integer;
    function GetMinZoom: Integer;
    function GetName: String;
    function GetProjection: TProjection;
    function GetRadius: Integer;
    function GetTileSize: TSize;
  protected
    procedure InitHandler; virtual;
    class function New(Script: TCustomScript): IDispatch; override;
    //JavaScript handler:
    procedure ReleaseTileHandler;
    function GetTileHandler: Variant;
  public
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    //Gibt die angegebene Kachel frei und fhrt gegebenenfalls ntige
    //Bereinigungen durch. Die bereitgestellte Kachel wurde dann bereits aus dem
    //Dokument entfernt. Optional:
    //Releases the given tile, performing any necessary cleanup. The provided
    //tile will have already been removed from the document. Optional:
    procedure ReleaseTile(Tile: TNode); virtual; abstract;
    //Gibt eine Kachel fr die angegebenen Kachelkoordinaten (x,y) und die
    //angegebene Zoomstufe zurck. Diese Kachel wird an das angegebene
    //ownerDocument angehngt:
    //Returns a tile for the given tile coordinate (x, y) and zoom level.
    //This tile will be appended to the given ownerDocument:
    function GetTile(TileCoord: TPoint; Zoom: Integer; OwnerDocument: TDocument): TNode; virtual; abstract;
    //Alternativer Text, der angezeigt werden soll, wenn die Maus in der
    //MapTypeControl ber die Schaltflche dieses MapTypes bewegt wird. Optional:
    //Alt text to display when this MapType's button is hovered over in
    //the MapTypeControl. Optional:
    property Alt: String read GetAlt write SetAlt;
    //Die maximale Zoomstufe fr die Karte, wenn dieser MapType angezeigt wird.
    //Erforderlich fr grundlegende MapTypes; wird ignoriert fr Overlay-MapTypes:
    //The maximum zoom level for the map when displaying this MapType.
    //Required for base MapTypes, ignored for overlay MapTypes:
    property MaxZoom: Integer read GetMaxZoom write SetMaxZoom;
    //Die minimale Zoomstufe fr die Karte, wenn dieser MapType angezeigt wird.
    //Optional; der Standardwert ist "0":
    //The minimum zoom level for the map when displaying this MapType.
    //Optional; defaults to 0:
    property MinZoom: Integer read GetMinZoom write SetMinZoom;
    //Name, der im MapTypeControl angezeigt werden soll. Optional:
    //Name to display in the MapTypeControl. Optional:
    property Name: String read GetName write SetName;
    //Die zur Wiedergabe dieses MapTypes verwendete Projektion. Optional; der
    //Standardwert ist "Mercator":
    //The Projection used to render this MapType. Optional; defaults to Mercator:
    property Projection: TProjection read GetProjection write SetProjection;
    //Radius des Planeten fr die Karte, angegeben in Metern. Optional; der
    //Standardwert ist der quatoriale Radius der Erde, der 6378137 Meter betrgt:
    //Radius of the planet for the map, in meters. Optional; defaults to Earth's
    //equatorial radius of 6378137 meters:
    property Radius: Integer read GetRadius write SetRadius;
    //Die Abmessungen der einzelnen Kacheln. Erforderlich:
    //The dimensions of each tile. Required:
    property TileSize: TSize read GetTileSize write SetTileSize;
  end;

  TMapTypeStyle = class(TCustomJScriptObject)
  private
    FStylers: TMapTypeStylerArray;
    procedure SetElementType(Value: String);
    procedure SetFeatureType(Value: String);
    procedure SetStylers(Value: TMapTypeStylerArray);
    function GetElementType: String;
    function GetFeatureType: String;
    function GetStylers: TMapTypeStylerArray;
  public
    destructor Destroy; override;
    //zustzliche Delphi-Methode:
    function AddStyler: TMapTypeStyler;
    //Whlt den Elementtyp aus, auf den ein Styler angewendet werden soll. Durch
    //den Elementtyp wird zwischen den verschiedenen Darstellungen einer Funktion
    //unterschieden. Optional; falls kein elementType angegeben ist, wird fr den
    //Wert 'all' angenommen:
    property ElementType: String read GetElementType write SetElementType;
    //Whlt die Funktion oder Gruppe von Funktionen aus, auf die ein Styler
    //angewendet werden soll. Optional; falls kein featureType angegeben ist, wird
    //fr den Wert 'all' angenommen:
    property FeatureType: String read GetFeatureType write SetFeatureType;
    //Die Stilregeln, die auf die Selektoren angewendet werden sollen. Die Regeln
    //werden in der Reihenfolge auf die Kartenelemente angewendet, in der sie in
    //diesem Array aufgefhrt sind:
    property Stylers: TMapTypeStylerArray read GetStylers write SetStylers;
  end;

  //Array.<MapTypeStyle>
  TMapTypeStyleArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TMapTypeStyle);
    function GetItem(Index: Integer): TMapTypeStyle;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    procedure Insert(Index: Integer; Value: TMapTypeStyle); reintroduce;
    function AddStyle: TMapTypeStyle;
    function InsertStyle(Index: Integer): TMapTypeStyle;
    property Items[Index: Integer]: TMapTypeStyle read GetItem write SetItem; default;
  end;

  TMapTypeStyler = class(TCustomJScriptObject)
  private
    procedure SetGamma(Value: Double);
    procedure SetHue(Value: String);
    procedure SetInvert_Lightness(Value: Boolean);
    procedure SetLightness(Value: Double);
    procedure SetSaturation(Value: Double);
    procedure SetVisibility(Value: String);
    function GetGamma: Double;
    function GetHue: String;
    function GetInvert_Lightness: Boolean;
    function GetLightness: Double;
    function GetSaturation: Double;
    function GetVisibility: String;
  public
    //Gamma. ndert den Gammawert durch Erhhen der Helligkeit auf die angegebene
    //Strke. Gltige Werte: Gleitkommazahlen, [0.01, 10], wobei 1.0 fr das
    //Beibehalten des Werts steht:
    property Gamma: Double read GetGamma write SetGamma;
    //Legt als Farbton der Funktion den angegebene Farbton fest. Beachten Sie,
    //dass die Sttigung und Helligkeit der Funktion beibehalten wird; die Farbe
    //der Funktion stimmt also nicht genau mit dem angegebenen Farbwert berein.
    property Hue: String read GetHue write SetHue;
    //Kehrt die Helligkeit um. Falls als Wert true festgelegt ist, wird die
    //Helligkeit der Funktion umgekehrt, whrend der Farbton und die Sttigung
    //beibehalten werden:
    property Invert_Lightness: Boolean read GetInvert_Lightness write SetInvert_Lightness;
    //Helligkeit. ndert die Helligkeit der Farben, und zwar bei abnehmender
    //Helligkeit um einen prozentualen Wert bezogen auf den Originalwert und bei
    //zunehmender Helligkeit um einen prozentualen Wert bezogen auf den Zielwert.
    //Gltige Werte: [-100, 100]:
    property Lightness: Double read GetLightness write SetLightness;
    //Sttigung. ndert die Sttigung der Farben, und zwar bei abnehmender
    //Sttigung um einen prozentualen Wert bezogen auf den Originalwert und bei
    //zunehmender Sttigung um einen prozentualen Wert bezogen auf den Zielwert.
    //Gltige Werte: [-100, 100]:
    property Saturation: Double read GetSaturation write SetSaturation;
    //Sichtbarkeit: Gltige Werte: 'on', 'off' oder 'simplifed':
    property Visibility: String read GetVisibility write SetVisibility;
  end;

  //Array.<MapTypeStylers>
  TMapTypeStylerArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TMapTypeStyler);
    function GetItem(Index: Integer): TMapTypeStyler;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    function Add: TMapTypeStyler; reintroduce;
    function Insert(Index: Integer): TMapTypeStyler; reintroduce;
    property Items[Index: Integer]: TMapTypeStyler read GetItem write SetItem; default;
  end;

  {$IFDEF USE_ANONYMOUS_METHODS}
  TGetTileUrlMethod = TFunc<TObject,TPoint,Integer,String>;
  {$ELSE}
  TGetTileUrlMethod = function(Sender: TObject; Point: TPoint; Zoom: Integer): String of object;
  {$ENDIF}

  //Diese Klasse implementiert die MapType-Schnittstelle und wird fr die
  //Wiedergabe von Bildkacheln bereitgestellt:
  TImageMapType = class(TMapType)
  private
    FGetTileUrl: TGetTileUrlMethod;
    FGetTileUrlFunction: TJScriptFunction;
  protected
    procedure InitHandler; override;
    class function New(Script: TCustomScript; Opts: TImageMapTypeOptions; GetTileUrlHandler: TExternalFunc): IDispatch; reintroduce;
    //JavaScript handler:
    function GetTileUrlHandler: Variant;
  public
    constructor Create(Opts: TImageMapTypeOptions); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TImageMapTypeOptions); reintroduce; overload;
    destructor Destroy; override;
    procedure ReleaseTile(Tile: TNode); override;
    function GetTile(TileCoord: TPoint; Zoom: Integer; OwnerDocument: TDocument): TNode; override;
    property GetTileUrl: TGetTileUrlMethod read FGetTileUrl write FGetTileUrl;
  end;

  //Mit dieser Klasse wird ein MapType fr die Wiedergabe von Bildkacheln erstellt:
  TImageMapTypeOptions = class(TCustomJScriptObject)
  private
    FGetTileUrl: TGetTileUrlMethod;
    FGetTileUrlFunction: TJScriptFunction;
    FTileSize: TSize;
    procedure SetAlt(Value: String);
    procedure SetIsPng(Value: Boolean);
    procedure SetMaxZoom(Value: Integer);
    procedure SetMinZoom(Value: Integer);
    procedure SetName(Value: String);
    procedure SetOpacity(Value: Double);
    procedure SetRadius(Value: Integer);
    procedure SetTileSize(Value: TSize);
    function GetAlt: String;
    function GetIsPng: Boolean;
    function GetMaxZoom: Integer;
    function GetMinZoom: Integer;
    function GetName: String;
    function GetOpacity: Double;
    function GetRadius: Integer;
    function GetTileSize: TSize;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //Alternativer Text, der angezeigt werden soll, wenn die Maus in der
    //MapTypeControl ber die Schaltflche dieses MapTypes bewegt wird:
    property Alt: String read GetAlt write SetAlt;
    //Gibt eine Zeichenfolge (URL) fr die angegebenen Kachelkoordinaten (x, y)
    //und die angegebene Zoomstufe zurck. Diese Funktion sollte die Signatur
    //getTileUrl(Point, number):string aufweisen:
    property GetTileUrl: TGetTileUrlMethod read FGetTileUrl write FGetTileUrl;
    //Handelt es sich um ein Bild im PNG-Format? Dieses Element wird von einigen
    //lteren Browsern bentigt:
    property IsPng: Boolean read GetIsPng write SetIsPng;
    //Die maximale Zoomstufe fr die Karte, wenn dieser MapType angezeigt wird:
    property MaxZoom: Integer read GetMaxZoom write SetMaxZoom;
    //Die minimale Zoomstufe fr die Karte, wenn dieser MapType angezeigt wird.
    //Optional:
    property MinZoom: Integer read GetMinZoom write SetMinZoom;
    //Name, der im MapTypeControl angezeigt werden soll:
    property Name: String read GetName write SetName;
    //Die Deckkraft, die auf die Kacheln angewendet werden soll. Die Deckkraft
    //sollte als Gleitkommazahl zwischen 0 und 1.0 angegeben werden, wobei 0
    //vollstndige Transparenz bedeutet und 1 vollstndige Deckkraft:
    property Opacity: Double read GetOpacity write SetOpacity;
    //Radius des Planeten fr die Karte, angegeben in Metern. Optional; der
    //Standardwert ist der quatoriale Radius der Erde, der 6378137 Meter betrgt:
    property Radius: Integer read GetRadius write SetRadius;
    //Die Kachelgre:
    property TileSize: TSize read GetTileSize write SetTileSize;
  end;

  //Die MapTypeRegistry beinhaltet die benutzerdefinierten Kartentypen, die fr
  //die Karte verfgbar sind. Auf diese Registrierung beruft sich das API zum
  //Beispiel, um die Liste der verfgbaren Kartentypen innerhalb der Steuerelemente
  //bereitzustellen:
  TMapTypeRegistry = class(TMVCObject)
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Legt fest, dass in der Registrierung der bergebene Zeichenfolgen-Kennzeichner
    //dem bergebenen MapType zugewiesen werden soll:
    procedure {$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(ID: String; MapType: TMapType); reintroduce;
  end;

  TProjection = class(TCustomJScriptObject)
  private
    FFromLatLngToPointFunction: TJScriptFunction;
    FFromPointToLatLngFunction: TJScriptFunction;
  protected
    function FromLatLngToPointHandler: Variant;
    function FromPointToLatLngHandler: Variant;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    function FromLatLngToPoint(LatLng: TLatLng; Point: TPoint): TPoint; virtual; abstract;
    function FromPointToLatLng(Pixel: TPoint; NoWrap: Boolean): TLatLng; virtual; abstract;
  end;

  TStyledMapType = class(TMapType)
  protected
    procedure InitHandler; override;
    class function New(Script: TCustomScript; Styles: TMapTypeStyleArray): IDispatch; reintroduce; overload;
    class function New(Script: TCustomScript; Styles: TMapTypeStyleArray; Opts: TStyledMapTypeOptions): IDispatch; reintroduce; overload;
  public
    constructor Create(Styles: TMapTypeStyleArray; Opts: TStyledMapTypeOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Styles: TMapTypeStyleArray; Opts: TStyledMapTypeOptions = nil); reintroduce; overload;
    procedure ReleaseTile(Tile: TNode); override;
    function GetTile(TileCoord: TPoint; Zoom: Integer; OwnerDocument: TDocument): TNode; override;
  end;

  TStyledMapTypeOptions = class(TCustomJScriptObject)
  private
    FBaseMapType: TStyledMapType;
    procedure SetAlt(Value: String);
    procedure SetBaseMapType(Value: TStyledMapType);
    procedure SetMaxZoom(Value: Integer);
    procedure SetMinZoom(Value: Integer);
    procedure SetName(Value: String);
    function GetAlt: String;
    function GetBaseMapType: TStyledMapType;
    function GetMaxZoom: Integer;
    function GetMinZoom: Integer;
    function GetName: String;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //Alternativer Text, der angezeigt werden soll, wenn die Maus im Kartensteuerelement
    //ber die Schaltflche dieses MapTypes bewegt wird:
    property Alt: String read GetAlt write SetAlt;
    //Ein StyledMapType, dessen Stil als Grundlage fr die Definition des Stils
    //von StyledMapTypes verwendet werden soll. Die MapTypeStyle-Regeln werden
    //an die Stile der Grundlage angehngt:
    property BaseMapType: TStyledMapType read GetBaseMapType write SetBaseMapType;
    //Die maximale Zoomstufe fr die Karte, wenn dieser MapType angezeigt wird. Optional:
    property MaxZoom: Integer read GetMaxZoom write SetMaxZoom;
    //Die minimale Zoomstufe fr die Karte, wenn dieser MapType angezeigt wird. Optional:
    property MinZoom: Integer read GetMinZoom write SetMinZoom;
    //Name, der im Kartentyp-Steuerelement angezeigt werden soll:
    property Name: String read GetName write SetName;
  end;

const
  MapTypeStyleElementType: TMapTypeStyleElementType = (
    All:      'all';
    Geometry: 'geometry';
    Labels:   'labels');

  MapTypeStyleFeatureType: TMapTypeStyleFeatureType = (
    All:                'all';
    Administrative: (
      All:              'administrative';
      Country:          'administrative.country';
      Land_Parcel:      'administrative.land_parcel';
      Locality:         'administrative.locality';
      Neighborhood:     'administrative.neighborhood';
      Province:         'administrative.province');
    Landscape: (
      All:              'landscape';
      Man_Made:         'landscape.man_made';
      Natural:          'landscape.natural');
    Poi: (
      All:              'poi';
      Attraction:       'poi.attraction';
      Business:         'poi.business';
      Government:       'poi.government';
      Medical:          'poi.medical';
      Park:             'poi.park';
      Place_Of_Worship: 'poi.place_of_worship';
      School:           'poi.school';
      Sports_Complex:   'poi.sports_complex');
    Road: (
      All:              'road';
      Arterial:         'road.arterial';
      Highway:          'road.highway';
      Local:            'road.local');
    Transit: (
      All:              'transit';
      Line:             'transit.line';
      Station: (
        All:            'transit.station';
        Airport:        'transit.station.airport';
        Bus:            'transit.station.bus';
        Rail:           'transit.station.rail'););
    Water:              'water');

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TImageMapType

constructor TImageMapType.Create(Opts: TImageMapTypeOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TImageMapType.Create(Script: TCustomScript; Opts: TImageMapTypeOptions);
begin
  inherited Create(Script,New(Script,Opts,GetTileURLHandler));
  FGetTileUrl:=Opts.FGetTileUrl;
  FGetTileUrlFunction:=Opts.FGetTileUrlFunction;
end;

destructor TImageMapType.Destroy;
begin
  FGetTileUrlFunction.Free;
  inherited;
end;

class function TImageMapType.New(Script: TCustomScript;
                                 Opts: TImageMapTypeOptions;
                                 GetTileUrlHandler: TExternalFunc): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then raise Exception.Create(ExceptionNoOpts);
  Opts.FGetTileUrlFunction:=Script.NewFunction(GetTileURLHandler);
  Opts.SetProp(ImageMapTypeOptionsProp_GetTileURL,Opts.FGetTileUrlFunction,true);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_ImageMapType,Opts.Disp,true);
end;

procedure TImageMapType.InitHandler;
begin
  //Don't initialize GetTile and ReleaseTile functions.
  //They are used internally by Google Maps objects.
end;

function TImageMapType.GetTile(TileCoord: TPoint; Zoom: Integer;
                               OwnerDocument: TDocument): TNode;
begin
  Result:=nil;
end;

procedure TImageMapType.ReleaseTile(Tile: TNode);
begin
end;

function TImageMapType.GetTileURLHandler: Variant;
var
  Point: TPoint;
  Zoom: Integer;
begin
  Result:='';
  if assigned(FGetTileURL) then
  begin
    try
      Point:=nil;
      Zoom:=0;
      with FGetTileUrlFunction do
      begin
        if (Arguments.Count>=1) and VarIsValidDispatch(Arguments[0])
          then Point:=TPoint(CreateObjectWrapper(Script,TPoint,Arguments[0]));
        if Arguments.Count>=2
          then Zoom:=Arguments[1];
      end;
      try
        Result:=GetTileURL(Self,Point,Zoom);
      finally
        //Point.Free;
      end;
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
end;

//------------------------------------------------------------------------------
//TImageMapTypeOptions

constructor TImageMapTypeOptions.Create;
begin
  inherited;
end;

constructor TImageMapTypeOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TImageMapTypeOptions.Destroy;
begin
  FTileSize.Free;
  inherited;
end;

function TImageMapTypeOptions.GetAlt: String;
begin
  Result:=GetPropAsString(ImageMapTypeOptionsProp_Alt);
end;

procedure TImageMapTypeOptions.SetAlt(Value: String);
begin
  SetProp(ImageMapTypeOptionsProp_Alt,Value,true);
end;

function TImageMapTypeOptions.GetIsPng: Boolean;
begin
  Result:=GetPropAsBoolean(ImageMapTypeOptionsProp_IsPng);
end;

procedure TImageMapTypeOptions.SetIsPng(Value: Boolean);
begin
  SetProp(ImageMapTypeOptionsProp_IsPng,Value,true);
end;

function TImageMapTypeOptions.GetMaxZoom: Integer;
begin
  Result:=GetPropAsInteger(ImageMapTypeOptionsProp_MaxZoom);
end;

procedure TImageMapTypeOptions.SetMaxZoom(Value: Integer);
begin
  SetProp(ImageMapTypeOptionsProp_MaxZoom,Value,true);
end;

function TImageMapTypeOptions.GetMinZoom: Integer;
begin
  Result:=GetPropAsInteger(ImageMapTypeOptionsProp_MinZoom);
end;

procedure TImageMapTypeOptions.SetMinZoom(Value: Integer);
begin
  SetProp(ImageMapTypeOptionsProp_MinZoom,Value,true);
end;

function TImageMapTypeOptions.GetName: String;
begin
  Result:=GetPropAsString(ImageMapTypeOptionsProp_Name);
end;

procedure TImageMapTypeOptions.SetName(Value: String);
begin
  SetProp(ImageMapTypeOptionsProp_Name,Value,true);
end;

function TImageMapTypeOptions.GetOpacity: Double;
begin
  Result:=GetPropAsDouble(ImageMapTypeOptionsProp_Opacity);
end;

procedure TImageMapTypeOptions.SetOpacity(Value: Double);
begin
  SetProp(ImageMapTypeOptionsProp_Opacity,Value,true);
end;

function TImageMapTypeOptions.GetRadius: Integer;
begin
  Result:=GetPropAsInteger(ImageMapTypeOptionsProp_Radius);
end;

procedure TImageMapTypeOptions.SetRadius(Value: Integer);
begin
  SetProp(ImageMapTypeOptionsProp_Radius,Value,true);
end;

function TImageMapTypeOptions.GetTileSize: TSize;
begin
  Result:=TSize(GetPropAsObject(ImageMapTypeOptionsProp_TileSize,TSize,FTileSize));
end;

procedure TImageMapTypeOptions.SetTileSize(Value: TSize);
begin
  SetProp(ImageMapTypeOptionsProp_TileSize,Value,true);
end;

//------------------------------------------------------------------------------
//TMapType

constructor TMapType.Create;
begin
  inherited;
end;

constructor TMapType.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TMapType.Destroy;
begin
  FGetTileFunction.Free;
  FProjection.Free;
  FReleaseTileFunction.Free;
  FTileSize.Free;
  inherited;
end;

procedure TMapType.AfterConstruction;
begin
  inherited;
  InitHandler;
end;

procedure TMapType.InitHandler;
begin
  FGetTileFunction:=Script.NewFunction(GetTileHandler);
  FReleaseTileFunction:=Script.NewFunction(ReleaseTileHandler);
  SetProp(MapTypeMethod_GetTile,FGetTileFunction,true);
  SetProp(MapTypeMethod_ReleaseTile,FReleaseTileFunction,true);
end;

class function TMapType.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_MVCObject,true);
end;

procedure TMapType.ReleaseTileHandler;
var
  Tile: TNode;
begin
  try
    Tile:=nil;
    with FReleaseTileFunction do
    if (Arguments.Count>=1) and VarIsValidDispatch(Arguments[0])
      then Tile:=TNode(CreateObjectWrapper(Script,TNode,Arguments[0]));
    try
      ReleaseTile(Tile);
    finally
      Tile.Free;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

function TMapType.GetAlt: String;
begin
  Result:=GetPropAsString(MapTypeProp_Alt);
end;

procedure TMapType.SetAlt(Value: String);
begin
  SetProp(MapTypeProp_Alt,Value,true);
end;

function TMapType.GetMaxZoom: Integer;
begin
  Result:=GetPropAsInteger(MapTypeProp_MaxZoom);
end;

procedure TMapType.SetMaxZoom(Value: Integer);
begin
  SetProp(MapTypeProp_MaxZoom,Value,true);
end;

function TMapType.GetMinZoom: Integer;
begin
  Result:=GetPropAsInteger(MapTypeProp_MinZoom);
end;

procedure TMapType.SetMinZoom(Value: Integer);
begin
  SetProp(MapTypeProp_MinZoom,Value,true);
end;

function TMapType.GetName: String;
begin
  Result:=GetPropAsString(MapTypeProp_Name);
end;

procedure TMapType.SetName(Value: String);
begin
  SetProp(MapTypeProp_Name,Value,true);
end;

function TMapType.GetProjection: TProjection;
begin
  Result:=TProjection(GetPropAsObject(MapTypeProp_Projection,TProjection,FProjection));
end;

procedure TMapType.SetProjection(Value: TProjection);
begin
  SetProp(MapTypeProp_Projection,Value,true);
end;

function TMapType.GetRadius: Integer;
begin
  Result:=GetPropAsInteger(MapTypeProp_Radius);
end;

procedure TMapType.SetRadius(Value: Integer);
begin
  SetProp(MapTypeProp_Radius,Value,true);
end;

function TMapType.GetTileHandler: Variant;
var
  OwnerDocument: TDocument;
  TileCoord: TPoint;
  Zoom: Integer;
  Node: TNode;
begin
  try
    OwnerDocument:=nil;
    TileCoord:=nil;
    Zoom:=0;
    with FGetTileFunction do
    begin
      if (Arguments.Count>=1) and VarIsValidDispatch(Arguments[0])
        then TileCoord:=TPoint(CreateObjectWrapper(Script,TPoint,Arguments[0]));
      if Arguments.Count>=2
        then Zoom:=Arguments[1];
      if (Arguments.Count>=3) and VarIsValidDispatch(Arguments[2])
        then OwnerDocument:=TDocument(CreateObjectWrapper(Script,TDocument,Arguments[2]));
    end;
    try
      Node:=GetTile(TileCoord,Zoom,OwnerDocument);
      if assigned(Node) then
      begin
        Result:=Node.Disp;
        Node.Free;
      end else Result:=Null;
    finally
      OwnerDocument.Free;
      TileCoord.Free;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

function TMapType.GetTileSize: TSize;
begin
  Result:=TSize(GetPropAsObject(MapTypeProp_TileSize,TSize,FTileSize));
end;

procedure TMapType.SetTileSize(Value: TSize);
begin
  SetProp(MapTypeProp_TileSize,Value,true);
end;

//------------------------------------------------------------------------------
//TMapTypeRegistry

constructor TMapTypeRegistry.Create;
begin
  inherited;
end;

constructor TMapTypeRegistry.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TMapTypeRegistry.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MVCObjectMethod_Set]);
end;

class function TMapTypeRegistry.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_MapTypeRegistry,true);
end;

procedure TMapTypeRegistry.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(ID: String; MapType: TMapType);
begin
  if not assigned(MapType)
    then Exec(MVCObjectMethod_Set,[ID,Null])
    else Exec(MVCObjectMethod_Set,[ID,MapType.Disp]);
end;

//------------------------------------------------------------------------------
//TMapTypeStyle

destructor TMapTypeStyle.Destroy;
begin
  FStylers.Free;
  inherited;
end;

function TMapTypeStyle.GetElementType: String;
begin
  Result:=GetPropAsString(MapTypeStyleProp_ElementType);
end;

procedure TMapTypeStyle.SetElementType(Value: String);
begin
  SetProp(MapTypeStyleProp_ElementType,Value,true);
end;

function TMapTypeStyle.GetFeatureType: String;
begin
  Result:=GetPropAsString(MapTypeStyleProp_FeatureType);
end;

procedure TMapTypeStyle.SetFeatureType(Value: String);
begin
  SetProp(MapTypeStyleProp_FeatureType,Value,true);
end;

function TMapTypeStyle.GetStylers: TMapTypeStylerArray;
begin
  Result:=TMapTypeStylerArray(GetPropAsObject(MapTypeStyleProp_Stylers,
                                              TMapTypeStylerArray,FStylers,true));
end;

procedure TMapTypeStyle.SetStylers(Value: TMapTypeStylerArray);
begin
  SetProp(MapTypeStyleProp_Stylers,Value,true);
end;

function TMapTypeStyle.AddStyler: TMapTypeStyler;
begin
  Result:=Stylers.Add;
end;

//------------------------------------------------------------------------------
//TMapTypeStyleArray

function TMapTypeStyleArray.AddStyle: TMapTypeStyle;
begin
  Result:=InsertStyle(Count);                              //add new TMapTypeStyle object
end;

function TMapTypeStyleArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TMapTypeStyle,Dispatch);
end;

function TMapTypeStyleArray.GetItem(Index: Integer): TMapTypeStyle;
begin
  Result:=TMapTypeStyle(inherited Items[Index]);
end;

procedure TMapTypeStyleArray.Insert(Index: Integer; Value: TMapTypeStyle);
begin
  inherited Insert(Index,Value);
end;

function TMapTypeStyleArray.InsertStyle(Index: Integer): TMapTypeStyle;
begin
  Result:=TMapTypeStyle.Create(Script);
  Insert(Index,Result);
end;

procedure TMapTypeStyleArray.SetItem(Index: Integer; Value: TMapTypeStyle);
begin
  inherited Items[Index]:=Value;
end;

//------------------------------------------------------------------------------
//TMapTypeStyler

function TMapTypeStyler.GetGamma: Double;
begin
  Result:=GetPropAsDouble(MapTypeStylerProp_Gamma);
end;

procedure TMapTypeStyler.SetGamma(Value: Double);
begin
  SetProp(MapTypeStylerProp_Gamma,Value,true);
end;

function TMapTypeStyler.GetHue: String;
begin
  Result:=GetPropAsString(MapTypeStylerProp_Hue);
end;

procedure TMapTypeStyler.SetHue(Value: String);
begin
  SetProp(MapTypeStylerProp_Hue,Value,true);
end;

function TMapTypeStyler.GetInvert_Lightness: Boolean;
begin
  Result:=GetPropAsBoolean(MapTypeStylerProp_InvertLightness);
end;

procedure TMapTypeStyler.SetInvert_Lightness(Value: Boolean);
begin
  SetProp(MapTypeStylerProp_InvertLightness,Value,true);
end;

function TMapTypeStyler.GetLightness: Double;
begin
  Result:=GetPropAsDouble(MapTypeStylerProp_Lightness);
end;

procedure TMapTypeStyler.SetLightness(Value: Double);
begin
  SetProp(MapTypeStylerProp_Lightness,Value,true);
end;

function TMapTypeStyler.GetSaturation: Double;
begin
  Result:=GetPropAsDouble(MapTypeStylerProp_Saturation);
end;

procedure TMapTypeStyler.SetSaturation(Value: Double);
begin
  SetProp(MapTypeStylerProp_Saturation,Value,true);
end;

function TMapTypeStyler.GetVisibility: String;
begin
  Result:=GetPropAsString(MapTypeStylerProp_Visibility);
end;

procedure TMapTypeStyler.SetVisibility(Value: String);
begin
  SetProp(MapTypeStylerProp_Visibility,Value,true);
end;

//------------------------------------------------------------------------------
//TMapTypeStylerArray

function TMapTypeStylerArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TMapTypeStyler.Create(Script,Dispatch);
end;

procedure TMapTypeStylerArray.SetItem(Index: Integer; Value: TMapTypeStyler);
begin
  inherited Items[Index]:=Value;
end;

function TMapTypeStylerArray.GetItem(Index: Integer): TMapTypeStyler;
begin
  Result:=TMapTypeStyler(inherited Items[Index]);
end;

function TMapTypeStylerArray.Add: TMapTypeStyler;
begin
  Result:=Insert(Count);
end;

function TMapTypeStylerArray.Insert(Index: Integer): TMapTypeStyler;
begin
  Result:=TMapTypeStyler.Create(Script);
  inherited Insert(Index,Result);
end;

//------------------------------------------------------------------------------
//TProjection

constructor TProjection.Create;
begin
  inherited;
end;

constructor TProjection.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TProjection.Destroy;
begin
  FFromLatLngToPointFunction.Free;
  FFromPointToLatLngFunction.Free;
  inherited;
end;

procedure TProjection.AfterConstruction;
begin
  inherited;
  FFromLatLngToPointFunction:=Script.NewFunction(FromLatLngToPointHandler);
  FFromPointToLatLngFunction:=Script.NewFunction(FromPointToLatLngHandler);
  SetProp(ProjectionMethod_FromLatLngToPoint,FFromLatLngToPointFunction,true);
  SetProp(ProjectionMethod_FromPointToLatLng,FFromPointToLatLngFunction,true);
end;

class function TProjection.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[ProjectionMethod_FromLatLngToPoint,
                                 ProjectionMethod_FromPointToLatLng]);
end;

function TProjection.FromLatLngToPointHandler: Variant;
var
  LatLng: TLatLng;
  ResultPoint, Point: TPoint;
begin
  try
    Result:=Null;
    LatLng:=nil;
    Point:=nil;
    with FFromLatLngToPointFunction do
    begin
      if (Arguments.Count>=1) and VarIsValidDispatch(Arguments[0])
        then LatLng:=TLatLng(CreateObjectWrapper(Script,TLatLng,Arguments[0]));
      if (Arguments.Count>=2) and VarIsValidDispatch(Arguments[1])
        then Point:=TPoint(CreateObjectWrapper(Script,TPoint,Arguments[1]));
    end;
    try
      ResultPoint:=FromLatLngToPoint(LatLng,Point);
      if assigned(ResultPoint) then
      begin
        Result:=ResultPoint.Disp;
        //ResultPoint.Free;
      end;
    finally
      //LatLng.Free;
      //Point.Free;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

function TProjection.FromPointToLatLngHandler: Variant;
var
  Pixel: TPoint;
  NoWrap: Boolean;
  ResultLatLng: TLatLng;
begin
  try
    Result:=Null;
    Pixel:=nil;
    NoWrap:=true;
    with FFromPointToLatLngFunction do
    begin
      if (Arguments.Count>=1) and VarIsValidDispatch(Arguments[0])
        then Pixel:=TPoint(CreateObjectWrapper(Script,TPoint,Arguments[0]));
      if Arguments.Count>=2
        then NoWrap:=VarAsBoolean(Arguments[1],true);
    end;
    try
      ResultLatLng:=FromPointToLatLng(Pixel,NoWrap);
      if assigned(ResultLatLng) then
      begin
        Result:=ResultLatLng.Disp;
        //ResultLatLng.Free;
      end;
    finally
      //Pixel.Free;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

//------------------------------------------------------------------------------
//TStyledMapType

constructor TStyledMapType.Create(Styles: TMapTypeStyleArray; Opts: TStyledMapTypeOptions);
begin
  Create(JScriptObjects.Script,Styles,Opts);
end;

constructor TStyledMapType.Create(Script: TCustomScript; Styles: TMapTypeStyleArray;
                                  Opts: TStyledMapTypeOptions);
begin
  inherited Create(Script,New(Script,Styles,Opts));
end;

function TStyledMapType.GetTile(TileCoord: TPoint; Zoom: Integer; OwnerDocument: TDocument): TNode;
begin
  Result:=nil;
end;

procedure TStyledMapType.ReleaseTile(Tile: TNode);
begin
end;
 
procedure TStyledMapType.InitHandler;
begin
  //Don't initialize GetTile and ReleaseTile functions.
  //They are used internally by Google Maps objects.
end;

class function TStyledMapType.New(Script: TCustomScript; Styles: TMapTypeStyleArray): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Styles)
    then raise Exception.Create(ExceptionNoStyles);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_StyledMapType,Styles.Disp,true);
end;

class function TStyledMapType.New(Script: TCustomScript; Styles: TMapTypeStyleArray;
                                  Opts: TStyledMapTypeOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Styles)
    then raise Exception.Create(ExceptionNoStyles);
  if not assigned(Opts)
    then Result:=New(Script,Styles)
    else Result:=BrowserTools.Create(TScript(SCript).Google.Maps.Disp,
                                     MapsMethod_StyledMapType,
                                     {$IFDEF CAN_OVERLOAD}
                                     [Styles.Disp,Opts.Disp]
                                     {$ELSE}
                                     Styles.Disp,Opts.Disp
                                     {$ENDIF}
                                     ,true);
end;

//------------------------------------------------------------------------------
//TStyledMapTypeOptions

constructor TStyledMapTypeOptions.Create;
begin
  inherited;
end;

constructor TStyledMapTypeOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TStyledMapTypeOptions.Destroy;
begin
  FBaseMapType.Free;
  inherited;
end;

function TStyledMapTypeOptions.GetAlt: String;
begin
  Result:=GetPropAsString(StyledMapTypeOptionsProp_Alt);
end;

procedure TStyledMapTypeOptions.SetAlt(Value: String);
begin
  SetProp(StyledMapTypeOptionsProp_Alt,Value,true);
end;

function TStyledMapTypeOptions.GetBaseMapType: TStyledMapType;
begin
  Result:=TStyledMapType(GetPropAsObject(StyledMapTypeOptionsProp_BaseMapType,
                                         TStyledMapType,FBaseMapType));
end;

procedure TStyledMapTypeOptions.SetBaseMapType(Value: TStyledMapType);
begin
  SetProp(StyledMapTypeOptionsProp_BaseMapType,Value,true);
end;

function TStyledMapTypeOptions.GetMaxZoom: Integer;
begin
  Result:=GetPropAsInteger(StyledMapTypeOptionsProp_MaxZoom);
end;

procedure TStyledMapTypeOptions.SetMaxZoom(Value: Integer);
begin
  SetProp(StyledMapTypeOptionsProp_MaxZoom,Value,true);
end;

function TStyledMapTypeOptions.GetMinZoom: Integer;
begin
  Result:=GetPropAsInteger(StyledMapTypeOptionsProp_MinZoom);
end;

procedure TStyledMapTypeOptions.SetMinZoom(Value: Integer);
begin
  SetProp(StyledMapTypeOptionsProp_MinZoom,Value,true);
end;

function TStyledMapTypeOptions.GetName: String;
begin
  Result:=GetPropAsString(StyledMapTypeOptionsProp_Name);
end;

procedure TStyledMapTypeOptions.SetName(Value: String);
begin
  SetProp(StyledMapTypeOptionsProp_Name,Value,true);
end;

end.
