unit gmServicesGeocoder;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, {$IFDEF DEBUG}Dialogs,{$ENDIF} SysUtils, BrowserTools, DispObject,
  JScriptObjects, gmBase, gmAdditionalConstants;

type
  TGeocoder = class;
  TGeocoderAddressComponent = class;
  TGeocoderAddressComponentArray = class;
  TGeocoderGeometry = class;
  TGeocoderLocationType = class;
  TGeocoderRequest = class;
  TGeocoderResult = class;
  TGeocoderResultArray = class;
  TGeocoderStatus = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TGeocoderCallback'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Result: </c>
  ///  <see cref="gmServices.Geocoder|TGeocoderResultArray"/><c>; Status: </c>
  ///  <see cref="String"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Geocoder"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.Geocoder|TGeocoder"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TGeocoderCallback = procedure(Sender: TObject; Result: TGeocoderResultArray; Status: String) of object;
  {$IFDEF USE_ANONYMOUS_METHODS}
  TGeocoderCallbackA = reference to procedure(Sender: TObject; Result: TGeocoderResultArray; Status: String);
  {$ENDIF}

  //A service for converting between an address and a LatLng.
  TGeocoder = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    //Geocodieren einer Anforderung:
    //Geocode a request:
    procedure Geocode(Request: TJScriptObject; Callback: TGeocoderCallback); overload;
    procedure Geocode(Request: TGeocoderRequest; Callback: TGeocoderCallback); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure Geocode(Request: TJScriptObject; Callback: TGeocoderCallbackA); overload;
    procedure Geocode(Request: TGeocoderRequest; Callback: TGeocoderCallbackA); overload;
    {$ENDIF}
  end;

  //A single address component within a GeocoderResult.
  //A full address may consist of multiple address components.
  TGeocoderAddressComponent = class(TCustomJScriptObject)
  private
    FTypes: TStringArray;
    function GetLongName: String;
    function GetShortName: String;
    function GetTypes: TStringArray;
  public
    destructor Destroy; override;
    //Der vollstndige Text der Adresskomponente:
    //The full text of the address component:
    property LongName: String read GetLongName;
    //Der abgekrzte Text der angegebenen Adresskomponente:
    //The abbreviated, short text of the given address component:
    property ShortName: String read GetShortName;
    //Ein Array mit Zeichenfolgen, die den Typ dieser Adresskomponente bezeichnen:
    //An array of strings denoting the type of this address component:
    property Types: TStringArray read GetTypes;
  end;

  //Array.<GeocoderAddressComponent>
  TGeocoderAddressComponentArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TGeocoderAddressComponent;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    property Items[Index: Integer]: TGeocoderAddressComponent read GetItem; default;
  end;

  //Geometry information about this GeocoderResult.
  TGeocoderGeometry = class(TCustomJScriptObject)
  private
    FBounds: TLatLngBounds;
    FLocation: TLatLng;
    FViewport: TLatLngBounds;
    function GetBounds: TLatLngBounds;
    function GetLocation: TLatLng;
    //Ist zwar im API als GeocoderLocationType definiert, ist aber in
    //Wirklichkeit kein Objekt, sondern ein String:
    function GetLocationType: String;
    function GetViewport: TLatLngBounds;
  public
    destructor Destroy; override;
    //Die genauen Grenzen dieses GeocodeResults, falls zutreffend:
    //The precise bounds of this GeocodeResult, if applicable:
    property Bounds: TLatLngBounds read GetBounds;
    //Die Breitengrad/Lngengrad-Koordinaten dieses Ergebnisses:
    //The latitude/longitude coordinates of this result:
    property Location: TLatLng read GetLocation;
    //Der Typ des in location zurckgegebenen Standorts:
    //The type of location returned in location:
    property LocationType: String read GetLocationType;
    //Die Grenzen des empfohlenen Darstellungsbereichs fr die Anzeige dieses GeocodeResults:
    //The bounds of the recommended viewport for displaying this GeocodeResult:
    property Viewport: TLatLngBounds read GetViewport;
  end;

  //Describes the type of location returned from a geocode.
  TGeocoderLocationType = class(TCustomJScriptObject)
  private
    procedure SetApproximate(Value: String);
    procedure SetGeometricCenter(Value: String);
    procedure SetRangeInterpolated(Value: String);
    procedure SetRooftop(Value: String);
    function GetApproximate: String;
    function GetGeometricCenter: String;
    function GetRangeInterpolated: String;
    function GetRooftop: String;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Das zurckgegebene Ergebnis ist ein Nherungswert:
    //The returned result is approximate:
    property Approximate: String read GetApproximate write SetApproximate;
    //Das zurckgegebene Ergebnis ist die geometrische Mitte eines Ergebnisses
    //wie einer Linie (zum Beispiel einer Strae) oder eines Polygons (einer Region):
    //The returned result is the geometric center of a result such a line
    //(e.g. street) or polygon (region):
    property GeometricCenter: String read GetGeometricCenter write SetGeometricCenter;
    //Das zurckgegebene Ergebnis stellt eine Nherung dar (normalerweise auf
    //einer Strae), die zwischen zwei przise lokalisierten Punkten (wie z.B.
    //Kreuzungen) interpoliert wurde. Interpolierte Ergebnisse werden
    //blicherweise dann zurckgegeben, wenn hausnummern-genaue Geocodierungen
    //fr eine Adresse in einer Strae nicht zur Verfgung stehen:
    //The returned result reflects an approximation (usually on a road) interpolated
    //between two precise points (such as intersections). Interpolated results are
    //generally returned when rooftop geocodes are unavilable for a street address:
    property RangeInterpolated: String read GetRangeInterpolated write SetRangeInterpolated;
    //Das zurckgegebene Ergebnis stellt eine przise Geocodierung dar:
    //The returned result reflects a precise geocode:
    property Rooftop: String read GetRooftop write SetRooftop;
  end;

  //The specification for a geocoding request to be sent to the Geocoder.
  TGeocoderRequest = class(TCustomJScriptObject)
  private
    FBounds: TLatLngBounds;
    FLatLng: TLatLng;
    FLocation: TLatLng;
    procedure SetAddress(Value: String);
    procedure SetBounds(Value: TLatLngBounds);
    //procedure SetLanguage(Value: String);
    //procedure SetLatLng(Value: TLatLng);
    procedure SetLocation(Value: TLatLng);
    procedure SetRegion(Value: String);
    function GetAddress: String;
    function GetBounds: TLatLngBounds;
    //function GetLanguage: String;
    //function GetLatLng: TLatLng;
    function GetLocation: TLatLng;
    function GetRegion: String;
  public
    constructor Create; overload; override;
    constructor Create(Address: String); overload;
    constructor Create(Script: TCustomScript); overload; override;
    constructor Create(Script: TCustomScript; Address: String); overload;
    destructor Destroy; override;
    //Adresse. Optional:
    //Address. Optional:
    property Address: String read GetAddress write SetAddress;
    //LatLngBounds, innerhalb derer gesucht werden soll. Optional:
    //LatLngBounds within which to search. Optional:
    property Bounds: TLatLngBounds read GetBounds write SetBounds;
    //Bevorzugte Sprache fr Ergebnisse. Optional:
    //Removed V3.5 June 8, 2011
    //property Language: String read GetLanguage write SetLanguage; deprecated;
    //Koordinaten fr Reverse Geocoding. Optional:
    //property LatLng: TLatLng read GetLatLng write SetLatLng;
    //LatLng, bei der gesucht werden soll. Optional:
    //LatLng about which to search. Optional:
    property Location: TLatLng read GetLocation write SetLocation;
    //Domne hchster Ebene mit Lndercode, in der gesucht werden soll. Optional:
    //Country code top-level domain within which to search. Optional:
    property Region: String read GetRegion write SetRegion;
  end;

  //A single geocoder result retrieved from the geocode server. A geocode request
  //may return multiple result objects. Note that though this result is "JSON-like,"
  //it is not strictly JSON, as it indirectly includes a LatLng object.
  TGeocoderResult = class(TCustomJScriptObject)
  private
    FAddressComponents: TGeocoderAddressComponentArray;
    FGeometry: TGeocoderGeometry;
    FTypes: TStringArray;
    function GetAddressComponents: TGeocoderAddressComponentArray;
    function GetFormattedAddress: String;
    function GetGeometry: TGeocoderGeometry;
    function GetTypes: TStringArray;
  public
    destructor Destroy; override;
    //Ein Array mit GeocoderAddressComponent-Werten:
    //An array of GeocoderAddressComponents:
    property AddressComponents: TGeocoderAddressComponentArray read GetAddressComponents;
    //A string containing the human-readable address of this location.
    property FormattedAddress: String read GetFormattedAddress;
    //Ein GeocoderGeometry-Objekt:
    //A GeocoderGeometry object:
    property Geometry: TGeocoderGeometry read GetGeometry;
    //Ein Array mit Zeichenfolgen, die die Typen der zurckgegebenen geocodierten
    //Elemente bezeichnen. Ein Typ besteht aus einer eindeutigen Zeichenfolge,
    //anhand derer das Geocode-Ergebnis identifiziert wird. Beispiel:
    //"administrative_area_level_1", "country" und so weiter:
    //An array of strings denoting the type of the returned geocoded element.
    //A type consists of a unique string identifying the geocode result.
    //(For example, "administrative_area_level_1", "country", etc.):
    property Types: TStringArray read GetTypes;
  end;

  //Array.<GeocoderResult>
  TGeocoderResultArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TGeocoderResult;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    property Items[Index: Integer]: TGeocoderResult read GetItem; default;
  end;

  //The status returned by the Geocoder on the completion of a call to geocode().
  TGeocoderStatus = class(TCustomJScriptObject)
  private
    procedure SetError(Value: String);
    procedure SetInvalidRequest(Value: String);
    procedure SetOK(Value: String);
    procedure SetOverQueryLimit(Value: String);
    procedure SetRequestDenied(Value: String);
    procedure SetUnknownError(Value: String);
    procedure SetZeroResults(Value: String);
    function GetError: String;
    function GetInvalidRequest: String;
    function GetOK: String;
    function GetOverQueryLimit: String;
    function GetRequestDenied: String;
    function GetUnknownError: String;
    function GetZeroResults: String;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Beim Aufbau der Verbindung zu den Google-Servern ist ein Problem aufgetreten:
    //There was a problem contacting the Google servers:
    property Error: String read GetError write SetError;
    //Diese GeocoderRequest war ungltig:
    //This GeocoderRequest was invalid:
    property InvalidRequest: String read GetInvalidRequest write SetInvalidRequest;
    //Die Antwort enthlt eine gltige GeocoderResponse:
    //The response contains a valid GeocoderResponse:
    property OK: String read GetOK write SetOK;
    //Die Anfragebeschrnkungen wurden von der Webseite in einem zu geringen
    //Zeitraum berschritten:
    //The webpage has gone over the requests limit in too short a period of time:
    property OverQueryLimit: String read GetOverQueryLimit write SetOverQueryLimit;
    //Es ist der Webpage nicht gestattet, den Geocoder zu verwenden:
    //The webpage is not allowed to use the geocoder:
    property RequestDenied: String read GetRequestDenied write SetRequestDenied;
    //Eine Geocodierungsanfrage konnte aufgrund eines Serverfehlers nicht
    //verarbeitet werden. Die Anfrage ist mglicherweise erfolgreich, wenn Sie
    //es erneut versuchen:
    //A geocoding request could not be processed due to a server error.
    //The request may succeed if you try again:
    property UnknownError: String read GetUnknownError write SetUnknownError;
    //Fr diese GeocoderRequest wurde kein Ergebnis gefunden:
    //No result was found for this GeocoderRequest:
    property ZeroResults: String read GetZeroResults write SetZeroResults;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TGeocoder

constructor TGeocoder.Create;
begin
  inherited;
end;

constructor TGeocoder.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TGeocoder.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Geocoder,true);
end;

type
  TGeocoderCallbackObject = class(TCallbackObject)
  private
    FCallback: TGeocoderCallback;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FCallbackA: TGeocoderCallbackA;
    {$ENDIF}
    FGeocoder: TGeocoder;
  protected
    procedure Callback; override;
  public
    constructor Create(Geocoder: TGeocoder; Request: TJScriptObject); reintroduce; overload;
    constructor Create(Geocoder: TGeocoder; Request: TJScriptObject; Callback: TGeocoderCallback); reintroduce; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    constructor Create(Geocoder: TGeocoder; Request: TJScriptObject; Callback: TGeocoderCallbackA); reintroduce; overload;
    {$ENDIF}
  end;

constructor TGeocoderCallbackObject.Create(Geocoder: TGeocoder; Request: TJScriptObject);
begin
  inherited Create(Geocoder.Script);
  FGeocoder:=Geocoder;
  FGeocoder.Exec(GeocoderMethod_Geocode,[Request.Disp,JScriptFunction.Disp]);
end;

constructor TGeocoderCallbackObject.Create(Geocoder: TGeocoder; Request: TJScriptObject; Callback: TGeocoderCallback);
begin
  FCallback:=Callback;
  Create(Geocoder,Request);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
constructor TGeocoderCallbackObject.Create(Geocoder: TGeocoder; Request: TJScriptObject; Callback: TGeocoderCallbackA);
begin
  FCallbackA:=Callback;
  Create(Geocoder,Request);
end;
{$ENDIF}

procedure TGeocoderCallbackObject.Callback;
var
  GeocoderResult: TGeocoderResultArray;
begin
  try
    if ArgumentCount>=2 then
    begin
      GeocoderResult:=nil;
      if VarIsValidDispatch(Arguments[0])
        then GeocoderResult:=TGeocoderResultArray(CreateObjectWrapper(Script,
                                                                      TGeocoderResultArray,
                                                                      Arguments[0]));
      try
        {$IFDEF USE_ANONYMOUS_METHODS}
        if assigned(FCallbackA)
          then FCallbackA(FGeocoder,GeocoderResult,Arguments[1])
          else
        {$ENDIF}
        if assigned(FCallback)
          then FCallback(FGeocoder,GeocoderResult,Arguments[1]);
      finally
        //GeocoderResult.Free;    -> to save it for future use in anonymous methods
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TGeocoder.Geocode(Request: TGeocoderRequest; Callback: TGeocoderCallback);
begin
  Geocode(TJScriptObject(Request),Callback);
end;

procedure TGeocoder.Geocode(Request: TJScriptObject; Callback: TGeocoderCallback);
begin
  if assigned(Request) and assigned(Callback)
    then TGeocoderCallbackObject.Create(Self,Request,Callback);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TGeocoder.Geocode(Request: TGeocoderRequest; Callback: TGeocoderCallbackA);
begin
  Geocode(TJScriptObject(Request),Callback);
end;

procedure TGeocoder.Geocode(Request: TJScriptObject; Callback: TGeocoderCallbackA);
begin
  if assigned(Request) and assigned(Callback)
    then TGeocoderCallbackObject.Create(Self,Request,Callback);
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TGeocoderAddressComponent

destructor TGeocoderAddressComponent.Destroy;
begin
  FTypes.Free;
  inherited;
end;

function TGeocoderAddressComponent.GetLongName: String;
begin
  Result:=GetPropAsString(GeocoderAddressComponentProp_LongName);
end;

function TGeocoderAddressComponent.GetShortName: String;
begin
  Result:=GetPropAsString(GeocoderAddressComponentProp_ShortName);
end;

function TGeocoderAddressComponent.GetTypes: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(GeocoderAddressComponentProp_Types,TStringArray,FTypes));
end;

//------------------------------------------------------------------------------
//TGeocoderAddressComponentArray

function TGeocoderAddressComponentArray.GetItem(Index: Integer): TGeocoderAddressComponent;
begin
  Result:=TGeocoderAddressComponent(inherited Items[Index]);
end;

function TGeocoderAddressComponentArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TGeocoderAddressComponent.Create(Script,Dispatch);
end;

//------------------------------------------------------------------------------
//TGeocoderGeometry

destructor TGeocoderGeometry.Destroy;
begin
  FBounds.Free;
  FLocation.Free;
  FViewport.Free;
  inherited;
end;

function TGeocoderGeometry.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(GeocoderGeometryProp_Bounds,TLatLngBounds,FBounds));
end;

function TGeocoderGeometry.GetLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(GeocoderGeometryProp_Location,TLatLng,FLocation));
end;

function TGeocoderGeometry.GetLocationType: String;
begin
  Result:=GetPropAsString(GeocoderGeometryProp_LocationType);
end;

function TGeocoderGeometry.GetViewport: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(GeocoderGeometryProp_Viewport,TLatLngBounds,FViewport));
end;

//------------------------------------------------------------------------------
//TGeocoderLocationType

class function TGeocoderLocationType.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[GeocoderLocationTypeProp_Approximate,
                                 GeocoderLocationTypeProp_GeometricCenter,
                                 GeocoderLocationTypeProp_RangeInterpolated]);
end;

function TGeocoderLocationType.GetApproximate: String;
begin
  Result:=GetPropAsString(GeocoderLocationTypeProp_Approximate);
end;

procedure TGeocoderLocationType.SetApproximate(Value: String);
begin
  SetProp(GeocoderLocationTypeProp_Approximate,Value);
end;

function TGeocoderLocationType.GetGeometricCenter: String;
begin
  Result:=GetPropAsString(GeocoderLocationTypeProp_GeometricCenter);
end;

procedure TGeocoderLocationType.SetGeometricCenter(Value: String);
begin
  SetProp(GeocoderLocationTypeProp_GeometricCenter,Value);
end;

function TGeocoderLocationType.GetRangeInterpolated: String;
begin
  Result:=GetPropAsString(GeocoderLocationTypeProp_RangeInterpolated);
end;

procedure TGeocoderLocationType.SetRangeInterpolated(Value: String);
begin
  SetProp(GeocoderLocationTypeProp_RangeInterpolated,Value);
end;

function TGeocoderLocationType.GetRooftop: String;
begin
  Result:=GetPropAsString(GeocoderLocationTypeProp_RoofTop);
end;

procedure TGeocoderLocationType.SetRooftop(Value: String);
begin
  SetProp(GeocoderLocationTypeProp_RoofTop,Value);
end;

//------------------------------------------------------------------------------
//TGeocoderRequest

constructor TGeocoderRequest.Create;
begin
  inherited;
end;

constructor TGeocoderRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

constructor TGeocoderRequest.Create(Address: String);
begin
  Create(JScriptObjects.Script,Address);
end;

constructor TGeocoderRequest.Create(Script: TCustomScript; Address: String);
begin
  inherited Create(Script);
  SetAddress(Address);
end;

destructor TGeocoderRequest.Destroy;
begin
  FBounds.Free;
  FLatLng.Free;
  FLocation.Free;
  inherited;
end;

function TGeocoderRequest.GetAddress: String;
begin
  Result:=GetPropAsString(GeocoderRequestProp_Address);
end;

procedure TGeocoderRequest.SetAddress(Value: String);
begin
  SetProp(GeocoderRequestProp_Address,Value,true);
end;

function TGeocoderRequest.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(GeocoderRequestProp_Bounds,TLatLngBounds,FBounds));
end;

procedure TGeocoderRequest.SetBounds(Value: TLatLngBounds);
begin
  SetProp(GeocoderRequestProp_Bounds,Value,true);
end;
{
function TGeocoderRequest.GetLanguage: String;
begin
  Result:=GetPropAsString(GeocoderRequestProp_Language);
end;

procedure TGeocoderRequest.SetLanguage(Value: String);
begin
  SetProp(GeocoderRequestProp_Language,Value,true);
end;

function TGeocoderRequest.GetLatLng: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(GeocoderRequestProp_LatLng,TLatLng,FLatLng));
end;

procedure TGeocoderRequest.SetLatLng(Value: TLatLng);
begin
  SetProp(GeocoderRequestProp_LatLng,Value,true);
end;
}
function TGeocoderRequest.GetLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(GeocoderRequestProp_Location,TLatLng,FLocation));
end;

procedure TGeocoderRequest.SetLocation(Value: TLatLng);
begin
  SetProp(GeocoderRequestProp_Location,Value,true);
end;

function TGeocoderRequest.GetRegion: String;
begin
  Result:=GetPropAsString(GeocoderRequestProp_Region);
end;

procedure TGeocoderRequest.SetRegion(Value: String);
begin
  SetProp(GeocoderRequestProp_Region,Value,true);
end;

//------------------------------------------------------------------------------
//TGeocoderResult

destructor TGeocoderResult.Destroy;
begin
  FAddressComponents.Free;
  FGeometry.Free;
  FTypes.Free;
  inherited;
end;

function TGeocoderResult.GetAddressComponents: TGeocoderAddressComponentArray;
begin
  Result:=TGeocoderAddressComponentArray(GetPropAsObject(GeocoderResultProp_AddressComponents,
                                                         TGeocoderAddressComponentArray,FAddressComponents));
end;

function TGeocoderResult.GetFormattedAddress: String;
begin
  Result:=GetPropAsString(GeocoderResultProp_FormattedAddress);
end;

function TGeocoderResult.GetGeometry: TGeocoderGeometry;
begin
  Result:=TGeocoderGeometry(GetPropAsObject(GeocoderResultProp_Geometry,TGeocoderGeometry,FGeometry));
end;

function TGeocoderResult.GetTypes: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(GeocoderResultProp_Types,TStringArray,FTypes));
end;

//------------------------------------------------------------------------------
//TGeocoderResultArray

function TGeocoderResultArray.GetItem(Index: Integer): TGeocoderResult;
begin
  Result:=TGeocoderResult(inherited Items[Index]);
end;

function TGeocoderResultArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TGeocoderResult.Create(Script,Dispatch);
end;

//------------------------------------------------------------------------------
//TGeocoderStatus

class function TGeocoderStatus.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[GeocoderStatusProp_Error,
                                 GeocoderStatusProp_InvalidRequest,
                                 GeocoderStatusProp_Ok]);
end;

function TGeocoderStatus.GetError: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_Error);
end;

procedure TGeocoderStatus.SetError(Value: String);
begin
  SetProp(GeocoderStatusProp_Error,Value);
end;

function TGeocoderStatus.GetInvalidRequest: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_InvalidRequest);
end;

procedure TGeocoderStatus.SetInvalidRequest(Value: String);
begin
  SetProp(GeocoderStatusProp_InvalidRequest,Value);
end;

function TGeocoderStatus.GetOK: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_OK);
end;

procedure TGeocoderStatus.SetOK(Value: String);
begin
  SetProp(GeocoderStatusProp_OK,Value);
end;

function TGeocoderStatus.GetOverQueryLimit: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_OverQueryLimit);
end;

procedure TGeocoderStatus.SetOverQueryLimit(Value: String);
begin
  SetProp(GeocoderStatusProp_OverQueryLimit,Value);
end;

function TGeocoderStatus.GetRequestDenied: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_RequestDenied);
end;

procedure TGeocoderStatus.SetRequestDenied(Value: String);
begin
  SetProp(GeocoderStatusProp_RequestDenied,Value);
end;

function TGeocoderStatus.GetUnknownError: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_UnknownError);
end;

procedure TGeocoderStatus.SetUnknownError(Value: String);
begin
  SetProp(GeocoderStatusProp_UnknownError,Value);
end;

function TGeocoderStatus.GetZeroResults: String;
begin
  Result:=GetPropAsString(GeocoderStatusProp_ZeroResults);
end;

procedure TGeocoderStatus.SetZeroResults(Value: String);
begin
  SetProp(GeocoderStatusProp_ZeroResults,Value);
end;

end.


