unit gmStreetView;

interface

{$INCLUDE gmConfig.inc}

uses
  Controls, Classes, Graphics, SysUtils, {$IFDEF DEBUG} Dialogs,{$ENDIF}
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} BrowserTools, JScriptObjects,
  HTMLObjects, DispObject, gmBase, gmMVC, gmControls, gmEvents,
  gmAdditionalConstants, gmAdditionalMessages;

type
  TStreetViewAddressControlOptions = class;
  TStreetViewLinkArray = class;
  TStreetViewLink = class;
  TStreetViewLocation = class;
  TStreetViewPanorama = class;
  TStreetViewPanoramaData = class;
  TStreetViewPanoramaOptions = class;
  TStreetViewPov = class;
  TStreetViewService = class;
  TStreetViewStatus = class;
  TStreetViewTileData = class;

  //Options for the rendering of the Street View address control.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewAddressControlOptions
  TStreetViewAddressControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    function GetPosition: Integer;
  public
    //Position id. This id is used to specify the position of the control on
    //the map. The default position is <c>TopLeft</c>.
    property Position: Integer read GetPosition write SetPosition;
  end;

  //Array.<StreetViewLink>
  TStreetViewLinkArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TStreetViewLink);
    function GetItem(Index: Integer): TStreetViewLink;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste an der Position <i>Index</i> ein
    ///  <see cref="gmStreetView|TStreetViewLink"> TStreetViewLink</see>-Objekt
    ///  hinzu.<br/><br/>
    ///  Add a new <see cref="gmStreetView|TStreetViewLink">TStreetViewLink</see>
    ///  object at position <i>Index</i>.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; StreetViewLink: TStreetViewLink); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste am Ende ein neu erstelltes
    ///  <see cref="gmStreetView|TStreetViewLink">TStreetViewLink</see>-Objekt
    ///  hinzu.
    ///</summary>
    ///<returns>
    ///  Index des hinzugefgten <see cref="gmStreetView|TStreetViewLink"></see>-
    ///  Objektes.<br/><br/>
    ///  Index of added <see cref="gmStreetView|TStreetViewLink">TStreetViewLink</see>
    ///  object.<br/><br/>
    ///</returns>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(StreetViewLink: TStreetViewLink): Integer; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Liste mit den enthaltenen <see cref="gmStreetView|TStreetViewLink">
    ///  TStreetViewLink</see>-Objekten.<br/><br/>
    ///  List of all contained <see cref="gmStreetView|TStreetViewLink">
    ///  TStreetViewLink</see> objects.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TStreetViewLink read GetItem write SetItem; default;
  end;

  //A collection of references to adjacent Street View panos.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewLink
  TStreetViewLink = class(TCustomJScriptObject)
  private
    procedure SetDescription(Value: String);
    procedure SetHeading(Value: Double);
    procedure SetPano(Value: String);
    procedure SetRoadColor(Value: TColor);
    procedure SetRoadOpacity(Value: Double);
    function GetDescription: String;
    function GetHeading: Double;
    function GetPano: String;
    function GetRoadColor: TColor;
    function GetRoadOpacity: Double;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    //A localized string describing the link.
    property Description: String read GetDescription write SetDescription;
    //The heading of the link.
    property Heading: Double read GetHeading write SetHeading;
    //A unique identifier for the panorama. This id is stable within a
    //session but unstable across sessions.
    property Pano: String read GetPano write SetPano;
    //Color of the link.
    property RoadColor: TColor read GetRoadColor write SetRoadColor;
    //Opacity of the Link.
    property RoadOpacity: Double read GetRoadOpacity write SetRoadOpacity;
  end;

  //A representation of a location in the Street View panorama.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewLocation
  TStreetViewLocation = class(TCustomJScriptObject)
  private
    FLatLng: TLatLng;
    procedure SetDescription(Value: String);
    procedure SetLatLng(Value: TLatLng);
    procedure SetPano(Value: String);
    function GetDescription: String;
    function GetLatLng: TLatLng;
    function GetPano: String;
  public
    destructor Destroy; override;
    //A localized string describing the location.
    property Description: String read GetDescription write SetDescription;
    //The latlng of the panorama.
    property LatLng: TLatLng read GetLatLng write SetLatLng;
    //A unique identifier for the panorama. This is stable within a session
    //but unstable across sessions.
    property Pano: String read GetPano write SetPano;
  end;

  TPanoProvider = function(Sender: TObject; Pano: String): TStreetViewPanoramaData of object;
  {$IFDEF USE_ANONYMOUS_METHODS}
  TPanoProviderA = reference to function(Sender: TObject; Pano: String): TStreetViewPanoramaData;
  {$ENDIF}

  //Displays the panorama for a given LatLng or panorama ID. A
  //StreetViewPanorama object provides a Street View "viewer" which can be
  //stand-alone within a separate <div> or bound to a Map. This class
  //extends MVCObject.
  TStreetViewPanorama = class(TMVCObject)
  private
    FContainer: TNode;
    FControls: TNodeMVCArrayArray;
    FLatLng: TLatLng;
    FLinks: TJScriptArray;
    FPanoProvider: TPanoProvider;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FPanoProviderA: TPanoProviderA;
    {$ENDIF}
    FPanoProviderFunction: TJScriptFunction;
    FPosition: TLatLng;
    FPov: TStreetViewPov;
    FOnCloseClick: TMapEventEvent;
    FOnCloseClickListener: TMapsEventListener;
    FOnCloseClickFunction: TJScriptFunction;
    FOnLinksChanged: TNotifyEvent;
    FOnLinksChangedListener: TMapsEventListener;
    FOnLinksChangedFunction: TJScriptFunction;
    FOnPanoChanged: TNotifyEvent;
    FOnPanoChangedListener: TMapsEventListener;
    FOnPanoChangedFunction: TJScriptFunction;
    FOnPositionChanged: TNotifyEvent;
    FOnPositionChangedListener: TMapsEventListener;
    FOnPositionChangedFunction: TJScriptFunction;
    FOnPovChanged: TNotifyEvent;
    FOnPovChangedListener: TMapsEventListener;
    FOnPovChangedFunction: TJScriptFunction;
    FOnResize: TNotifyEvent;
    FOnResizeListener: TMapsEventListener;
    FOnResizeFunction: TJScriptFunction;
    FOnVisibleChanged: TNotifyEvent;
    FOnVisibleChangedListener: TMapsEventListener;
    FOnVisibleChangedFunction: TJScriptFunction;
    procedure SetAlign(Value: TAlign);
    procedure SetControls(Value: TNodeMVCArrayArray);
    procedure SetHeight(Value: Integer);
    procedure SetLeft(Value: Integer);
    procedure SetOnCloseClick(Value: TMapEventEvent);
    procedure SetOnLinksChanged(Value: TNotifyEvent);
    procedure SetOnPanoChanged(Value: TNotifyEvent);
    procedure SetOnPositionChanged(Value: TNotifyEvent);
    procedure SetOnPovChanged(Value: TNotifyEvent);
    procedure SetOnResize(Value: TNotifyEvent);
    procedure SetOnVisibleChanged(Value: TNotifyEvent);
    procedure SetTop(Value: Integer);
    procedure SetWidth(Value: Integer);
    procedure UpdateOptions(Script: TCustomScript; Opts: TStreetViewPanoramaOptions);
    function GetAlign: TAlign;
    function GetControls:TNodeMVCArrayArray;
    function GetHeight: Integer;
    function GetLeft: Integer;
    function GetTop: Integer;
    function GetWidth: Integer;
  protected
    class function New(Script: TCustomScript; Node: TNode): IDispatch; reintroduce; overload;
    class function New(Script: TCustomScript; Node: TNode; Opts: TStreetViewPanoramaOptions): IDispatch; reintroduce; overload;
    procedure Visible_changed;
    function N: OleVariant;
    //JScript event handler:
    procedure CloseClick;
    procedure LinksChanged;
    procedure PanoChanged;
    procedure PositionChanged;
    procedure PovChanged;
    procedure Resize;
    procedure VisibleChanged;
    function PanoProviderCallback: Variant;
  public
    constructor Create(Opts: TStreetViewPanoramaOptions = nil); reintroduce; overload;
    constructor Create(Container: TNode; Opts: TStreetViewPanoramaOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TStreetViewPanoramaOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Container: TNode; Opts: TStreetViewPanoramaOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Set the custom panorama provider called on pano change to load custom
    //panoramas.
    procedure RegisterPanoProvider(PanoProvider: TPanoProvider); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure RegisterPanoProvider(PanoProvider: TPanoProviderA); overload;
    {$ENDIF}
    //Deprecated.
    procedure SetEnableCloseButton(Flag: Boolean); {$IFDEF USE_DEPRECATED}deprecated;{$ENDIF}
    procedure SetOptions(Options: TStreetViewPanoramaOptions); reintroduce;
    //Sets the current panorama ID for the Street View panorama.
    procedure SetPano(Pano: String);
    //Sets the current LatLng position for the Street View panorama.
    procedure SetPosition(LatLng: TLatLng);
    //Sets the point of view for the Street View panorama.
    procedure SetPov(Pov: TStreetViewPov);
    //Sets to true to make the panorama visible. If set to false, the
    //panorama will be hidden whether it is embedded in the map or in its own
    //<div>.
    procedure SetVisible(Flag: Boolean);
    //Deprecated.
    function GetEnableCloseButton: Boolean; {$IFDEF USE_DEPRECATED}deprecated;{$ENDIF}
    //Returns the set of navigation links for the Street View panorama.
    function GetLinks: TStreetViewLinkArray;
    //Returns the current panorama ID for the Street View panorama. This id is
    //stable within the browser's current session only.
    function GetPano: String;
    //Returns the current LatLng position for the Street View panorama.
    function GetPosition: TLatLng;
    //Returns the current point of view for the Street View panorama.
    function GetPov: TStreetViewPov;
    //Returns true if the panorama is visible. It does not specify whether
    //Street View imagery is available at the specified position.
    function GetVisible: Boolean;
    property Align: TAlign read GetAlign write SetAlign;
    property Container: TNode read FContainer;
    property Controls: TNodeMVCArrayArray read GetControls write SetControls;
    //property EnableCloseButton: Boolean read GetEnableCloseButton write SetEnableCloseButton;
    property Height: Integer read GetHeight write SetHeight;
    property Left: Integer read GetLeft write SetLeft;
    property Links: TStreetViewLinkArray read GetLinks;
    property Pano: String read GetPano write SetPano;
    property Position: TLatLng read GetPosition write SetPosition;
    property Pov: TStreetViewPov read GetPov write SetPov;
    property Top: Integer read GetTop write SetTop;
    property Visible: Boolean read GetVisible write SetVisible;
    property Width: Integer read GetWidth write SetWidth;
    //This event is fired when the close button is clicked.
    property OnCloseClick: TMapEventEvent read FOnCloseClick write SetOnCloseClick;
    //This event is fired when the panorama's links change. The links change
    //asynchronously following a pano id change.
    property OnLinksChanged: TNotifyEvent read FOnLinksChanged write SetOnLinksChanged;
    //This event is fired when the panorama's pano id changes. The pano may
    //change as the user navigates through the panorama or the position is
    //manually set. Note that not all position changes trigger a OnPanoChanged.
    property OnPanoChanged: TNotifyEvent read FOnPanoChanged write SetOnPanoChanged;
    //This event is fired when the panorama's position changes. The position
    //changes as the user navigates through the panorama or the position is
    //set manually.
    property OnPositionChanged: TNotifyEvent read FOnPositionChanged write SetOnPositionChanged;
    //This event is fired when the panorama's point-of-view changes. The
    //point of view changes as the pitch, zoom, or heading changes.
    property OnPovChanged: TNotifyEvent read FOnPovChanged write SetOnPovChanged;
    //Developers should trigger this event on the panorama when its div
    //changes size: Google.Maps.Event.Trigger(Panorama'resize').
    property OnResize: TNotifyEvent read FOnResize write SetOnResize;
    //This event is fired when the panorama's visibility changes. The
    //visibility is changed when the Pegman id dragged onto the map, the
    //close button is clicked, or SetVisible() is called.
    property OnVisibleChanged: TNotifyEvent read FOnVisibleChanged write SetOnVisibleChanged;
  end;

  //The representation of a panorama returned from the provider defined using
  //registerPanoProvider.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewPanoramaData
  TStreetViewPanoramaData = class(TCustomJScriptObject)
  private
    FLinks: TStreetViewLinkArray;
    FLocation: TStreetViewLocation;
    FTiles: TStreetViewTileData;
    procedure SetCopyright(Value: String);
    procedure SetLinks(Value: TStreetViewLinkArray);
    procedure SetLocation(Value: TStreetViewLocation);
    procedure SetTiles(Value: TStreetViewTileData);
    function GetCopyright: String;
    function GetLinks: TStreetViewLinkArray;
    function GetLocation: TStreetViewLocation;
    function GetTiles: TStreetViewTileData;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //Specifies the copyright text for this panorama.
    property Copyright: String read GetCopyright write SetCopyright;
    //Specifies the navigational links to adjacent panoramas.
    property Links: TStreetViewLinkArray read GetLinks write SetLinks;
    //Specifies the location meta-data for this panorama.
    property Location: TStreetViewLocation read GetLocation write SetLocation;
    //Specifies the custom tiles for this panorama.
    property Tiles: TStreetViewTileData read GetTiles write SetTiles;
  end;

  //Options defining the properties of a StreetViewPanorama object.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewPanoramaOptions
  TStreetViewPanoramaOptions = class(TCustomJScriptObject)
  private
    FAddressControlOptions: TStreetViewAddressControlOptions;
    FAlign: TAlign;
    FHeight: Integer;
    FLeft: Integer;
    FPanControlOptions: TPanControlOptions;
    FPanoProvider: TPanoProvider;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FPanoProviderA: TPanoProviderA;
    {$ENDIF}
    FPosition: TLatLng;
    FPov: TStreetViewPov;
    FTop: Integer;
    FWidth: Integer;
    FZoomControlOptions: TZoomControlOptions;
    procedure SetAddressControl(Value: Boolean);
    procedure SetAddressControlOptions(Value: TStreetViewAddressControlOptions);
    procedure SetDisableDoubleClickZoom(Value: Boolean);
    procedure SetEnableCloseButton(Value: Boolean);
    procedure SetLinksControl(Value: Boolean);
    procedure SetPanControl(Value: Boolean);
    procedure SetPanControlOptions(Value: TPanControlOptions);
    procedure SetPano(Value: String);
    procedure SetPosition(Value: TLatLng);
    procedure SetPov(Value: TStreetViewPov);
    procedure SetScrollWheel(Value: Boolean);
    procedure SetVisible(Value: Boolean);
    procedure SetZoomControl(Value: Boolean);
    procedure SetZoomControlOptions(Value: TZoomControlOptions);
    function GetAddressControl: Boolean;
    function GetAddressControlOptions: TStreetViewAddressControlOptions;
    function GetDisableDoubleClickZoom: Boolean;
    function GetEnableCloseButton: Boolean;
    function GetLinksControl: Boolean;
    function GetPanControl: Boolean;
    function GetPanControlOptions: TPanControlOptions;
    function GetPano: String;
    function GetPosition: TLatLng;
    function GetPov: TStreetViewPov;
    function GetScrollWheel: Boolean;
    function GetVisible: Boolean;
    function GetZoomControl: Boolean;
    function GetZoomControlOptions: TZoomControlOptions;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AddressControl'}{$ENDIF}
    ///<summary>
    ///  The enabled/disabled state of the address control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AddressControl: Boolean read GetAddressControl write SetAddressControl;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AddressControlOptions'}{$ENDIF}
    ///<summary>
    ///  The display options for the address control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AddressControlOptions: TStreetViewAddressControlOptions read GetAddressControlOptions write SetAddressControlOptions;
    property Align: TAlign read FAlign write FAlign;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DisableDoubleClickZoom'}{$ENDIF}
    ///<summary>
    ///  Enables/disables zoom on double click. Enabled by default.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DisableDoubleClickZoom: Boolean read GetDisableDoubleClickZoom write SetDisableDoubleClickZoom;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation EnableCloseButton'}{$ENDIF}
    ///<summary>
    ///  If true, the close button is displayed. Disabled by default.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property EnableCloseButton: Boolean read GetEnableCloseButton write SetEnableCloseButton;
    property Height: Integer read FHeight write FHeight;
    property Left: Integer read FLeft write FLeft;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LinksControl'}{$ENDIF}
    ///<summary>
    ///  The enabled/disabled state of the links control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LinksControl: Boolean read GetLinksControl write SetLinksControl;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PanControl'}{$ENDIF}
    ///<summary>
    ///  The enabled/disabled state of the pan control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PanControl: Boolean read GetPanControl write SetPanControl;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PanControlOptions'}{$ENDIF}
    ///<summary>
    ///  The display options for the pan control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PanControlOptions: TPanControlOptions read GetPanControlOptions write SetPanControlOptions;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pano'}{$ENDIF}
    ///<summary>
    ///  The panorama ID, which should be set when specifying a custom panorama.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Pano: String read GetPano write SetPano;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PanoProvider'}{$ENDIF}
    ///<summary>
    ///  Custom panorama provider, which takes a string pano id and returns an
    ///  object defining the panorama given that id. This function must be
    ///  defined to specify custom panorama imagery.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PanoProvider: TPanoProvider read FPanoProvider write FPanoProvider;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PanoProvider'}{$ENDIF}
    ///<summary>
    ///  Custom panorama provider, which takes a string pano id and returns an
    ///  object defining the panorama given that id. This function must be
    ///  defined to specify custom panorama imagery.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    property PanoProviderA: TPanoProviderA read FPanoProviderA write FPanoProviderA;
    {$ENDIF}
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  The LatLng position of the Street View panorama.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: TLatLng read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pov'}{$ENDIF}
    ///<summary>
    ///  The camera orientation, specified as heading, pitch, and zoom, for the
    ///  panorama.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Pov: TStreetViewPov read GetPov write SetPov;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ScrollWheel'}{$ENDIF}
    ///<summary>
    ///  If false, disables scrollwheel zooming in Street View. The scrollwheel
    ///  is enabled by default.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ScrollWheel: Boolean read GetScrollWheel write SetScrollWheel;
    property Top: Integer read FTop write FTop;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Visible'}{$ENDIF}
    ///<summary>
    ///  If true, the Street View panorama is visible on load.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Visible: Boolean read GetVisible write SetVisible;
    property Width: Integer read FWidth write FWidth;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZoomControl'}{$ENDIF}
    ///<summary>
    ///  The enabled/disabled state of the zoom control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZoomControl: Boolean read GetZoomControl write SetZoomControl;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZoomControlOptions'}{$ENDIF}
    ///<summary>
    ///  The display options for the zoom control.
    ///</summary>
    ///<seealso cref="gmStreetview|TStreetViewPanoramaOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZoomControlOptions: TZoomControlOptions read GetZoomControlOptions write SetZoomControlOptions;
  end;

  //A point of view object which specifies the camera's orientation at the
  //Street View panorama's position. The point of view is defined as heading,
  //pitch and zoom.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewPov
  TStreetViewPov = class(TCustomJScriptObject)
  private
    procedure SetHeading(Value: Double);
    procedure SetPitch(Value: Double);
    procedure SetZoom(Value: Integer);
    function GetHeading: Double;
    function GetPitch: Double;
    function GetZoom: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //The camera heading in degrees relative to true north. True north is 0,
    //east is 90, south is 180, west is 270.
    property Heading: Double read GetHeading write SetHeading;
    //The camera pitch in degrees, relative to the street view vehicle.
    //Ranges from 90 (directly upwards) to -90 (directly downwards).
    property Pitch: Double read GetPitch write SetPitch;
    //The zoom level. Fully zoomed-out is level 0, zooming in increases the
    //zoom level.
    property Zoom: Integer read GetZoom write SetZoom;
  end;

  TStreetViewServiceCallback = procedure(Sender: TObject;
                                         StreetViewPanoramaData: TStreetViewPanoramaData;
                                         Status: String) of object;
  {$IFDEF USE_ANONYMOUS_METHODS}
  TStreetViewServiceCallbackA = reference to procedure(Sender: TObject;
                                                       StreetViewPanoramaData: TStreetViewPanoramaData;
                                                       Status: String);
  {$ENDIF}

  //A StreetViewService object performs searches for Street View data.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewService
  TStreetViewService = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    //Retrieves the data for the given pano id and passes it to the provided
    //callback as a StreetViewPanoramaData object. Pano ids are unique per
    //panorama and stable for the lifetime of a session, but are liable to
    //change between sessions.
    procedure GetPanoramaByID(Pano: String; Callback: TStreetViewServiceCallback); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetPanoramaByID(Pano: String; Callback: TStreetViewServiceCallbackA); overload;
    {$ENDIF}
    //Retrieves the StreetViewPanoramaData for a panorama within a given
    //radius of the given LatLng. The StreetViewPanoramaData is passed to
    //the provided callback. If the radius is less than 50 meters, the
    //nearest panorama will be returned.
    procedure GetPanoramaByLocation(Latlng: TLatLng; Radius: Integer; Callback: TStreetViewServiceCallback); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetPanoramaByLocation(Latlng: TLatLng; Radius: Integer; Callback: TStreetViewServiceCallbackA); overload;
    {$ENDIF}
  end;

  //The status returned by the StreetViewService on completion of a Street
  //View request.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewStatus
  TStreetViewStatus = class(TCustomJScriptObject)
  private
    procedure SetOK(Value: String);
    procedure SetUnknownError(Value: String);
    procedure SetZeroResults(Value: String);
    function GetOK: String;
    function GetUnknownError: String;
    function GetZeroResults: String;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //The request was successful.
    property OK: String read GetOK write SetOK;
    //The request could not be successfully processed, yet the exact reason
    //for failure is unknown.
    property UnknownError: String read GetUnknownError write SetUnknownError;
    //There are no nearby panoramas.
    property ZeroResults: String read GetZeroResults write SetZeroResults;
  end;

  TGetStreetViewTileUrlMethod = function(Sender: TObject; Pano: String; TileZoom, TileX, TileY: Integer): String of object;
  {$IFDEF USE_ANONYMOUS_METHODS}
  TGetStreetViewTileUrlMethodA = reference to function(Sender: TObject; Pano: String; TileZoom, TileX, TileY: Integer): String;
  {$ENDIF}
  
  //The properties of the tile set used in a Street View panorama.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewTileData
  TStreetViewTileData = class(TCustomJScriptObject)
  private
    FGetTileUrl: TGetStreetViewTileUrlMethod;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FGetTileUrlA: TGetStreetViewTileUrlMethodA;
    {$ENDIF}
    FGetTileUrlFunction: TJScriptFunction;
    FTileSize: TSize;
    FWorldSize: TSize;
    procedure SetCenterHeading(Value: Integer);
    procedure SetGetTileUrl(Value: TGetStreetViewTileUrlMethod);
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure SetGetTileUrlA(Value: TGetStreetViewTileUrlMethodA);
    {$ENDIF}
    procedure SetTileSize(Value: TSize);
    procedure SetWorldSize(Value: TSize);
    function GetCenterHeading: Integer;
    function GetTileSize: TSize;
    function GetWorldSize: TSize;
  protected
    function GetTileUrlCallback: Variant;
  public
    destructor Destroy; override;
    //Gets the tile image URL for the specified tile.
    //Pano is the panorama ID of the Street View tile.
    //TileZoom is the zoom level of the tile.
    //TileX is the x-coordinate of the tile.
    //TileY is the y-coordinate of the tile.
    //Returns the URL for the tile image.
    //function GetTileUrl(Pano: String; TileZoom, TileX, TileY: Integer): String;
    property GetTileUrl: TGetStreetViewTileUrlMethod read FGetTileUrl write SetGetTileUrl;
    {$IFDEF USE_ANONYMOUS_METHODS}
    property GetTileUrlA: TGetStreetViewTileUrlMethodA read FGetTileUrlA write SetGetTileUrlA;
    {$ENDIF}
    //The heading (in degrees) at the center of the panoramic tiles.
    property CenterHeading: Integer read GetCenterHeading write SetCenterHeading;
    //The size (in pixels) at which tiles will be rendered. This may not be
    //the native tile image size.
    property TileSize: TSize read GetTileSize write SetTileSize;
    //The size (in pixels) of the whole panorama's "world".
    property WorldSize: TSize read GetWorldSize write SetWorldSize;
   end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TStreetViewAddressControlOptions

function TStreetViewAddressControlOptions.GetPosition: Integer;
var
  Default: Integer;
begin
  try
    CheckMaps(Script);
    Default:=TScript(Script).Google.Maps.ControlPosition.TopLeft;
  except
    Default:=0;
  end;
  Result:=GetPropAsInteger(StreetViewAddressControlOptionsProp_Position,Default);
end;

procedure TStreetViewAddressControlOptions.SetPosition(Value: Integer);
begin
  SetProp(StreetViewAddressControlOptionsProp_Position,Value,true);
end;

//------------------------------------------------------------------------------
//TStreetViewLinkArray

function TStreetViewLinkArray.Add(StreetViewLink: TStreetViewLink): Integer;
begin
  Result:=inherited Add(StreetViewLink);
end;

function TStreetViewLinkArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TStreetViewLink,Dispatch);
end;

function TStreetViewLinkArray.GetItem(Index: Integer): TStreetViewLink;
begin
  Result:=TStreetViewLink(inherited Items[Index]);
end;

procedure TStreetViewLinkArray.Insert(Index: Integer; StreetViewLink: TStreetViewLink);
begin
  inherited Insert(Index,StreetViewLink);
end;

procedure TStreetViewLinkArray.SetItem(Index: Integer; Value: TStreetViewLink);
begin
  inherited Items[Index]:=Value;
end;

//------------------------------------------------------------------------------
//TStreetViewLink

constructor TStreetViewLink.Create;
begin
  inherited;
end;

constructor TStreetViewLink.Create(Script: TCustomScript);
begin
  inherited;
end;

function TStreetViewLink.GetDescription: String;
begin
  Result:=GetPropAsString(StreetViewLinkProp_Description);
end;

function TStreetViewLink.GetHeading: Double;
begin
  Result:=GetPropAsDouble(StreetViewLinkProp_Heading);
end;

function TStreetViewLink.GetPano: String;
begin
  Result:=GetPropAsString(StreetViewLinkProp_Pano);
end;

function TStreetViewLink.GetRoadColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(StreetViewLinkProp_RoadColor));
end;

function TStreetViewLink.GetRoadOpacity: Double;
begin
  Result:=GetPropAsDouble(StreetViewLinkProp_RoadOpacity);
end;

procedure TStreetViewLink.SetDescription(Value: String);
begin
  SetProp(StreetViewLinkProp_Description,Value,true);
end;

procedure TStreetViewLink.SetHeading(Value: Double);
begin
  SetProp(StreetViewLinkProp_Heading,Value,true);
end;

procedure TStreetViewLink.SetPano(Value: String);
begin
  SetProp(StreetViewLinkProp_Pano,Value,true);
end;

procedure TStreetViewLink.SetRoadColor(Value: TColor);
begin
  SetProp(StreetViewLinkProp_RoadColor,ColorToWebColorStr(Value),true);
end;

procedure TStreetViewLink.SetRoadOpacity(Value: Double);
begin
  SetProp(StreetViewLinkProp_RoadOpacity,Value,true);
end;

//------------------------------------------------------------------------------
//TStreetViewLocation

destructor TStreetViewLocation.Destroy;
begin
  FLatLng.Free;
  inherited;
end;

function TStreetViewLocation.GetDescription: String;
begin
  Result:=GetPropAsString(StreetViewLocationProp_Description);
end;

function TStreetViewLocation.GetLatLng: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(StreetViewLocationProp_LatLng,TLatLng,FLatLng));
end;

function TStreetViewLocation.GetPano: String;
begin
  Result:=GetPropAsString(StreetViewLocationProp_Pano);
end;

procedure TStreetViewLocation.SetDescription(Value: String);
begin
  SetProp(StreetViewLocationProp_Description,Value,true);
end;

procedure TStreetViewLocation.SetLatLng(Value: TLatLng);
begin
  SetProp(StreetViewLocationProp_LatLng,Value,true);
end;

procedure TStreetViewLocation.SetPano(Value: String);
begin
  SetProp(StreetViewLocationProp_Pano,Value,true);
end;

//------------------------------------------------------------------------------
//TStreetViewPanorama

constructor TStreetViewPanorama.Create(Opts: TStreetViewPanoramaOptions);
begin
  Create(JScriptObjects.Script,nil,Opts);
end;

constructor TStreetViewPanorama.Create(Container: TNode;
                                       Opts: TStreetViewPanoramaOptions);
begin
  Create(JScriptObjects.Script,Container,Opts);
end;

constructor TStreetViewPanorama.Create(Script: TCustomScript;
                                       Opts: TStreetViewPanoramaOptions);
begin
  Create(Script,nil,Opts);
end;

constructor TStreetViewPanorama.Create(Script: TCustomScript; Container: TNode;
                                       Opts: TStreetViewPanoramaOptions);
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
  if not assigned(Container) then
  begin
    Container:=Script.Document.CreateDivNode;
    Script.Document.Body.AppendChild(Container);
    FContainer:=Container;
  end else FContainer:=TNode(CreateObjectWrapper(Script,TNode,Container.Disp));
  if not assigned(Container)
    then raise Exception.Create(ExceptionNoDiv);
  if assigned(Opts) then
  begin
    with Container do
    begin
      Style.Position:='absolute';
      Left:=Opts.Left;
      Top:=Opts.Top;
      Width:=Opts.Width;
      Height:=Opts.Height;
      Align:=Opts.Align;
    end;
    UpdateOptions(Script,Opts);
  end;
  inherited Create(Script,New(Script,Container,Opts));
end;

destructor TStreetViewPanorama.Destroy;
begin
  SetOnCloseClick(nil);
  SetOnLinksChanged(nil);
  SetOnPanoChanged(nil);
  SetOnPositionChanged(nil);
  SetOnPovChanged(nil);
  SetOnResize(nil);
  SetOnVisibleChanged(nil);
  FPanoProviderFunction.Free;
  FOnCloseClickFunction.Free;
  FOnLinksChangedFunction.Free;
  FOnPanoChangedFunction.Free;
  FOnPositionChangedFunction.Free;
  FOnPovChangedFunction.Free;
  FOnResizeFunction.Free;
  FOnVisibleChangedFunction.Free;
  FContainer.Free;
  FControls.Free;
  FLatLng.Free;
  FLinks.Free;
  FPosition.Free;
  FPov.Free;
  inherited;
end;

class function TStreetViewPanorama.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[StreetViewPanoramaMethod_GetEnableCloseButton,
                                 StreetViewPanoramaMethod_SetEnableCloseButton,
                                 StreetViewPanoramaMethod_GetPano]);
end;

class function TStreetViewPanorama.New(Script: TCustomScript; Node: TNode): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Node)
    then raise Exception.Create(ExceptionNoNode);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_StreetViewPanorama,Node.Disp,true);
end;

class function TStreetViewPanorama.New(Script: TCustomScript; Node: TNode;
                                       Opts: TStreetViewPanoramaOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Node)
    then raise Exception.Create(ExceptionNoNode);
  if not assigned(Opts)
    then Result:=New(Script,Node)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_StreetViewPanorama,
                                     {$IFDEF CAN_OVERLOAD}
                                     [Node.Disp,Opts.Disp]
                                     {$ELSE}
                                     Node.Disp,Opts.Disp
                                     {$ENDIF}
                                     ,true);
end;

procedure TStreetViewPanorama.UpdateOptions(Script: TCustomScript; Opts: TStreetViewPanoramaOptions);
begin
  if assigned(Script) and assigned(Opts) then
  begin
    FPanoProvider:=Opts.PanoProvider;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FPanoProviderA:=Opts.PanoProviderA;
    {$ENDIF}
    if assigned(FPanoProvider)
       {$IFDEF USE_ANONYMOUS_METHODS}or assigned(FPanoProviderA){$ENDIF} then
    begin
      if not assigned(FPanoProviderFunction)
        then FPanoProviderFunction:=Script.NewFunction(PanoProviderCallback);
      Opts.SetProp(StreetViewPanoramaOptionsProp_PanoProvider,FPanoProviderFunction,true);
    end;
  end;
end;

function TStreetViewPanorama.PanoProviderCallback: Variant;
var
  StreetViewPanoramaData: TStreetViewPanoramaData;
begin
  Result:=Null;
  if FPanoProviderFunction.Arguments.Count>=1 then
  begin
    {$IFDEF USE_ANONYMOUS_METHODS}
    if assigned(FPanoProviderA)
      then StreetViewPanoramaData:=FPanoProviderA(Self,FPanoProviderFunction.Arguments[0])
      else
    {$ENDIF}
    StreetViewPanoramaData:=FPanoProvider(Self,FPanoProviderFunction.Arguments[0]);
    if assigned(StreetViewPanoramaData)
      then Result:=StreetViewPanoramaData.Disp;
  end;
end;

procedure TStreetViewPanorama.CloseClick;
begin
  CallEvent(FOnCloseClick,FOnCloseClickFunction.Arguments[0]);
end;

procedure TStreetViewPanorama.LinksChanged;
begin
  CallEvent(FOnLinksChanged);
end;

procedure TStreetViewPanorama.PanoChanged;
begin
  CallEvent(FOnPanoChanged);
end;

procedure TStreetViewPanorama.PositionChanged;
begin
  CallEvent(FOnPositionChanged);
end;

procedure TStreetViewPanorama.PovChanged;
begin
  CallEvent(FOnPovChanged);
end;

procedure TStreetViewPanorama.Resize;
begin
  CallEvent(FOnResize);
end;

procedure TStreetViewPanorama.VisibleChanged;
begin
  CallEvent(FOnVisibleChanged);
end;

function TStreetViewPanorama.GetAlign: TAlign;
begin
  Result:=FContainer.Align;
end;

procedure TStreetViewPanorama.SetAlign(Value: TAlign);
begin
  FContainer.Align:=Value;
end;

function TStreetViewPanorama.GetControls: TNodeMVCArrayArray;
begin
  Result:=TNodeMVCArrayArray(GetPropAsObject(StreetViewPanoramaProp_Controls,
                                             TNodeMVCArrayArray,FControls));
end;

procedure TStreetViewPanorama.SetControls(Value: TNodeMVCArrayArray);
begin
  SetProp(StreetViewPanoramaProp_Controls,Value);
end;

function TStreetViewPanorama.GetEnableCloseButton: Boolean;
begin
  Result:=VarAsBoolean(Exec(StreetViewPanoramaMethod_GetEnableCloseButton));
end;

procedure TStreetViewPanorama.SetEnableCloseButton(Flag: Boolean);
begin
  Exec(StreetViewPanoramaMethod_SetEnableCloseButton,[Flag]);
end;

function TStreetViewPanorama.GetHeight: Integer;
begin
  Result:=FContainer.Height;
end;

procedure TStreetViewPanorama.SetHeight(Value: Integer);
begin
  FContainer.Height:=Value;
end;

function TStreetViewPanorama.GetLeft: Integer;
begin
  Result:=FContainer.Left;
end;

procedure TStreetViewPanorama.SetLeft(Value: Integer);
begin
  FContainer.Left:=Value;
end;

procedure TStreetViewPanorama.SetOnCloseClick(Value: TMapEventEvent);
begin
  FOnCloseClick:=Value;
  RemoveListener(FOnCloseClickListener);
  if assigned(FOnCloseClick)
    then AddListener(StreetViewPanoramaEvent_CloseClick,FOnCloseClickListener,
                     FOnCloseClickFunction,CloseClick);
end;

procedure TStreetViewPanorama.SetOnLinksChanged(Value: TNotifyEvent);
begin
  FOnLinksChanged:=Value;
  RemoveListener(FOnLinksChangedListener);
  if assigned(FOnLinksChanged)
    then AddListener(StreetViewPanoramaEvent_LinksChanged,FOnLinksChangedListener,
                     FOnLinksChangedFunction,LinksChanged);
end;

procedure TStreetViewPanorama.SetOnPanoChanged(Value: TNotifyEvent);
begin
  FOnPanoChanged:=Value;
  RemoveListener(FOnPanoChangedListener);
  if assigned(FOnPanoChanged)
    then AddListener(StreetViewPanoramaEvent_PanoChanged,FOnPanoChangedListener,
                     FOnPanoChangedFunction,PanoChanged);
end;

procedure TStreetViewPanorama.SetOnPositionChanged(Value: TNotifyEvent);
begin
  FOnPositionChanged:=Value;
  RemoveListener(FOnPositionChangedListener);
  if assigned(FOnPositionChanged)
    then AddListener(StreetViewPanoramaEvent_PositionChanged,
                     FOnPositionChangedListener,
                     FOnPositionChangedFunction,PositionChanged);
end;

procedure TStreetViewPanorama.SetOnPovChanged(Value: TNotifyEvent);
begin
  FOnPovChanged:=Value;
  RemoveListener(FOnPovChangedListener);
  if assigned(FOnPovChanged)
    then AddListener(StreetViewPanoramaEvent_PovChanged,FOnPovChangedListener,
                     FOnPovChangedFunction,PovChanged);
end;

procedure TStreetViewPanorama.SetOnResize(Value: TNotifyEvent);
begin
  FOnResize:=Value;
  RemoveListener(FOnResizeListener);
  if assigned(FOnResize)
    then AddListener(StreetViewPanoramaEvent_Resize,FOnResizeListener,
                     FOnResizeFunction,Resize);
end;

procedure TStreetViewPanorama.SetOnVisibleChanged(Value: TNotifyEvent);
begin
  FOnVisibleChanged:=Value;
  RemoveListener(FOnVisibleChangedListener);
  if assigned(FOnVisibleChanged)
    then AddListener(StreetViewPanoramaEvent_VisibleChanged,
                     FOnVisibleChangedListener,
                     FOnVisibleChangedFunction,VisibleChanged);
end;

procedure TStreetViewPanorama.SetOptions(Options: TStreetViewPanoramaOptions);
begin
  UpdateOptions(Script,Options);
  inherited SetOptions(Options);
end;

function TStreetViewPanorama.GetPano: String;
begin
  Result:=VarAsString(Exec(StreetViewPanoramaMethod_GetPano));
end;

procedure TStreetViewPanorama.SetPano(Pano: String);
begin
  Exec(StreetViewPanoramaMethod_SetPano,[Pano]);
end;

function TStreetViewPanorama.GetPosition: TLatLng;
begin
  Result:=TLatLng(GetObjectWithExec(StreetViewPanoramaMethod_GetPosition,
                                    TLatLng,FPosition));
end;

procedure TStreetViewPanorama.SetPosition(LatLng: TLatLng);
begin
  Exec(StreetViewPanoramaMethod_SetPosition,LatLng);
end;

function TStreetViewPanorama.GetPov: TStreetViewPov;
begin
  Result:=TStreetViewPov(GetObjectWithExec(StreetViewPanoramaMethod_GetPov,
                                           TStreetViewPov,FPov));
end;

procedure TStreetViewPanorama.SetPov(Pov: TStreetViewPov);
begin
  Exec(StreetViewPanoramaMethod_SetPov,Pov);
end;

function TStreetViewPanorama.GetTop: Integer;
begin
  Result:=FContainer.Top;
end;

procedure TStreetViewPanorama.SetTop(Value: Integer);
begin
  FContainer.Top:=Value;
end;

function TStreetViewPanorama.GetVisible: Boolean;
begin
  Result:=VarAsBoolean(Exec(StreetViewPanoramaMethod_GetVisible));
end;

procedure TStreetViewPanorama.SetVisible(Flag: Boolean);
begin
  Exec(StreetViewPanoramaMethod_SetVisible,[Flag]);
end;

function TStreetViewPanorama.GetWidth: Integer;
begin
  Result:=FContainer.Width;
end;

procedure TStreetViewPanorama.SetWidth(Value: Integer);
begin
  FContainer.Width:=Value;
end;

function TStreetViewPanorama.GetLinks: TStreetViewLinkArray;
begin
  Result:=TStreetViewLinkArray(GetObjectWithExec(StreetViewPanoramaMethod_GetLinks,
                                                 TStreetViewLinkArray,FLinks));
end;

function TStreetViewPanorama.N: OleVariant;
begin
  Result:=Exec(StreetViewPanoramaMethod_N);
end;

procedure TStreetViewPanorama.Visible_changed;
begin
  Exec(StreetViewPanoramaMethod_VisibleChanged);
end;

procedure TStreetViewPanorama.RegisterPanoProvider(PanoProvider: TPanoProvider);
begin
  FPanoProvider:=PanoProvider;
  if assigned(FPanoProvider) and not assigned(FPanoProviderFunction)
    then FPanoProviderFunction:=Script.NewFunction(PanoProviderCallback);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TStreetViewPanorama.RegisterPanoProvider(PanoProvider: TPanoProviderA);
begin
  FPanoProviderA:=PanoProvider;
  if assigned(FPanoProviderA) and not assigned(FPanoProviderFunction)
    then FPanoProviderFunction:=Script.NewFunction(PanoProviderCallback);
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TStreetViewPanoramaData

constructor TStreetViewPanoramaData.Create;
begin
  inherited;
end;

constructor TStreetViewPanoramaData.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TStreetViewPanoramaData.Destroy;
begin
  FLinks.Free;
  FLocation.Free;
  FTiles.Free;
  inherited;
end;

function TStreetViewPanoramaData.GetCopyright: String;
begin
  Result:=GetPropAsString(StreetViewPanoramaDataProp_Copyright);
end;

function TStreetViewPanoramaData.GetLinks: TStreetViewLinkArray;
begin
  Result:=TStreetViewLinkArray(GetPropAsObject(StreetViewPanoramaDataProp_Links,
                                               TStreetViewLinkArray,
                                               FLinks,true));
end;

function TStreetViewPanoramaData.GetLocation: TStreetViewLocation;
begin
  Result:=TStreetViewLocation(GetPropAsObject(StreetViewPanoramaDataProp_Location,
                                              TStreetViewLocation,
                                              FLocation,true));
end;

function TStreetViewPanoramaData.GetTiles: TStreetViewTileData;
begin
  Result:=TStreetViewTileData(GetPropAsObject(StreetViewPanoramaDataProp_Tiles,
                                              TStreetViewTileData,
                                              FTiles,true));
end;

procedure TStreetViewPanoramaData.SetCopyright(Value: String);
begin
  SetProp(StreetViewPanoramaDataProp_Copyright,Value,true);
end;

procedure TStreetViewPanoramaData.SetLinks(Value: TStreetViewLinkArray);
begin
  SetProp(StreetViewPanoramaDataProp_Links,Value,true);
end;

procedure TStreetViewPanoramaData.SetLocation(Value: TStreetViewLocation);
begin
  SetProp(StreetViewPanoramaDataProp_Location,Value,true);
end;

procedure TStreetViewPanoramaData.SetTiles(Value: TStreetViewTileData);
begin
  SetProp(StreetViewPanoramaDataProp_Tiles,Value,true);
end;

//------------------------------------------------------------------------------
//TStreetViewPanoramaOptions

constructor TStreetViewPanoramaOptions.Create;
begin
  inherited;
end;

constructor TStreetViewPanoramaOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TStreetViewPanoramaOptions.Destroy;
begin
  FAddressControlOptions.Free;
  FPanControlOptions.Free;
  FPosition.Free;
  FPov.Free;
  FZoomControlOptions.Free;
  inherited;
end;

procedure TStreetViewPanoramaOptions.AfterConstruction;
begin
  inherited;
  FWidth:=200;
  FHeight:=150;
  FAlign:=alClient;
end;

class function TStreetViewPanoramaOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=false; //(GetJSObjectType(Disp)=otObject) and ObjectHasMembers(Disp,[,,]);
end;

function TStreetViewPanoramaOptions.GetAddressControl: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_AddressControl,true);
end;

function TStreetViewPanoramaOptions.GetAddressControlOptions: TStreetViewAddressControlOptions;
begin
  Result:=TStreetViewAddressControlOptions(GetPropAsObject(StreetViewPanoramaOptionsProp_AddressControlOptions,
                                                           TStreetViewAddressControlOptions,
                                                           FAddressControlOptions,true));
end;

function TStreetViewPanoramaOptions.GetDisableDoubleClickZoom: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_DisableDoubleClickZoom);
end;

function TStreetViewPanoramaOptions.GetEnableCloseButton: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_EnableCloseButton,true);
end;

function TStreetViewPanoramaOptions.GetLinksControl: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_LinksControl,true);
end;

function TStreetViewPanoramaOptions.GetPanControl: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_PanControl,true);
end;

function TStreetViewPanoramaOptions.GetPanControlOptions: TPanControlOptions;
begin
  Result:=TPanControlOptions(GetPropAsObject(StreetViewPanoramaOptionsProp_PanControlOptions,
                                             TPanControlOptions,FPanControlOptions,true));
end;

function TStreetViewPanoramaOptions.GetPano: String;
begin
  Result:=GetPropAsString(StreetViewPanoramaOptionsProp_Pano);
end;

function TStreetViewPanoramaOptions.GetPosition: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(StreetViewPanoramaOptionsProp_Position,
                                  TLatLng,FPosition,true));
end;

function TStreetViewPanoramaOptions.GetPov: TStreetViewPov;
begin
  Result:=TStreetViewPov(GetPropAsObject(StreetViewPanoramaOptionsProp_Pov,
                                         TStreetViewPov,FPov,true));
end;

function TStreetViewPanoramaOptions.GetScrollWheel: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_ScrollWheel,true);
end;

function TStreetViewPanoramaOptions.GetVisible: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_Visible,true);
end;

function TStreetViewPanoramaOptions.GetZoomControl: Boolean;
begin
  Result:=GetPropAsBoolean(StreetViewPanoramaOptionsProp_ZoomControl,true);
end;

function TStreetViewPanoramaOptions.GetZoomControlOptions: TZoomControlOptions;
begin
  Result:=TZoomControlOptions(GetPropAsObject(StreetViewPanoramaOptionsProp_ZoomControlOptions,
                                              TZoomControlOptions,FZoomControlOptions,true));
end;

procedure TStreetViewPanoramaOptions.SetAddressControl(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_AddressControl,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetAddressControlOptions(Value: TStreetViewAddressControlOptions);
begin
  SetProp(StreetViewPanoramaOptionsProp_AddressControlOptions,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetDisableDoubleClickZoom(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_DisableDoubleClickZoom,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetEnableCloseButton(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_EnableCloseButton,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetLinksControl(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_LinksControl,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetPanControl(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_PanControl,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetPanControlOptions(Value: TPanControlOptions);
begin
  SetProp(StreetViewPanoramaOptionsProp_PanControlOptions,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetPano(Value: String);
begin
  SetProp(StreetViewPanoramaOptionsProp_Pano,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetPosition(Value: TLatLng);
begin
  SetProp(StreetViewPanoramaOptionsProp_Position,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetPov(Value: TStreetViewPov);
begin
  SetProp(StreetViewPanoramaOptionsProp_Pov,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetScrollWheel(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_ScrollWheel,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetVisible(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_Visible,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetZoomControl(Value: Boolean);
begin
  SetProp(StreetViewPanoramaOptionsProp_ZoomControl,Value,true);
end;

procedure TStreetViewPanoramaOptions.SetZoomControlOptions(Value: TZoomControlOptions);
begin
  SetProp(StreetViewPanoramaOptionsProp_ZoomControlOptions,Value,true);
end;

//------------------------------------------------------------------------------
//TStreetViewPov

constructor TStreetViewPov.Create;
begin
  inherited;
end;

constructor TStreetViewPov.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TStreetViewPov.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[StreetViewPovProp_Heading,
                                 StreetViewPovProp_Pitch,
                                 StreetViewPovProp_Zoom]);
end;

function TStreetViewPov.GetHeading: Double;
begin
  Result:=GetPropAsDouble(StreetViewPovProp_Heading);
end;

procedure TStreetViewPov.SetHeading(Value: Double);
begin
  SetProp(StreetViewPovProp_Heading,Value,true);
end;

function TStreetViewPov.GetPitch: Double;
begin
  Result:=GetPropAsDouble(StreetViewPovProp_Pitch);
end;

procedure TStreetViewPov.SetPitch(Value: Double);
begin
  SetProp(StreetViewPovProp_Pitch,Value,true);
end;

function TStreetViewPov.GetZoom: Integer;
begin
  Result:=GetPropAsInteger(StreetViewPovProp_Zoom);
end;

procedure TStreetViewPov.SetZoom(Value: Integer);
begin
  SetProp(StreetViewPovProp_Zoom,Value,true);
end;

//------------------------------------------------------------------------------
//TStreetViewService

type
  TStreetViewServiceCallbackObject = class(TCallbackObject)
  private
    FCallback: TStreetViewServiceCallback;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FCallbackA: TStreetViewServiceCallbackA;
    {$ENDIF}
    FStreetViewService: TStreetViewService;
  protected
    procedure Callback; override;
  public
    constructor Create(StreetViewService: TStreetViewService; Pano: String;
                       Callback: TStreetViewServiceCallback); reintroduce; overload;
    constructor Create(StreetViewService: TStreetViewService; LatLng: TLatLng;
                       Radius: Integer; Callback: TStreetViewServiceCallback); reintroduce; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    constructor Create(StreetViewService: TStreetViewService; Pano: String;
                       Callback: TStreetViewServiceCallbackA); reintroduce; overload;
    constructor Create(StreetViewService: TStreetViewService; LatLng: TLatLng;
                       Radius: Integer; Callback: TStreetViewServiceCallbackA); reintroduce; overload;
    {$ENDIF}
  end;

const
  StreetViewServiceMethod_GetPanoramaByID       = 'getPanoramaById';
  StreetViewServiceMethod_GetPanoramaByLocation = 'getPanoramaByLocation';

constructor TStreetViewServiceCallbackObject.Create(StreetViewService: TStreetViewService;
                                                    Pano: String;
                                                    Callback: TStreetViewServiceCallback);
begin
  inherited Create(StreetViewService.Script);
  FCallback:=Callback;
  FStreetViewService:=StreetViewService;
  FStreetViewService.Exec(StreetViewServiceMethod_GetPanoramaByID,[Pano,JScriptFunction.Disp]);
end;

constructor TStreetViewServiceCallbackObject.Create(StreetViewService: TStreetViewService;
                                                    LatLng: TLatLng; Radius: Integer;
                                                    Callback: TStreetViewServiceCallback);
begin
  inherited Create(StreetViewService.Script);
  FCallback:=Callback;
  FStreetViewService:=StreetViewService;
  FStreetViewService.Exec(StreetViewServiceMethod_GetPanoramaByLocation,
                          [LatLng.Disp,Radius,JScriptFunction.Disp]);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
constructor TStreetViewServiceCallbackObject.Create(StreetViewService: TStreetViewService;
                                                    Pano: String;
                                                    Callback: TStreetViewServiceCallbackA);
begin
  inherited Create(StreetViewService.Script);
  FCallbackA:=Callback;
  FStreetViewService:=StreetViewService;
  FStreetViewService.Exec(StreetViewServiceMethod_GetPanoramaByID,[Pano,JScriptFunction.Disp]);
end;

constructor TStreetViewServiceCallbackObject.Create(StreetViewService: TStreetViewService;
                                                    LatLng: TLatLng;
                                                    Radius: Integer;
                                                    Callback: TStreetViewServiceCallbackA);
begin
  inherited Create(StreetViewService.Script);
  FCallbackA:=Callback;
  FStreetViewService:=StreetViewService;
  FStreetViewService.Exec(StreetViewServiceMethod_GetPanoramaByLocation,
                          [LatLng.Disp,Radius,JScriptFunction.Disp]);
end;
{$ENDIF}

procedure TStreetViewServiceCallbackObject.Callback;
var
  StreetViewPanoramaData: TStreetViewPanoramaData;
begin
  if assigned(FCallback)
     {$IFDEF USE_ANONYMOUS_METHODS}or assigned(FCallbackA){$ENDIF} then
  begin
    try
      if ArgumentCount>=2 then
      begin
        StreetViewPanoramaData:=nil;
        if VarIsValidDispatch(Arguments[0])
          then StreetViewPanoramaData:=TStreetViewPanoramaData
                                         (CreateObjectWrapper(Script,
                                          TStreetViewPanoramaData,
                                          Arguments[0]));
        try
          {$IFDEF USE_ANONYMOUS_METHODS}
          if assigned(FCallbackA)
            then FCallbackA(FStreetViewService,StreetViewPanoramaData,Arguments[1])
            else
          {$ENDIF}
          if assigned(FCallback)
            then FCallback(FStreetViewService,StreetViewPanoramaData,Arguments[1]);
        finally
          //StreetViewPanoramaData.Free;    -> to protect them for future use in anonymous methods
        end;
      end;
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
  Ready:=true;
end;

constructor TStreetViewService.Create;
begin
  inherited;
end;

constructor TStreetViewService.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TStreetViewService.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_StreetViewService,true);
end;

procedure TStreetViewService.GetPanoramaByID(Pano: String;
                                             Callback: TStreetViewServiceCallback);
begin
  TStreetViewServiceCallbackObject.Create(Self,Pano,Callback);
end;

procedure TStreetViewService.GetPanoramaByLocation(Latlng: TLatLng;
                                                   Radius: Integer;
                                                   Callback: TStreetViewServiceCallback);
begin
  TStreetViewServiceCallbackObject.Create(Self,LatLng,Radius,Callback);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TStreetViewService.GetPanoramaByID(Pano: String;
                                             Callback: TStreetViewServiceCallbackA);
begin
  TStreetViewServiceCallbackObject.Create(Self,Pano,Callback);
end;

procedure TStreetViewService.GetPanoramaByLocation(Latlng: TLatLng; Radius: Integer;
                                                   Callback: TStreetViewServiceCallbackA);
begin
  TStreetViewServiceCallbackObject.Create(Self,LatLng,Radius,Callback);
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TStreetViewStatus

class function TStreetViewStatus.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[StreetViewStatusProp_Ok,
                                 StreetViewStatusProp_UnknownError,
                                 StreetViewStatusProp_ZeroResults]);
end;

function TStreetViewStatus.GetOK: String;
begin
  Result:=GetPropAsString(StreetViewStatusProp_Ok);
end;

procedure TStreetViewStatus.SetOK(Value: String);
begin
  SetProp(StreetViewStatusProp_OK,Value);
end;

function TStreetViewStatus.GetUnknownError: String;
begin
  Result:=GetPropAsString(StreetViewStatusProp_UnknownError);
end;

procedure TStreetViewStatus.SetUnknownError(Value: String);
begin
  SetProp(StreetViewStatusProp_UnknownError,Value);
end;

function TStreetViewStatus.GetZeroResults: String;
begin
  Result:=GetPropAsString(StreetViewStatusProp_ZeroResults);
end;

procedure TStreetViewStatus.SetZeroResults(Value: String);
begin
  SetProp(StreetViewStatusProp_ZeroResults,Value);
end;

//------------------------------------------------------------------------------
//TStreetViewTileData

destructor TStreetViewTileData.Destroy;
begin
  FGetTileUrlFunction.Free;
  FTileSize.Free;
  FWorldSize.Free;
  inherited;
end;

function TStreetViewTileData.GetCenterHeading: Integer;
begin
  Result:=GetPropAsInteger(StreetViewTileDataProp_CenterHeading);
end;

function TStreetViewTileData.GetTileSize: TSize;
begin
  Result:=TSize(GetPropAsObject(StreetViewTileDataProp_TileSize,TSize,FTileSize));
end;

function TStreetViewTileData.GetTileUrlCallback: Variant;
begin
  Result:=Null;
  try
    if assigned(FGetTileUrl) and (FGetTileUrlFunction.Arguments.Count>=4)
      then Result:=FGetTileUrl(Self,
                               FGetTileUrlFunction.Arguments[0],     //Pano
                               FGetTileUrlFunction.Arguments[1],     //TileZoom
                               FGetTileUrlFunction.Arguments[2],     //TileX
                               FGetTileUrlFunction.Arguments[3]);    //TileY
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

function TStreetViewTileData.GetWorldSize: TSize;
begin
  Result:=TSize(GetPropAsObject(StreetViewTileDataProp_WorldSize,TSize,FWorldSize));
end;

procedure TStreetViewTileData.SetCenterHeading(Value: Integer);
begin
  SetProp(StreetViewTileDataProp_CenterHeading,Value,true);
end;

procedure TStreetViewTileData.SetGetTileUrl(Value: TGetStreetViewTileUrlMethod);
begin
  FGetTileUrl:=Value;
  if assigned(FGetTileUrl) and not assigned(FGetTileUrlFunction) then
  begin
    FGetTileUrlFunction:=Script.NewFunction(GetTileUrlCallback);
    SetProp(StreetViewTileDataMethod_GetTileUrl,FGetTileUrlFunction,true);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TStreetViewTileData.SetGetTileUrlA(Value: TGetStreetViewTileUrlMethodA);
begin
  FGetTileUrlA:=Value;
  if assigned(FGetTileUrlA) and not assigned(FGetTileUrlFunction) then
  begin
    FGetTileUrlFunction:=Script.NewFunction(GetTileUrlCallback);
    SetProp(StreetViewTileDataMethod_GetTileUrl,FGetTileUrlFunction,true);
  end;
end;
{$ENDIF}

procedure TStreetViewTileData.SetTileSize(Value: TSize);
begin
  SetProp(StreetViewTileDataProp_TileSize,Value,true);
end;

procedure TStreetViewTileData.SetWorldSize(Value: TSize);
begin
  SetProp(StreetViewTileDataProp_WorldSize,Value,true);
end;

end.
